/*************************************************************************
 *
 *  $RCSfile: userimpl.hxx,v $
 *
 *  $Revision: 1.1 $
 *
 *  last change: $Author: dg $ $Date: 2000/11/10 22:43:20 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef CONFIGMGR_API_USERIMPL_HXX_
#define CONFIGMGR_API_USERIMPL_HXX_

#ifndef _CPPUHELPER_IMPLBASE2_HXX_
#include <cppuhelper/implbase2.hxx>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XNAMECONTAINER_HPP_
#include <com/sun/star/container/XNameContainer.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XSINGLESERVICEFACTORY_HPP_
#include <com/sun/star/lang/XSingleServiceFactory.hpp>
#endif
#ifndef CONFIGMGR_API_SVCCOMPONENT_HXX_
#include "confsvccomponent.hxx"
#endif

namespace configmgr
{
	namespace css  = ::com::sun::star;
	namespace uno  = css::uno;
	namespace container  = css::container;
	namespace lang = css::lang;		

	class OAdminProviderImpl;
	extern const ServiceInfo aUserAccessSI;
	
	
	//==========================================================================
	//= OUserAccess
	//==========================================================================
	typedef ::cppu::ImplHelper2	<	container::XNameContainer,
									lang::XSingleServiceFactory
								>	OUserAccess_Base;	


	/** class for adding and removing users and to retrieve user profiles
	*/
	class OUserAccess : public ServiceComponentImpl,
						public OUserAccess_Base
	{			
		OAdminProviderImpl*			m_pProviderImpl;	/// access to the configserver

	public:
		/** constructs the object.
			@param		_rSafety			mutex to be used for thread safety
			@param		_rNodeFactory		a pointer to the factory which can be used to create new children. Must not be NULL.
			@param		_rTemplateName		the name of the template to be used to create new children. Must be a valid template name for the node factory.
			@param		_bAllowUnescaped	flag if on inserting new children, unescaped names are allowed
		*/
		OUserAccess(OAdminProviderImpl* _pProviderImpl);			

		// XInterface
		uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);
		virtual void SAL_CALL acquire() throw() { ServiceComponentImpl::acquire(); }
		virtual void SAL_CALL release() throw() { ServiceComponentImpl::release(); }

		// XTypeProvider
		uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);
		uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() throw (uno::RuntimeException);

		// XComponent
		virtual void SAL_CALL		disposing();	

		// XNameContainer
		virtual void SAL_CALL		insertByName( const ::rtl::OUString& _rName, const uno::Any& _rElement ) throw(lang::IllegalArgumentException, container::ElementExistException, lang::WrappedTargetException, uno::RuntimeException);
		virtual void SAL_CALL		removeByName( const ::rtl::OUString& _rName ) throw(container::NoSuchElementException, lang::WrappedTargetException, uno::RuntimeException);

		// XNameReplace
		virtual void SAL_CALL		replaceByName( const ::rtl::OUString& _rName, const uno::Any& _rElement ) throw(lang::IllegalArgumentException, container::NoSuchElementException, lang::WrappedTargetException, uno::RuntimeException);

		// XNameAccess		
		virtual uno::Any SAL_CALL	getByName( const ::rtl::OUString& aName ) throw(container::NoSuchElementException, lang::WrappedTargetException, uno::RuntimeException);
		virtual uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) throw(uno::RuntimeException);
		virtual sal_Bool SAL_CALL	hasByName( const ::rtl::OUString& aName ) throw(uno::RuntimeException);

		// XElementAccess		
		virtual uno::Type SAL_CALL	getElementType(  ) throw(uno::RuntimeException);
		virtual sal_Bool SAL_CALL	hasElements(  ) throw(uno::RuntimeException);					

		// XSingleServiceFactory
		virtual uno::Reference< uno::XInterface > SAL_CALL createInstance(  ) throw(uno::Exception, uno::RuntimeException);
		virtual uno::Reference< uno::XInterface > SAL_CALL createInstanceWithArguments( const uno::Sequence< uno::Any >& aArguments ) throw(uno::Exception, uno::RuntimeException);
	};

} // namespace configmgr

#endif // CONFIGMGR_API_CONFPROVIDER2_HXX_


