/*************************************************************************
 *
 *  $RCSfile: binarybasereader.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: dg $ $Date: 2001/07/06 12:46:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <osl/file.hxx>

#include "binarybasereader.hxx"
#include "oslstream.hxx"

#ifndef _COM_SUN_STAR_IO_XACTIVEDATASINK_HPP_
#include <com/sun/star/io/XActiveDataSink.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XOBJECTINPUTSTREAM_HPP_
#include <com/sun/star/io/XObjectInputStream.hpp>
#endif

#ifndef CONFIGMGR_STREAMCOMPRESS_HXX_
#include "streamcompress.hxx"
#endif

#ifndef _COMPHELPER_SEQSTREAM_HXX
#include <comphelper/seqstream.hxx>
#endif

#ifndef _CONFIGMGR_SERVER_VERSION_HXX_
#include "server_version.hxx"
#endif


#include "tracer.hxx"

// -----------------------------------------------------------------------------
#define ASCII(x) rtl::OUString::createFromAscii(x)
// -----------------------------------------------------------------------------
namespace configmgr
{

// -----------------------------------------------------------------------------
void OBinaryBaseReader::open()
{
	ensure_File();
}

// -----------------------------------------------------------------------------
void OBinaryBaseReader::ensure_File() throw (com::sun::star::io::IOException)
{
	OTrivialBufferedFile* pFile = new OTrivialBufferedFile(m_sFileURL );

	OSL_ENSURE(pFile, "could not get the file handle.");
	osl::FileBase::RC eError = pFile->open(OpenFlag_Read);
	if (eError != osl_File_E_None)
	{
		throw io::IOException(FileHelper::createOSLErrorString(eError), NULL);
	}
	
	// create a inputstream out of the binary file
	uno::Reference<io::XInputStream> xInput = new OSLInputBufferedStreamWrapper(pFile, true);

#ifdef _USE_COMPRESSION
	// decompress the inputstream to the buffer
	::com::sun::star::uno::Sequence< sal_Int8 >	aBuffer(8192);
	uno::Reference< io::XOutputStream > xOutput(new comphelper::OSequenceOutputStream(aBuffer,1.3));

	sal_Int32 nRealSize = OStreamCompressor().decompress(xInput,xOutput);
	aBuffer.realloc(nRealSize);

	// delegate the data to a new InputStream, using the buffer containing the decompressed data
	xInput = new comphelper::SequenceInputStream(aBuffer);	
#endif
	
	uno::Reference< io::XMarkableStream > xMarkableStream(m_xServiceProvider->createInstance(ASCII("com.sun.star.io.MarkableInputStream")), uno::UNO_QUERY);	
	if (xMarkableStream.is())
	{
		uno::Reference< io::XActiveDataSink > xMarkSink(xMarkableStream, uno::UNO_QUERY);
		xMarkSink->setInputStream(xInput);

		uno::Reference< io::XObjectInputStream > xObjectInputStream(m_xServiceProvider->createInstance(ASCII("com.sun.star.io.ObjectInputStream")), uno::UNO_QUERY);
		if (xObjectInputStream.is())
		{
			uno::Reference< io::XActiveDataSink > xDataSink(xObjectInputStream, uno::UNO_QUERY);
			uno::Reference< io::XInputStream > xInputStream(xMarkableStream, uno::UNO_QUERY);
			xDataSink->setInputStream(xInputStream);
			
			m_xMarkableStream = uno::Reference<io::XMarkableStream>(xObjectInputStream, uno::UNO_QUERY);
			m_xDataInputStream = uno::Reference<io::XDataInputStream>(xObjectInputStream, uno::UNO_QUERY);
		}
		else
		{
			CFG_TRACE_ERROR("ensure_File(): no ObjectInputStream, Exception!");
			throw io::IOException(ASCII("There is no ObjectInputStream"), NULL);
		}
	}
	else
	{
		CFG_TRACE_ERROR("ensure_File(): no MarkableStream, Exception!");
		throw io::IOException(ASCII("There is no MarkableStream"), NULL);
	}
}

// -----------------------------------------------------------------------------
void OBinaryBaseReader::dispose()
{
	// if (m_aFileIn.get())
	// {
	// 	   m_aFileIn->close();
	// 	   m_aFileIn.release(); // drop pointer
	// }
}
// -----------------------------------------------------------------------------
OBinaryBaseReader::~OBinaryBaseReader()
{
	uno::Reference< io::XInputStream> xInputStream(m_xDataInputStream, uno::UNO_QUERY);
	if (xInputStream.is())
	{
		xInputStream->closeInput();
	}
	
	// if (m_aFileIn.get())
	// {
	// 	   OSL_ENSURE(false, "dispose forgotten?");
	// 	   dispose();
	// }
}

// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(sal_Bool &_bValue) throw (io::IOException)
{
	_bValue = m_xDataInputStream->readBoolean();
}
// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(sal_Int8 &_nValue) throw (io::IOException)
{
	_nValue = m_xDataInputStream->readByte();
}
// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(sal_Int16 &_nValue) throw (io::IOException)
{
	_nValue = m_xDataInputStream->readShort();
}
// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(sal_Int32 &_nValue) throw (io::IOException)
{
	_nValue = m_xDataInputStream->readLong();
}
// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(sal_Int64 &_nValue) throw (io::IOException)
{
	_nValue = m_xDataInputStream->readHyper();
}
// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(double &_nValue) throw (io::IOException)
{
	_nValue = m_xDataInputStream->readDouble();
}
// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(sal_Unicode &_sValue) throw (io::IOException)
{
	_sValue = m_xDataInputStream->readChar();
}

// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(sal_Unicode *pStr, sal_Int32 _nCount) throw (io::IOException)
{
	sal_uInt64 nLength = _nCount;
	sal_Unicode* pPtr = pStr;
	while(nLength--)
	{
		*pPtr++ = m_xDataInputStream->readChar();
	}
}
// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(uno::Sequence<sal_Int8> &_aBuf, sal_Int32 _nCount) throw (io::IOException)
{
	m_xDataInputStream->readBytes(_aBuf, _nCount);
}

// -----------------------------------------------------------------------------
void readString(OBinaryBaseReader& _aReader, rtl::OUString &_aString) throw (io::IOException);

void OBinaryBaseReader::read(rtl::OUString& _aStr) throw (io::IOException)
{
	readString(*this, _aStr);
}
// -----------------------------------------------------------------------------
OInputMark::OInputMark(OBinaryBaseReader* _pReader)
		:m_pReader(_pReader)
{
	m_nMark = _pReader->getMarkableStream()->createMark();
	_pReader->read(m_nPosition);
}

void OInputMark::jumpToMark()
{
	m_pReader->getMarkableStream()->jumpToMark(m_nMark);
	m_pReader->getDataInputStream()->skipBytes(m_nPosition);
	m_pReader->getMarkableStream()->deleteMark(m_nMark);
}

// -----------------------------------------------------------------------------
template <class Element>
static
void readSequence(OBinaryBaseReader& _rReader, uno::Sequence< Element > & aSequence)
{
	// PRE: the Sequence must exist
	sal_Int32 nLength;
	_rReader.read(nLength);

	if (aSequence.getLength() != nLength)		 // check size
	{
		aSequence.realloc(nLength);
	}

	Element* pElement = aSequence.getArray();	 // fill the hole array
	for(sal_Int32 i=0; i<nLength; ++i) 
	{
		Element aValue;
		_rReader.read(aValue);					 // read one element
		*pElement++ = aValue;					 // like pElement[i]
	}
}

// -----------------------------------------------------------------------------
void OBinaryBaseReader::read(uno::Sequence<sal_Int8> &_aValue) throw (io::IOException)
{
	readSequence(*this, _aValue);
}
// -----------------------------------------------------------------------------

#define CASE_READ_SEQUENCE(TYPE_CLASS, DATA_TYPE)   \
    case TYPE_CLASS:                                \
    {                                               \
		OSL_ENSURE( ::getCppuType(static_cast<DATA_TYPE const*>(0)).getTypeClass() == (TYPE_CLASS), "Typeclass does not match element type" );    \
		uno::Sequence< DATA_TYPE > aData;                \
        readSequence(_rReader, aData);                   \
        _aValue <<= aData;                               \
    }   break                                             

void readSequenceValue(OBinaryBaseReader& _rReader, uno::Any &_aValue, uno::Type const& aElementType)
{
	switch(aElementType.getTypeClass())
	{
	CASE_READ_SEQUENCE( uno::TypeClass_BOOLEAN, sal_Bool );

	CASE_READ_SEQUENCE( uno::TypeClass_SHORT, sal_Int16 );

	CASE_READ_SEQUENCE( uno::TypeClass_LONG, sal_Int32 );

	CASE_READ_SEQUENCE( uno::TypeClass_HYPER, sal_Int64 );

	CASE_READ_SEQUENCE( uno::TypeClass_DOUBLE, double );

	CASE_READ_SEQUENCE( uno::TypeClass_STRING, rtl::OUString );

	CASE_READ_SEQUENCE( uno::TypeClass_SEQUENCE, uno::Sequence<sal_Int8> );

	CASE_READ_SEQUENCE( uno::TypeClass_BYTE, sal_Int8 ); /* Binary */

	default: 
		OSL_ENSURE(false, "Unexpected typeclass for sequence elements");
		break;
	}
}

#undef CASE_READ_SEQUENCE
 
}
