/*************************************************************************
 *
 *  $RCSfile: simple_collator.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: nn $ $Date: 2001/05/07 16:28:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <simple_collator.hxx>
#include <tools/isolang.hxx>
#include <com/sun/star/i18n/CollatorOptions.hpp>

SimpleCollator::SimpleCollator () 
		: mpInternational (NULL)
		, mnCompFlags (0)
{
}

SimpleCollator::~SimpleCollator ()
{
	if (mpInternational != NULL)
		delete mpInternational;
}

sal_Int32 SAL_CALL
SimpleCollator::compareSubstring (
    	const ::rtl::OUString& str1, sal_Int32 off1, sal_Int32 len1, 
    	const ::rtl::OUString& str2, sal_Int32 off2, sal_Int32 len2
	) throw (::com::sun::star::uno::RuntimeException)
{
    String newStr1 = str1.copy(off1, len1);
    String newStr2 = str2.copy(off2, len2);
    
    return compareString (newStr1, newStr2);
}

sal_Int32 SAL_CALL
SimpleCollator::compareString (
    	const ::rtl::OUString& str1, 
    	const ::rtl::OUString& str2
	) throw (::com::sun::star::uno::RuntimeException)
{
	if (mpInternational == NULL)
		throw ::com::sun::star::uno::RuntimeException();

	// let the International class do the work
	StringCompare eCompare = mpInternational->Compare (str1, str2, mnCompFlags);
	if (eCompare == COMPARE_LESS)
		return -1;
	else
	if (eCompare == COMPARE_EQUAL)
		return 0;
	else
	// (eCompare == COMPARE_GREATER)
		return 1;
}

sal_Int32 SAL_CALL
SimpleCollator::loadDefaultCollator ( 
		const ::com::sun::star::lang::Locale& rLocale,
		sal_Int32 collatorOptions 
	) throw (::com::sun::star::uno::RuntimeException)
{
	// convert the Locale to Language
	LanguageType eLang = ConvertIsoNamesToLanguage (rLocale.Language, rLocale.Country);
	if (eLang == LANGUAGE_DONTKNOW)
		eLang = LANGUAGE_ENGLISH_US;

	// make sure the International member is sane and fits the language
	if (mpInternational == NULL)
	{
		mpInternational = new International (eLang);
	}
	else
	if (mpInternational->GetLanguage() != eLang)
	{
		delete mpInternational;
		mpInternational = new International (eLang);
	} 

	if (collatorOptions & ::com::sun::star::i18n::CollatorOptions::CollatorOptions_IGNORE_CASE)
		mnCompFlags = INTN_COMPARE_IGNORECASE;
	else
		mnCompFlags = 0;

    return 0;
}

sal_Int32 SAL_CALL 
SimpleCollator::loadCollatorAlgorithm(	
		const ::rtl::OUString& impl, 
		const ::com::sun::star::lang::Locale& rLocale, 
		sal_Int32 collatorOptions 
	) throw(::com::sun::star::uno::RuntimeException)
{
	return loadDefaultCollator (rLocale, collatorOptions);
}

::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL 
SimpleCollator::listCollatorAlgorithms( 
		const ::com::sun::star::lang::Locale& rLocale 
	) throw(::com::sun::star::uno::RuntimeException)
{
	return ::com::sun::star::uno::Sequence< ::rtl::OUString >();
}

void SAL_CALL 
SimpleCollator::loadCollatorAlgorithmWithEndUserOption( 
		const ::rtl::OUString& impl, 
		const ::com::sun::star::lang::Locale& rLocale, 
		const ::com::sun::star::uno::Sequence< sal_Int32 >& collatorOptions 
	) throw(::com::sun::star::uno::RuntimeException)
{
	loadDefaultCollator (rLocale, 0 /* collatorOptions[0] */);
}

::com::sun::star::uno::Sequence< sal_Int32 > SAL_CALL 
SimpleCollator::listCollatorOptions( 
		const ::rtl::OUString& collatorAlgorithmName 
	) throw(::com::sun::star::uno::RuntimeException)
{
	return ::com::sun::star::uno::Sequence< sal_Int32 >();
}

