/*************************************************************************
 *
 *  $RCSfile: apearcfg.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: os $ $Date: 2001/07/24 07:28:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PRECOMPILED
#include "ofapch.hxx"
#endif

#pragma hdrstop

#include "app.hrc"
#include "app.hxx"
#include "apearcfg.hxx"
#include "resid.hxx"
#include "optgdlg.hxx"

#define DEFAULT_LOOKNFEEL   0
#define DEFAULT_DRAGMODE    2
#define DEFAULT_SNAPMODE    0
#define DEFAULT_SCALEFACTOR 100

using namespace ::rtl;
using namespace ::com::sun::star::uno;

sal_Bool OfaTabAppearanceCfg::bInitialized = sal_False;

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/
OfaTabAppearanceCfg::OfaTabAppearanceCfg() :
    ConfigItem(OUString::createFromAscii("Office.Common/View")),
	nLookNFeel   ( DEFAULT_LOOKNFEEL ),
	nDragMode    ( DEFAULT_DRAGMODE ),
	nSnapMode    ( DEFAULT_SNAPMODE ),
	nScaleFactor ( DEFAULT_SCALEFACTOR ),
	bMenuMouseFollow(FALSE),
	bSingleLineTabCtrl(FALSE),
    bColoredTabCtrl(FALSE)
{
    const Sequence<OUString>& rNames = GetPropertyNames();
    Sequence<Any> aValues = GetProperties(rNames);
	const Any* pValues = aValues.getConstArray();
    DBG_ASSERT(aValues.getLength() == rNames.getLength(), "GetProperties failed")
    if(aValues.getLength() == rNames.getLength())
	{
        for(int nProp = 0; nProp < rNames.getLength(); nProp++)
		{
			if(pValues[nProp].hasValue())
			{
				switch(nProp)
				{
                    case  0: pValues[nProp] >>= nScaleFactor; break; //"FontScaling",
                    case  1: pValues[nProp] >>= nLookNFeel; break;  //"LookAndFeel",
                    case  2: pValues[nProp] >>= nDragMode; break;   //"Window/Drag",
                    case  3: bMenuMouseFollow = *(sal_Bool*)pValues[nProp].getValue(); break; //"Menu/FollowMouse",
                    case  4: bSingleLineTabCtrl      = *(sal_Bool*)pValues[nProp].getValue(); break; //"Dialog/SingleLineTab",
                    case  5: bColoredTabCtrl = *(sal_Bool*)pValues[nProp].getValue(); break; //"Dialog/ColoredTab",
                    case  6: pValues[nProp] >>= nSnapMode; break; //"Dialog/MousePositioning",

                }
			}
		}
    }
}
/* -----------------------------22.05.01 11:53--------------------------------

 ---------------------------------------------------------------------------*/
OfaTabAppearanceCfg::~OfaTabAppearanceCfg( )
{
}
/* -----------------------------22.05.01 11:54--------------------------------

 ---------------------------------------------------------------------------*/
const Sequence<OUString>& OfaTabAppearanceCfg::GetPropertyNames()
{
    static Sequence<OUString> aNames;
    if(!aNames.getLength())
    {
        static const sal_Char* aPropNames[] =
        {
            "FontScaling",                      //  0
            "LookAndFeel",                      //  1
            "Window/Drag",                      //  2
            "Menu/FollowMouse",                 //  3
            "Dialog/SingleLineTab",             //  4
            "Dialog/ColoredTab",                //  5
            "Dialog/MousePositioning"           //  6
        };
        const int nCount = 7;
        aNames.realloc(nCount);
        OUString* pNames = aNames.getArray();
        for(int i = 0; i < nCount; i++)
            pNames[i] = OUString::createFromAscii(aPropNames[i]);
    }
    return aNames;
}
/* -----------------------------22.05.01 11:54--------------------------------

 ---------------------------------------------------------------------------*/
void  OfaTabAppearanceCfg::Commit()
{
    const Sequence<OUString>& rNames = GetPropertyNames();
    const OUString* pNames = rNames.getConstArray();
    Sequence<Any> aValues(rNames.getLength());
	Any* pValues = aValues.getArray();

	const Type& rType = ::getBooleanCppuType();
    for(int nProp = 0; nProp < rNames.getLength(); nProp++)
	{
		switch(nProp)
		{
            case  0: pValues[nProp] <<= nScaleFactor; break;            // "FontScaling",
            case  1: pValues[nProp] <<= nLookNFeel; break;              //"LookAndFeel",
            case  2: pValues[nProp] <<= nDragMode; break;               //"Window/Drag",
            case  3: pValues[nProp].setValue(&bMenuMouseFollow, rType); break; //"Menu/FollowMouse",
            case  4: pValues[nProp].setValue(&bSingleLineTabCtrl, rType); break; //"Dialog/SingleLineTab",
            case  5: pValues[nProp].setValue(&bColoredTabCtrl, rType); break; //"Dialog/ColoredTab",
            case  6: pValues[nProp] <<= nSnapMode; break;               //"Dialog/MousePositioning",
        }
	}
    PutProperties(rNames, aValues);
}
/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetLookNFeel ( USHORT nSet )
{
	nLookNFeel = nSet;
    SetModified();
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetDragMode  ( USHORT nSet )
{
	nDragMode = nSet;
    SetModified();
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetScaleFactor ( USHORT nSet )
{
	nScaleFactor = nSet;
    SetModified();
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetSnapMode ( USHORT nSet )
{
	nSnapMode = nSet;
    SetModified();
}


/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetApplicationDefaults ( Application* pApp )
{
	AllSettings   hAppSettings = pApp->GetSettings();
	StyleSettings hAppStyle    = hAppSettings.GetStyleSettings();

	// Look & Feel

	switch ( nLookNFeel )
	{
	case LookMotif:
		hAppStyle.SetStandardUnixStyles();
		break;
	case LookOSTwo:
		hAppStyle.SetStandardOS2Styles();
		break;
	case LookMacintosh:
		hAppStyle.SetStandardMacStyles();
		break;
	case LookWindows:
		hAppStyle.SetStandardWinStyles();
		break;
	case LookStardivision:
	default:
		hAppStyle.SetStandardStyles();
		break;
	}

	// Screen and ScreenFont Scaling

	hAppStyle.SetScreenZoom( nScaleFactor );
	hAppStyle.SetScreenFontZoom( nScaleFactor );

	// Mouse Snap

	MouseSettings hMouseSettings = hAppSettings.GetMouseSettings();
	ULONG         nMouseOptions  = hMouseSettings.GetOptions();

	nMouseOptions &=  ! (MOUSE_OPTION_AUTOCENTERPOS | MOUSE_OPTION_AUTODEFBTNPOS);

	switch ( nSnapMode )
	{
	case SnapToButton:
		nMouseOptions |= MOUSE_OPTION_AUTODEFBTNPOS;
		break;
	case SnapToMiddle:
		nMouseOptions |= MOUSE_OPTION_AUTOCENTERPOS;
		break;
	case NoSnap:
	default:
		break;
	}

	// Merge and Publish Settings

    ULONG nFollow = hMouseSettings.GetFollow();
    if(bMenuMouseFollow)
        nFollow |= MOUSE_FOLLOW_MENU;
    else
        nFollow &= ~MOUSE_FOLLOW_MENU;
    hMouseSettings.SetFollow( nFollow );
    sal_uInt16 nTabStyle = 0;
    if(bSingleLineTabCtrl)
        nTabStyle |= STYLE_TABCONTROL_SINGLELINE;

    if(bColoredTabCtrl)
        nTabStyle |= STYLE_TABCONTROL_COLOR;
    hAppStyle.SetTabControlStyle(nTabStyle);


    hAppSettings.SetMouseSettings( hMouseSettings );

	hAppSettings.SetStyleSettings( hAppStyle );
	pApp->MergeSystemSettings    ( hAppSettings );      // Allow system-settings to apply
	pApp->SystemSettingsChanging ( hAppSettings, NULL );// Allow overruling of system-settings
														//is concerned with window drag

	pApp->SetSettings ( hAppSettings );
}



