/*************************************************************************
 *
 *  $RCSfile: XclExpExternsheet.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/30 12:39:09 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//___________________________________________________________________

#ifndef _SC_XCLEXPEXTERNSHEET_HXX
#include "XclExpExternsheet.hxx"
#endif

#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

#ifndef SC_DOCUMENT_HXX
#include "document.hxx"
#endif
#ifndef _SCEXTOPT_HXX
#include "scextopt.hxx"
#endif

#ifndef _SC_XCLEXPSTREAM_HXX
#include "XclExpStream.hxx"
#endif

//___________________________________________________________________
// class XclExpTabNumBuffer

XclExpTabNumBuffer::XclExpTabNumBuffer( ScDocument& rDoc ) :
    bEnableLog( FALSE ),
    nCodeCnt( 0 )
{
    nScCnt = rDoc.GetTableCount();
    pBuffer = nScCnt ? new UINT32[ nScCnt ] : NULL;

    for( UINT16 nTab = 0; nTab < nScCnt; nTab++ )
    {
        pBuffer[ nTab ] = 0;

        // ignored tables (skipped by export, invalid Excel table number)
        if( rDoc.IsScenario( nTab ) )
            pBuffer[ nTab ] = EXC_TABBUF_FLAGIGNORE;

        // external tables (skipped, with valid Excel table number for ref's)
        else if( rDoc.GetLinkMode( nTab ) == SC_LINK_VALUE )
            pBuffer[ nTab ] = EXC_TABBUF_FLAGEXT;
    }
    ApplyBuffer();

    if( rDoc.GetExtDocOptions() )
    {
        CodenameList* pCList = rDoc.GetExtDocOptions()->GetCodenames();
        if( pCList )
            nCodeCnt = (UINT16) Min( pCList->Count(), 0xFFFFUL );
    }
}

XclExpTabNumBuffer::~XclExpTabNumBuffer()
{
    if( pBuffer )
        delete[] pBuffer;
}

void XclExpTabNumBuffer::ApplyBuffer()
{
    UINT16 nIndex = 0;
    UINT16 nTab;
    nExcCnt = nExtCnt = 0;

    // regular tables
    for( nTab = 0; nTab < nScCnt; nTab++ )
    {
        if( IsExportTable( nTab ) )
        {
            pBuffer[ nTab ] |= nIndex;
            nIndex++;
            nExcCnt++;
        }
        else
            pBuffer[ nTab ] |= EXC_TABBUF_INVALID;
    }

    // external tables
    for( nTab = 0; nTab < nScCnt; nTab++ )
    {
        if( IsExternal( nTab ) )
        {
            pBuffer[ nTab ] &= EXC_TABBUF_MASKFLAGS;
            pBuffer[ nTab ] |= nIndex;
            nIndex++;
            nExtCnt++;
        }
    }
}

BOOL XclExpTabNumBuffer::IsExternal( UINT16 nScTab ) const
{
    return (nScTab < nScCnt) ? TRUEBOOL( pBuffer[ nScTab ] & EXC_TABBUF_FLAGEXT ) : FALSE;
}

BOOL XclExpTabNumBuffer::IsExportTable( UINT16 nScTab ) const
{
    DBG_ASSERT( nScTab < nScCnt, "XclExpTabNumBuffer::IsExportTable - out of range!" );
    return (pBuffer[ nScTab ] & EXC_TABBUF_MASKFLAGS) == 0;
}

UINT16 XclExpTabNumBuffer::GetExcTable( UINT16 nScTab ) const
{
    return (nScTab < nScCnt) ? (UINT16)(pBuffer[ nScTab ] & EXC_TABBUF_MASKTAB) : EXC_TABBUF_INVALID;
}

void XclExpTabNumBuffer::AppendTabRef( UINT16 nExcFirst, UINT16 nExcLast )
{
    if( bEnableLog )
    {
        Append( nExcFirst );
        Append( nExcLast );
    }
}



//___________________________________________________________________
// class XclExpCrn

void XclExpCrn::SaveCont( XclExpStream& rStrm )
{
    rStrm << (UINT8) nCol << (UINT8) nCol << nRow << nId;
    SaveDiff( rStrm );
}

UINT16 XclExpCrn::GetNum() const
{
    return 0x005A;
}

ULONG XclExpCrn::GetLen() const
{
    return 5 + GetDiffLen();
}



//___________________________________________________________________
// class XclExpCrnDouble

XclExpCrnDouble::XclExpCrnDouble( UINT16 _nCol, UINT16 _nRow, double _fVal ) :
    XclExpCrn( _nCol, _nRow, EXC_CRN_DOUBLE ),
    fVal( _fVal )
{
}

void XclExpCrnDouble::SaveDiff( XclExpStream& rStrm )
{
    rStrm << fVal;
}

ULONG XclExpCrnDouble::GetDiffLen() const
{
    return 8;
}



//___________________________________________________________________
// class XclExpCrnString

XclExpCrnString::XclExpCrnString( UINT16 _nCol, UINT16 _nRow, const String& rTxt ) :
    XclExpCrn( _nCol, _nRow, EXC_CRN_STRING ),
    aText( rTxt )
{
}

void XclExpCrnString::SaveDiff( XclExpStream& rStrm )
{
    aText.Write( rStrm );
}

ULONG XclExpCrnString::GetDiffLen() const
{
    return aText.GetByteCount();
}



//___________________________________________________________________
// class XclExpCrnBool

XclExpCrnBool::XclExpCrnBool( UINT16 _nCol, UINT16 _nRow, BOOL bBoolVal ) :
    XclExpCrn( _nCol, _nRow, EXC_CRN_BOOL ),
    nBool( bBoolVal ? 1 : 0 )
{
}

void XclExpCrnBool::SaveDiff( XclExpStream& rStrm )
{
    rStrm << nBool << (UINT16)0 << (UINT32)0;
}

ULONG XclExpCrnBool::GetDiffLen() const
{
    return 8;
}



//___________________________________________________________________
// class XclExpXct

XclExpXct::~XclExpXct()
{
    for( XclExpCrn* pCrn = _First(); pCrn; pCrn = _Next() )
        delete pCrn;
}

BOOL XclExpXct::Exists( UINT16 nCol, UINT16 nRow )
{
    for( XclExpCrn* pCrn = _First(); pCrn; pCrn = _Next() )
        if( pCrn->IsAddress( nCol, nRow ) )
            return TRUE;
    return FALSE;
}

void XclExpXct::StoreCellRange( RootData& rRootData, const ScRange& rRange )
{
    ScDocument&         rDoc    = *rRootData.pDoc;
    SvNumberFormatter&  rFTab   = *rRootData.pFormTable;
    UINT16              nTab    = rRange.aStart.Tab();

    for( UINT16 nRow = rRange.aStart.Row(); nRow <= rRange.aEnd.Row(); nRow++ )
        for( UINT16 nCol = rRange.aStart.Col(); nCol <= rRange.aEnd.Col(); nCol++ )
            if( !Exists( nCol, nRow ) )
            {
                if( rDoc.HasValueData( nCol, nRow, nTab ) )
                {
                    ScAddress   aAddr( nCol, nRow, nTab );
                    double      fVal    = rDoc.GetValue( aAddr );
                    ULONG       nFormat = rDoc.GetNumberFormat( aAddr );
                    short       nType   = rFTab.GetType( nFormat );
                    BOOL        bIsBool = (nType == NUMBERFORMAT_LOGICAL);

                    if( !bIsBool && ((nFormat % SV_COUNTRY_LANGUAGE_OFFSET) == 0) &&
                        (rDoc.GetCellType( aAddr ) == CELLTYPE_FORMULA) )
                    {
                        ScFormulaCell* pCell = (ScFormulaCell*) rDoc.GetCell( aAddr );
                        if ( pCell )
                            bIsBool = (pCell->GetFormatType() == NUMBERFORMAT_LOGICAL);
                    }

                    if( bIsBool && ((fVal == 0.0) || (fVal == 1.0)) )
                        Append( new XclExpCrnBool( nCol, nRow, (fVal == 1.0) ) );
                    else
                        Append( new XclExpCrnDouble( nCol, nRow, fVal ) );
                }
                else
                {
                    String aText;
                    rDoc.GetString( nCol, nRow, nTab, aText );
                    Append( new XclExpCrnString( nCol, nRow, aText ) );
                }
            }
}

void XclExpXct::SaveCont( XclExpStream& rStrm )
{
    rStrm << (UINT16) List::Count() << nTabNum;
}

void XclExpXct::Save( XclExpStream& rStrm )
{
    ExcRecord::Save( rStrm );

    for( XclExpCrn* pCrn = _First(); pCrn; pCrn = _Next() )
        pCrn->Save( rStrm );
}

UINT16 XclExpXct::GetNum() const
{
    return 0x0059;
}

ULONG XclExpXct::GetLen() const
{
    return 4;
}



//___________________________________________________________________
// class XclExpExternname

XclExpExternname::XclExpExternname( const String& rName, UINT16 _nFlags ) :
    aName( rName ),
    nFlags( _nFlags )
{
    DBG_ASSERT( aName.Len() <= 255, "XclExpExternname::XclExpExternname - no strings longer than 255 allowed!" );
}

XclExpExternname::~XclExpExternname()
{
}

void XclExpExternname::SaveCont( XclExpStream& rStrm )
{
    rStrm   << nFlags                                       // grbit
            << (UINT32) 0;                                  // reserved
    XclExpUniString( aName, 255 ).Write( rStrm, FALSE );    // 8 bit length

    SaveDiff( rStrm );
}

UINT16 XclExpExternname::GetNum() const
{
    return 0x0023;
}

ULONG XclExpExternname::GetLen() const
{
    return aName.Len() + 8 + GetDiffLen();
}



//___________________________________________________________________
// class XclExpExternnameAddIn

void XclExpExternnameAddIn::SaveDiff( XclExpStream& rStrm )
{
    rStrm << (UINT16) 0x0002 << (UINT16) 0x171C;        // error value 0x17
}

ULONG XclExpExternnameAddIn::GetDiffLen() const
{
    return 4;
}



//___________________________________________________________________
// class XclExpExternnameDDE

XclExpExternnameDDE::~XclExpExternnameDDE()
{
    if( pOpList )
        delete pOpList;
}

BOOL XclExpExternnameDDE::InsertDDE(
        RootData& rRootData,
        const String& rApplic, const String& rTopic, const String& rItem )
{
    ScDocument& rScDoc = *rRootData.pDoc;

    USHORT nPos;
    if( rScDoc.FindDdeLink( rApplic, rTopic, rItem, SC_DDE_IGNOREMODE, nPos ) )
    {
        ScMatrix* pMatrix = NULL;
        USHORT nCols, nRows;
        if( rScDoc.GetDdeLinkResultDimension( nPos, nCols, nRows, pMatrix ) && pMatrix )
            pOpList = new XclExpCachedValueList( rScDoc, nCols, nRows, pMatrix, TRUE );
        return TRUE;
    }
    return FALSE;
}

void XclExpExternnameDDE::SaveDiff( XclExpStream& rStrm )
{
    if( pOpList )
        pOpList->Save( rStrm );
}

ULONG XclExpExternnameDDE::GetDiffLen() const
{
    return pOpList ? pOpList->GetLen() : 0;
}



//___________________________________________________________________
// class XclExpExternNameList

XclExpExternnameList::~XclExpExternnameList()
{
    for( XclExpExternname* pName = _First(); pName; pName = _Next() )
        delete pName;
}

UINT16 XclExpExternnameList::GetExtname( const String& rName )
{
    for( XclExpExternname* pName = _First(); pName; pName = _Next() )
        if( pName->GetName() == rName )
            return (UINT16)(List::GetCurPos() + 1);
    return 0;
}

UINT16 XclExpExternnameList::InsertAddin( const String& rName )
{
    UINT16 nIndex = GetExtname( rName );
    return nIndex ? nIndex : Append( new XclExpExternnameAddIn( rName ) );
}

UINT16 XclExpExternnameList::InsertDDE(
        RootData& rRootData,
        const String& rApplic, const String& rTopic, const String& rItem )
{
    if( !List::Count() )
        Append( new XclExpExternnameDDE( String( RTL_CONSTASCII_USTRINGPARAM( "StdDocumentName" ) ), 0x7FEA ) );

    UINT16 nIndex = GetExtname( rItem );
    if( !nIndex )
    {
        XclExpExternnameDDE* pExtname = new XclExpExternnameDDE( rItem, 0x7FE2 );
        if( pExtname->InsertDDE( rRootData, rApplic, rTopic, rItem ) )
            nIndex = Append( pExtname );
        else
            delete pExtname;
    }
    return nIndex;
}

void XclExpExternnameList::Save( XclExpStream& rStrm )
{
    for( XclExpExternname* pName = _First(); pName; pName = _Next() )
        pName->Save( rStrm );
}



//___________________________________________________________________
// class XclExpSupbook

XclExpSupbook::XclExpSupbook() :
    pExtnameList( NULL ),
    eType( SB_Addin ),
    nLen( 4 ),
    nTables( 1 )
{
}

XclExpSupbook::XclExpSupbook( UINT16 nTabs ) :
    pExtnameList( NULL ),
    eType( SB_Self ),
    nLen( 4 ),
    nTables( nTabs )
{
}

XclExpSupbook::XclExpSupbook( const String& rDocName, BOOL bDDE ) :
    aDocName( rDocName ),
    pExtnameList( NULL ),
    eType( SB_Ext )
{
    if( bDDE )
        aEncoded.Assign( rDocName );
    else
    {
        INetURLObject aURLObj( rDocName );
        DBG_ASSERT( !aURLObj.HasError(), "XclExpSupbook::XclExpSupbook - Corrupt external filename!" );

        aEncoded.Assign( aURLObj.getFSysPath( INetURLObject::FSYS_DOS ) );
        if( !aEncoded.GetLen() )
            aEncoded.Assign( rDocName );
    }
    nLen = 2 + aEncoded.GetByteCount();
}

XclExpSupbook::~XclExpSupbook()
{
    for( XclExpXct* pXct = _First(); pXct; pXct = _Next() )
        delete pXct;
    if( pExtnameList )
        delete pExtnameList;
}

XclExpExternnameList& XclExpSupbook::GetExternnameList()
{
    if( !pExtnameList )
        pExtnameList = new XclExpExternnameList;
    return *pExtnameList;
}

const XclExpUniString* XclExpSupbook::GetTableName( UINT16 nIndex ) const
{
    XclExpXct* pXct = _Get( nIndex );
    return pXct ? &pXct->GetTableName() : NULL;
}

void XclExpSupbook::StoreCellRange( RootData& rRootData, const ScRange& rRange, UINT16 nXct )
{
    XclExpXct* pXct = _Get( nXct );
    if( pXct )
        pXct->StoreCellRange( rRootData, rRange );
}

UINT16 XclExpSupbook::InsertTable( const String& rTabName )
{
    DBG_ASSERT( eType == SB_Ext, "XclExpSupbook::InsertTable - Don't insert table names here" );
    XclExpXct* pXct = new XclExpXct( rTabName );
    nLen += pXct->GetTableBytes();
    List::Insert( pXct, LIST_APPEND );

    UINT16 nTabNum = (UINT16) Min( List::Count() - 1, 0xFFFFUL );
    pXct->SetTableNum( nTabNum );
    return nTabNum;
}

UINT16 XclExpSupbook::InsertAddin( const String& rName )
{
    return GetExternnameList().InsertAddin( rName );
}

UINT16 XclExpSupbook::InsertDDE(
        RootData& rRootData,
        const String& rApplic, const String& rTopic, const String& rItem )
{
    return GetExternnameList().InsertDDE( rRootData, rApplic, rTopic, rItem );
}

void XclExpSupbook::SaveCont( XclExpStream& rStrm )
{
    switch( eType )
    {
        case SB_Addin:  rStrm << nTables << UINT8( 0x01 ) << UINT8( 0x3A ); break;
        case SB_Self:   rStrm << nTables << UINT8( 0x01 ) << UINT8( 0x04 ); break;
        default:
        {
            rStrm << (UINT16) List::Count();
            aEncoded.Write( rStrm );

            for( XclExpXct* pXct = _First(); pXct; pXct = _Next() )
                pXct->GetTableName().Write( rStrm );
        }
    }
}

void XclExpSupbook::Save( XclExpStream& rStrm )
{
    ExcRecord::Save( rStrm );

    for( XclExpXct* pXct = _First(); pXct; pXct = _Next() )
        pXct->Save( rStrm );

    if( pExtnameList )
        pExtnameList->Save( rStrm );
}

UINT16 XclExpSupbook::GetNum() const
{
    return 0x01AE;
}

ULONG XclExpSupbook::GetLen() const
{
    return nLen;
}



//___________________________________________________________________
// class XclExpSupbookList

XclExpSupbookList::XclExpSupbookList( RootData& rRootData ) :
        ExcRoot( &rRootData ),
        pSupbookBuffer( NULL ),
        pTableBuffer( NULL ),
        nAddinSupb( EXC_TAB_INVALID )
{
    XclExpTabNumBuffer& rTabBuffer = *rRootData.pTabBuffer;

    UINT16  nScCnt  = rTabBuffer.GetScTabCount();
    UINT16  nExcCnt = rTabBuffer.GetExcTabCount();
    UINT16  nExtCnt = rTabBuffer.GetExternTabCount();

    nRefdCnt = nExcCnt + nExtCnt;
    if( !nRefdCnt )
        return;

    pSupbookBuffer = new UINT16[ nRefdCnt ];
    pTableBuffer = new UINT16[ nRefdCnt ];

    // self-ref supbook first of list
    UINT16 nInd;
    UINT16 nSelfInd = Append( new XclExpSupbook( Max( nExcCnt, rRootData.nCodenames ) ) );
    for( nInd = 0; nInd < nExcCnt; nInd++ )
    {
        pSupbookBuffer[ nInd ] = nSelfInd;
        pTableBuffer[ nInd ] = nInd;
    }

    // add supbooks with external references
    for( nInd = 0; nInd < nScCnt; nInd++ )
        if( rTabBuffer.IsExternal( nInd ) )
            AddExt( nInd );
}

XclExpSupbookList::~XclExpSupbookList()
{
    for( XclExpSupbook* pBook = _First(); pBook; pBook = _Next() )
        delete pBook;
    if( pSupbookBuffer )
        delete[] pSupbookBuffer;
    if( pTableBuffer )
        delete[] pTableBuffer;
}

XclExpSupbook* XclExpSupbookList::GetSupbook( UINT16& rnIndex, const String& rDoc )
{
    for( XclExpSupbook* pBook = _First(); pBook; pBook = _Next() )
    {
        if( pBook->IsExt() && (pBook->GetName() == rDoc) )
        {
            rnIndex = (UINT16) Min( List::GetCurPos(), 0xFFFFUL );
            return pBook;
        }
    }
    return NULL;
}

UINT16 XclExpSupbookList::Append( XclExpSupbook* pBook )
{
    List::Insert( pBook, LIST_APPEND );
    return (UINT16) Min( List::Count() - 1, 0xFFFFUL );
}

void XclExpSupbookList::AddExt( UINT16 nScTab )
{
    // find ext doc name or append new one, save position in pSupbookBuffer
    const String& rExtDoc = pExcRoot->pDoc->GetLinkDoc( nScTab );
    DBG_ASSERT( rExtDoc.Len(), "XclExpSupbookList::AddExt - Missing external filename!" );

    UINT16 nPos;
    XclExpSupbook* pBook = GetSupbook( nPos, rExtDoc );
    if( !pBook )
        nPos = Append( pBook = new XclExpSupbook( rExtDoc, FALSE ) );

    UINT16 nInd = pExcRoot->pTabBuffer->GetExcTable( nScTab );
    DBG_ASSERT( nInd < nRefdCnt, "XclExpSupbookList::AddExt - Out of range!" );
    pSupbookBuffer[ nInd ] = nPos;

    // append new table name, save position in pTableBuffer
    nPos = pBook->InsertTable( pExcRoot->pDoc->GetLinkTab( nScTab ) );
    pTableBuffer[ nInd ] = nPos;
}

const XclExpUniString* XclExpSupbookList::GetDocumentName( UINT16 nExcTab ) const
{
    DBG_ASSERT( nExcTab < nRefdCnt, "XclExpSupbookList::GetDocumentName - out of range" );
    const XclExpSupbook* pSupbook = GetSupbook( nExcTab );
    const XclExpUniString* pString = pSupbook ? &pSupbook->GetEncName() : NULL;
    return (pString && pString->GetLen()) ? pString : NULL;
}

const XclExpUniString* XclExpSupbookList::GetTableName( UINT16 nExcTab ) const
{
    DBG_ASSERT( nExcTab < nRefdCnt, "XclExpSupbookList::GetTableName - out of range" );
    const XclExpSupbook* pSupbook = GetSupbook( nExcTab );
    return pSupbook ? pSupbook->GetTableName( pTableBuffer[ nExcTab ] ) : NULL;
}

void XclExpSupbookList::GetXtiRange(
        UINT16& rnSupb, UINT16& rnXtiFirst, UINT16& rnXtiLast,
        UINT16 nTabFirst, UINT16 nTabLast ) const
{
    if( (nTabFirst >= nRefdCnt) || (nTabLast >= nRefdCnt) )
    {
        rnSupb = 0;
        rnXtiFirst = nTabFirst;
        rnXtiLast = nTabLast;
    }
    else
    {
        rnSupb = pSupbookBuffer[ nTabFirst ];
        BOOL bConflict = FALSE;

        // all tables in the same supbook?
        for( UINT16 nTab = nTabFirst + 1; !bConflict && (nTab <= nTabLast); nTab++ )
        {
            bConflict = (pSupbookBuffer[ nTab ] != rnSupb);
            if( bConflict )
                nTabLast = nTab - 1;
        }
        rnXtiFirst = pTableBuffer[ nTabFirst ];
        rnXtiLast = pTableBuffer[ nTabLast ];
    }
}

void XclExpSupbookList::StoreCellRange( const ScRange& rRange )
{
    UINT16 nExcTab = pExcRoot->pTabBuffer->GetExcTable( rRange.aStart.Tab() );
    DBG_ASSERT( nExcTab < nRefdCnt, "XclExpSupbookList::StoreCellRange - out of range!" );

    XclExpSupbook* pBook = GetSupbook( nExcTab );
    if( pBook )
        pBook->StoreCellRange( *pExcRoot, rRange, pTableBuffer[ nExcTab ] );
}

void XclExpSupbookList::InsertAddin( UINT16& rnSupbook, UINT16& rnExtname, const String& rName )
{
    XclExpSupbook* pBook;
    if( nAddinSupb == EXC_TAB_INVALID )
        nAddinSupb = Append( pBook = new XclExpSupbook() );
    else
        pBook = _Get( nAddinSupb );
    DBG_ASSERT( pBook, "XclExpSupbookList::InsertAddin - missing Addin supbook" );
    rnSupbook = nAddinSupb;
    rnExtname = pBook->InsertAddin( rName );
}

void XclExpSupbookList::InsertDDE(
        UINT16& rnSupbook, UINT16& rnExtname,
        const String& rApplic, const String& rTopic, const String& rItem )
{
    String aSupbName( rApplic );
    aSupbName += (sal_Unicode) 0x03;
    aSupbName += rTopic;
    XclExpSupbook* pBook = GetSupbook( rnSupbook, aSupbName );
    if( !pBook )
        rnSupbook = Append( pBook = new XclExpSupbook( aSupbName, TRUE ) );
    rnExtname = pBook->InsertDDE( *pExcRoot, rApplic, rTopic, rItem );
}

void XclExpSupbookList::Save( XclExpStream& rStrm )
{
    for( XclExpSupbook* pBook = _First(); pBook; pBook = _Next() )
        pBook->Save( rStrm );
}



//___________________________________________________________________
// class XclExpExternsheetList

XclExpExternsheetList::XclExpExternsheetList( RootData& rRootData ) :
    ExcRoot( &rRootData ),
    aSupbookList( rRootData )
{
    Find( 0, 0 );   // dummy to avoid empty list
}

XclExpExternsheetList::~XclExpExternsheetList()
{
    for ( XclExpXti* pXti = _First(); pXti; pXti = _Next() )
        delete pXti;
}

UINT16 XclExpExternsheetList::Append( XclExpXti* pXti )
{
    DBG_ASSERT( List::Count() < 0xFFFF, "XclExpExternsheetList::Append - too much for Xcl" );
    List::Insert( pXti, LIST_APPEND );
    return GetVal16( List::Count() - 1 );
}

UINT16 XclExpExternsheetList::InsertXti( UINT16 nSupb, UINT16 nFirst, UINT16 nLast )
{
    for( XclExpXti* pXti = _First(); pXti; pXti = _Next() )
        if( pXti->Equals( nSupb, nFirst, nLast ) )
            return GetVal16( List::GetCurPos() );
    return Append( new XclExpXti( nSupb, nFirst, nLast ) );
}

UINT16 XclExpExternsheetList::Find( UINT16 nTabFirst, UINT16 nTabLast )
{
    UINT16 nSupb, nXtiFirst, nXtiLast;
    aSupbookList.GetXtiRange( nSupb, nXtiFirst, nXtiLast, nTabFirst, nTabLast );
    return InsertXti( nSupb, nXtiFirst, nXtiLast );
}

UINT16 XclExpExternsheetList::FindSpecial( UINT16 nSupb )
{
    return InsertXti( nSupb, EXC_TAB_EXTERNAL, EXC_TAB_EXTERNAL );
}

void XclExpExternsheetList::StoreCellCont( const SingleRefData& rRef )
{
    if( pExcRoot->pTabBuffer->IsExternal( rRef.nTab ) )
        aSupbookList.StoreCellRange( ScRange(
            rRef.nCol, rRef.nRow, rRef.nTab, rRef.nCol, rRef.nRow, rRef.nTab ) );
}

void XclExpExternsheetList::StoreCellRange( const SingleRefData& rRef1, const SingleRefData& rRef2 )
{
    for( UINT16 nTab = rRef1.nTab; nTab <= rRef2.nTab; nTab++ )
        if( pExcRoot->pTabBuffer->IsExternal( nTab ) )
            aSupbookList.StoreCellRange( ScRange(
                rRef1.nCol, rRef1.nRow, nTab, rRef2.nCol, rRef2.nRow, nTab ) );
}

void XclExpExternsheetList::InsertAddin( UINT16& rnXti, UINT16& rnExtname, const String& rName )
{
    UINT16 nSupbook;
    aSupbookList.InsertAddin( nSupbook, rnExtname, rName );
    rnXti = FindSpecial( nSupbook );
}

BOOL XclExpExternsheetList::InsertDDE(
        UINT16& rnXti, UINT16& rnExtname,
        const String& rApplic, const String& rTopic, const String& rItem )
{
    UINT16 nSupbook;
    aSupbookList.InsertDDE( nSupbook, rnExtname, rApplic, rTopic, rItem );
    if( !rnExtname )
        return FALSE;
    rnXti = FindSpecial( nSupbook );
    return TRUE;
}

void XclExpExternsheetList::SaveCont( XclExpStream& rStrm )
{
    UINT16 nCount16 = GetCount16();
    rStrm << nCount16;

    rStrm.SetSliceLen( 6 );
    for( UINT16 nIndex = 0; nIndex < nCount16; nIndex++ )
    {
        const XclExpXti* pXti = _Get( nIndex );
        DBG_ASSERT( pXti, "XclExpExternsheetList::SaveCont - no Xti found!" );
        pXti->Save( rStrm );
    }
}

void XclExpExternsheetList::Save( XclExpStream& rStrm )
{
    aSupbookList.Save( rStrm );     // SUPBOOKs, XCTs, CRNs
    ExcRecord::Save( rStrm );       // EXTERNSHEET
}

UINT16 XclExpExternsheetList::GetNum() const
{
    return 0x0017;
}

ULONG XclExpExternsheetList::GetLen() const
{
    return 2 + 6 * GetCount16();
}



//___________________________________________________________________

