/*************************************************************************
 *
 *  $RCSfile: XclImpExternsheet.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/30 11:24:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//___________________________________________________________________

#ifndef _SC_XCLIMPEXTERNSHEET_HXX
#include "XclImpExternsheet.hxx"
#endif

#ifndef SC_DOCUMENT_HXX
#include "document.hxx"
#endif
#ifndef SC_CELL_HXX
#include "cell.hxx"
#endif
#ifndef _SCEXTOPT_HXX
#include "scextopt.hxx"
#endif

#ifndef _SC_XCLIMPSTREAM_HXX
#include "XclImpStream.hxx"
#endif
#ifndef _SC_XCLIMPHELPER_HXX
#include "XclImpHelper.hxx"
#endif
#ifndef _EXCFORM_HXX
#include "excform.hxx"
#endif
#ifndef _XCLADDINNAMETRANS_HXX
#include "XclAddInNameTrans.hxx"
#endif


//___________________________________________________________________
// class XclImpTabIdBuffer

void XclImpTabIdBuffer::Append( UINT16 nTabId )
{
	DBG_ASSERT( nTabId, "XclImpTabIdBuffer::Append - zero value not allowed" );
	if( nTabId )
		UINT16List::Append( nTabId );
}

void XclImpTabIdBuffer::Fill( XclImpStream& rStrm, UINT16 nCount )
{
	Clear();
	UINT16 nTabId;
	for( UINT16 nIndex = 0; nIndex < nCount; nIndex++ )
	{
		rStrm >> nTabId;
		Append( nTabId );
	}
}

UINT16 XclImpTabIdBuffer::GetIndex( UINT16 nTabId, UINT16 nMaxTabId ) const
{
	UINT16 nReturn = 0;
	for( UINT32 nIndex = 0; nIndex < Count(); nIndex++ )
	{
		UINT16 nValue = Get( nIndex );
		if( nValue == nTabId )
			return nReturn;
		if( nValue <= nMaxTabId )
			nReturn++;
	}
	return 0;
}



//___________________________________________________________________
// class XclImpExtName

XclImpExtName::XclImpExtName( XclImpExtNameType _eType, const String& rName, UINT32 _nStorageId ) :
    aName( rName ),
    nStorageId( _nStorageId ),
    eType( _eType )
{
}



//___________________________________________________________________
// class XclImpExtNameBuffer

XclImpExtNameBuffer::~XclImpExtNameBuffer()
{
    for( XclImpExtName* pName = _First(); pName; pName = _Next() )
        delete pName;
}

void XclImpExtNameBuffer::AddName( const String& rName, RootData& rRootData )
{
    XclAddInNameTranslator* pTrans = rRootData.pAddInNameTranslator;
    Append( new XclImpExtName( xienName, pTrans ? pTrans->GetScName( rName ) : rName ) );
}

const XclImpExtName* XclImpExtNameBuffer::GetName( UINT16 nXclIndex ) const
{
    DBG_ASSERT( nXclIndex, "XclImpExtNameBuffer::GetName - index must be >0" );
    return (const XclImpExtName*) List::GetObject( nXclIndex - 1 );
}



//___________________________________________________________________
// class XclImpCrnBase

XclImpCrnBase::~XclImpCrnBase()
{
}



//___________________________________________________________________
// class XclImpCrnDouble

void XclImpCrnDouble::SetCell( ScDocument* pDoc, USHORT nTab )
{
	pDoc->SetValue( nCol, nRow, nTab, fValue );
}



//___________________________________________________________________
// class XclImpCrnString

void XclImpCrnString::SetCell( ScDocument* pDoc, USHORT nTab )
{
	ScStringCell* pStrCell = new ScStringCell( aText );
	pDoc->PutCell( ScAddress( nCol, nRow, nTab ), pStrCell );
}



//___________________________________________________________________
// class XclImpCrnFormula

XclImpCrnFormula::XclImpCrnFormula( USHORT nCol, USHORT nRow, double fVal, const ScTokenArray* pTokenArray ) :
        XclImpCrnDouble( nCol, nRow, fVal ),
        pTokArr( pTokenArray ? new ScTokenArray( *pTokenArray ) : NULL )
{
}

XclImpCrnFormula::~XclImpCrnFormula()
{
    if( pTokArr )
        delete pTokArr;
}

void XclImpCrnFormula::SetCell( ScDocument* pDoc, USHORT nTab )
{
	ScAddress aPos( nCol, nRow, nTab );
	ScFormulaCell* pFmlaCell = new ScFormulaCell( pDoc, aPos, pTokArr );
	pFmlaCell->SetDouble( fValue );
	pDoc->PutCell( aPos, pFmlaCell );
}



//___________________________________________________________________
// class XclImpSupbookTab

XclImpSupbookTab::~XclImpSupbookTab()
{
	for( XclImpCrnBase* pCrn = _First(); pCrn; pCrn = _Next() )
		delete pCrn;
}

void XclImpSupbookTab::CreateTable( ScDocument* pDoc, const String& rURL )
{
	if( pDoc->LinkEmptyTab( nScTab, ScGlobal::GetDocTabName( rURL, aName ), rURL, aName ) )
	{
		for( XclImpCrnBase* pCrn = _First(); pCrn; pCrn = _Next() )
			pCrn->SetCell( pDoc, nScTab );
	}
	else
		nScTab = EXC_TAB_INVALID;
}



//___________________________________________________________________
// class XclImpSupbook

XclImpSupbook::XclImpSupbook( XclImpStream& rStrm )
{
	UINT16 nTabCnt;
    rStrm >> nTabCnt;

    if( rStrm.GetRecLeft() < (ULONG)(2 + 2 * nTabCnt) )
    {
        bSelf = (rStrm.ReaduInt16() == 0x0401);
        return;
    }

    ReadDocName( rStrm, aFileName, bSelf );

	if( nTabCnt )
	{
		for( UINT16 nTab = 0; nTab < nTabCnt; nTab++ )
		{
			String aTabName;
            ReadTabName( rStrm, aTabName );
			Append( new XclImpSupbookTab( aTabName ) );
		}
	}
	else
		// create dummy list entry
		Append( new XclImpSupbookTab( aFileName ) );
}

XclImpSupbook::~XclImpSupbook()
{
	for( XclImpSupbookTab* pTab = _First(); pTab; pTab = _Next() )
		delete pTab;
}

void XclImpSupbook::AppendCrn( XclImpCrnBase*& rpCrn )
{
	XclImpSupbookTab* pTab = _Get( nCurrExcTab );
	if( pTab )
		pTab->AppendCrn( rpCrn );
	else
		delete rpCrn;
	rpCrn = NULL;
}

//static
void XclImpSupbook::ReadDocName( XclImpStream& rStrm, String& rDocName, BOOL& rbSelf )
{
    String aTabName;
    XclImpHelper::DecodeExternsheetUni( rStrm, rDocName, aTabName, rbSelf );
}

//static
void XclImpSupbook::ReadTabName( XclImpStream& rStrm, String& rTabName )
{
    rStrm.AppendUniString( rTabName );
	ScFilterTools::ConvertName( rTabName );
}

BOOL XclImpSupbook::GetLink( String& rApplic, String& rDoc ) const
{
    // first backslash is application name delimiter
    // (created by XclImpHelper::DecodeExternsheet())
    xub_StrLen nPos = aFileName.Search( '\\' );
    if( bSelf || !aFileName.Len() || (nPos == STRING_NOTFOUND) )
        return FALSE;
    rApplic = aFileName.Copy( 0, nPos );
    rDoc = aFileName.Copy( nPos + 1 );
    return (rApplic.Len() > 0) && (rDoc.Len() > 0);
}

UINT16 XclImpSupbook::GetScTabNum( UINT16 nTab ) const
{
	if( bSelf )
		return nTab;
	const XclImpSupbookTab* pTab = _Get( nTab );
	return pTab ? pTab->GetScTab() : EXC_TAB_INVALID;
}

UINT16 XclImpSupbook::GetScTabNum( const String& rTabName ) const
{
	for( UINT32 nIndex = 0; nIndex < List::Count(); nIndex++ )
	{
		const XclImpSupbookTab* pTab = _Get( nIndex );
		if( pTab && (pTab->GetName() == rTabName) )
			return pTab->GetScTab();
	}
	return EXC_TAB_INVALID;
}

void XclImpSupbook::ReadXct( XclImpStream& rStrm )
{
    rStrm.Ignore( 2 );
    rStrm >> nCurrExcTab;
}

void XclImpSupbook::ReadCrn( XclImpStream& rStrm, ExcelToSc* pFmlConv )
{
    DBG_ASSERT( pFmlConv, "XclImpSupbook::ReadCrn - missing formula converter" );

	UINT8 nLastCol, nFirstCol, nValType;
	UINT16 nRow;
    rStrm >> nLastCol >> nFirstCol >> nRow;

	XclImpCrnBase* pCrn;
    for( USHORT nCol = nFirstCol; (nCol <= nLastCol) && (rStrm.GetRecLeft() > 1); nCol++ )
	{
        rStrm >> nValType;
		switch( nValType )
		{
			case EXC_CRN_DOUBLE:
                pCrn = new XclImpCrnDouble( nCol, nRow, rStrm.ReadDouble() );
			break;
			case EXC_CRN_STRING:
                pCrn = new XclImpCrnString( nCol, nRow, rStrm.ReadUniString() );
			break;
			case EXC_CRN_BOOL:
			case EXC_CRN_ERROR:
			{
				BOOL bIsErr = (nValType == EXC_CRN_ERROR);
				UINT16 nErrBool;
				double fVal;
                rStrm >> nErrBool;
                rStrm.Ignore( 6 );

                const ScTokenArray* pTok = pFmlConv->GetBoolErr(
                    XclImpHelper::ErrorToEnum( bIsErr, (UINT8)nErrBool, fVal ) );
				pCrn = new XclImpCrnFormula( nCol, nRow, fVal, pTok );
			}
			break;
			default:
				DBG_ERROR( "ImportExcel8::Crn - unknown data type" );
				pCrn = NULL;
		}
		if( pCrn )
            AppendCrn( pCrn );
	}
}

void XclImpSupbook::ReadExternname( XclImpStream& rStrm, RootData& rRootData )
{
    UINT32 nStorageId;
    UINT16 nFlags;
    UINT8 nLen;

    rStrm >> nFlags >> nStorageId >> nLen;
    String aName( rStrm.ReadUniString( nLen ) );

    if( (nFlags & EXC_EXTN_BUILTIN) || !(nFlags & 0xFFFE) )
    {
		ScFilterTools::ConvertName( aName );
        aExtNameBuffer.AddName( aName, rRootData );
    }
    else if( nFlags & EXC_EXTN_OLE )
        aExtNameBuffer.AddOLE( aName, nStorageId );
    else
        aExtNameBuffer.AddDDE( aName );
}

void XclImpSupbook::CreateTables( RootData& rRootData, UINT16 nFirst, UINT16 nLast ) const
{
    if( bSelf || (rRootData.pExtDocOpt->nLinkCnt >= 1) )
		return;

	String aURL( ScGlobal::GetAbsDocName( aFileName, rRootData.pDoc->GetDocumentShell() ) );

	for( UINT16 nTab = nFirst; nTab <= nLast; nTab++ )
	{
		XclImpSupbookTab* pSBTab = _Get( nTab );
		if( pSBTab )
			pSBTab->CreateTable( rRootData.pDoc, aURL );
	}
}



//___________________________________________________________________
// class XclImpSupbookBuffer

XclImpSupbookBuffer::~XclImpSupbookBuffer()
{
	for( XclImpSupbook* pSupbook = _First(); pSupbook; pSupbook = _Next() )
		delete pSupbook;
}

const XclImpSupbook* XclImpSupbookBuffer::Get( const String& rDocName ) const
{
	for( UINT32 nIndex = 0; nIndex < List::Count(); nIndex++ )
	{
		const XclImpSupbook* pSupbook = Get( nIndex );
		if( pSupbook && (pSupbook->GetName() == rDocName) )
			return pSupbook;
	}
	return NULL;
}



//___________________________________________________________________
// class XclImpExternsheetBuffer

XclImpExternsheetBuffer::~XclImpExternsheetBuffer()
{
	for( XclImpXti* pXti = _First(); pXti; pXti = _Next() )
		delete pXti;
}

BOOL XclImpExternsheetBuffer::FindNextTabRange( UINT16 nSupb, UINT16 nStart, UINT16& rnFirst, UINT16& rnLast )
{
	rnFirst = rnLast = 0xFFFF;
	for( const XclImpXti* pXti = _First(); pXti; pXti = _Next() )
	{
		if( (nSupb == pXti->nSupbook) && (nStart <= pXti->nLast) && (pXti->nFirst < rnFirst) )
		{
			rnFirst = Max( nStart, pXti->nFirst );
			rnLast = pXti->nLast;
		}
	}
	return (rnFirst < 0xFFFF);
}

void XclImpExternsheetBuffer::Read( XclImpStream& rStrm )
{
	UINT16 nXtiCount;
    rStrm >> nXtiCount;

	XclImpXti* pXti;
	while( nXtiCount )
	{
		pXti = new XclImpXti;
        rStrm >> pXti->nSupbook >> pXti->nFirst >> pXti->nLast;
		List::Insert( pXti, LIST_APPEND );
		nXtiCount--;
	}
}

void XclImpExternsheetBuffer::CreateTables( RootData& rRootData )
{
	DBG_ASSERT( !bCreated, "XclImpExternsheetBuffer::CreateTables - multiple call!!" );
	if( bCreated ) return;

	UINT16 nFirst, nLast;

	for( UINT16 nSupbook = 0; nSupbook < aSupbookBuffer.Count(); nSupbook++ )
	{
		const XclImpSupbook* pSupbook = aSupbookBuffer.Get( nSupbook );
		BOOL bLoop = FindNextTabRange( nSupbook, 0, nFirst, nLast );
		while( bLoop && pSupbook )
		{
			pSupbook->CreateTables( rRootData, nFirst, nLast );
			bLoop = FindNextTabRange( nSupbook, nLast + 1, nFirst, nLast );
		}
	}

	bCreated = TRUE;
}

const XclImpSupbook* XclImpExternsheetBuffer::GetSupbook( ULONG nXtiIndex ) const
{
	const XclImpXti* pXti = GetXti( nXtiIndex );
	return pXti ? aSupbookBuffer.Get( pXti->nSupbook ) : NULL;
}


