/*************************************************************************
 *
 *  $RCSfile: XclImpHelper.cxx,v $
 *
 *  $Revision: 1.8 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/30 11:24:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//___________________________________________________________________

#ifndef _SC_XCLIMPHELPER_HXX
#include "XclImpHelper.hxx"
#endif

#ifndef _UNOTOOLS_CHARCLASS_HXX
#include <unotools/charclass.hxx>
#endif

#ifndef SC_DOCUMENT_HXX
#include "document.hxx"
#endif
#ifndef SC_SCGLOB_HXX
#include "global.hxx"
#endif

#ifndef _EXCDEFS_HXX
#include "excdefs.hxx"
#endif
#ifndef _EXCSST_HXX
#include "excsst.hxx"
#endif

//___________________________________________________________________
// class XclImpHelper

double XclImpHelper::GetDoubleFromRK( UINT32 nRKValue )
{
	double	fVal;

	// code from book Excel-Developer's-Kit V5 (p.223)
	if( nRKValue & 0x02 )
		fVal = ( double ) ( *( ( INT32 * ) &nRKValue ) >> 2 );		// integer
	else
	{// 64-Bit IEEE-Float
#ifdef __BIGENDIAN
		*((UINT32*) &fVal + 1) = 0;						// lower 32 bits = 0
		*((UINT32*) &fVal) = nRKValue & 0xFFFFFFFC;		// bit 0, 1 = 0
#else
		*((UINT32*) &fVal) = 0;							// lower 32 bits = 0
		*((UINT32*) &fVal + 1) = nRKValue & 0xFFFFFFFC;	// bit 0, 1 = 0
#endif
	}

	if( nRKValue & 0x01 )
		fVal *= 0.01;

	return fVal;
}

sal_Int32 XclImpHelper::GetRotation( UINT16 nExcRot )
{
	if( nExcRot > 180 )
		return 27000;
	return (sal_Int32)(100 * ((nExcRot > 90) ? 450 - nExcRot : nExcRot));
}

BoolError XclImpHelper::ErrorToEnum( BYTE bErrOrVal, BYTE nError, double& rfVal )
{
    BoolError eType;
    if( bErrOrVal )
    {// Error Value
        switch( nError )
        {
            case 0x00:  eType = BE_NULL;    break;
            case 0x07:  eType = BE_DIV0;    break;
            case 0x0F:  eType = BE_VALUE;   break;
            case 0x17:  eType = BE_REF;     break;
            case 0x1D:  eType = BE_NAME;    break;
            case 0x24:  eType = BE_NUM;     break;
            case 0x2A:  eType = BE_NA;      break;
            default:    eType = BE_UNKNOWN; break;
        }
        rfVal = 0.0;
    }
    else
    {// Boolean Value
        if( nError )
        {
            eType = BE_TRUE;
            rfVal = 1.0;
        }
        else
        {
            eType = BE_FALSE;
            rfVal = 0.0;
        }
    }
    return eType;
}


long XclImpHelper::CalcX( UINT16 nT, UINT16 nC, UINT16 nOC, double f, ScDocument* pDoc )
{
	return ( long ) (
		( ( double ) pDoc->GetColOffset( nC, nT )
			+ ( double ) pDoc->GetColWidth( nC, nT ) * ( ( double ) Min( nOC, (UINT16) 1024 ) / 1024.0 ) )
		* f );
}

long XclImpHelper::CalcY( UINT16 nT, UINT16 nR, UINT16 nOR, double f, ScDocument* pDoc )
{
	return ( long ) (
		( ( double ) pDoc->GetRowOffset( nR, nT )
			+ ( double ) pDoc->GetRowHeight( nR, nT ) * ( ( double ) Min( nOR, (UINT16) 256 ) / 256.0 ) )
		* f );
}

ShStrTabEntry* XclImpHelper::CreateUnicodeEntry( XclImpStream& rStrm, UINT16 nChars )
{
	ShStrTabEntry* pRet;

	UINT8 nFlags = rStrm.ReaduInt8();
	BOOL b16Bit, bRich, bFarEast;
	UINT16 nCrun;
	UINT32 nExtInf;

    rStrm.ReadUniStringExtHeader( b16Bit, bRich, bFarEast, nCrun, nExtInf, nFlags );
    String sString( rStrm.ReadRawUniString( nChars, b16Bit ) );

	pRet = bRich ? new ShStrTabFormEntry( sString, rStrm, nCrun ) : new ShStrTabEntry( sString );

	rStrm.Ignore( nExtInf );
	return pRet;
}

UINT16 XclImpHelper::ReadExternsheetChar( XclImpStream& rStrm, UINT16& nCharsLeft, BOOL b16Bit )
{
	nCharsLeft--;
	return b16Bit ? rStrm.ReaduInt16() : (UINT16) rStrm.ReaduInt8();
}

void XclImpHelper::DecodeExternsheetImpl(
        XclImpStream& rStrm, String& rFile, String& rTable,
        BOOL& rbSameWb, UINT16 nChars, BOOL b16Bit )
{
    enum State { S_INIT, S_PATH, S_FILENAME, S_TABNAME, S_READRAW };

	UINT16	cEnc;
	State	eState = S_INIT;
	UINT16	nCharsLeft = nChars;
    BOOL    bEncoded = TRUE;

	rbSameWb = FALSE;

	while( nCharsLeft )
	{
		cEnc = ReadExternsheetChar( rStrm, nCharsLeft, b16Bit );
		switch( eState )
		{
			case S_INIT:
			{
				switch( cEnc )
				{
					case 0x0000:								// empty
					case 0x0001:						break;	// encode
					case 0x0002:								// self
					case 0x0003:	rbSameWb = TRUE;	break;	// encode & self
                    default:
                        bEncoded = FALSE;
                        rFile += (sal_Unicode) cEnc;
				}
                eState = S_PATH;
			}
			break;
			case S_PATH:
			{
				switch( cEnc )
				{
					case 0x0001:
					{
						if( nCharsLeft > 1 )
						{
							cEnc = ReadExternsheetChar( rStrm, nCharsLeft, b16Bit );
							if( cEnc == '@' )
                                rFile.AppendAscii( "\\\\" );
							else
							{
								rFile += (sal_Unicode) cEnc;
								rFile.AppendAscii( ":\\" );
							}
						}
						else
							rFile.AppendAscii( "<NULL-DRIVE!>" );
					}
					break;
					case 0x0002:
                    case 0x0003:
                        rFile.AppendAscii( "\\" );
                        if( !bEncoded )
                            eState = S_READRAW;             // was DDE delimiter
                    break;
					case 0x0004:	rFile.AppendAscii( "..\\") ;	break;
					case 0x0005:
					{
						if( nCharsLeft )
						{
							UINT16 nVolLen = ReadExternsheetChar( rStrm, nCharsLeft, b16Bit );
							if( nVolLen )
							{
								while( nVolLen-- )
								{
									cEnc = ReadExternsheetChar( rStrm, nCharsLeft, b16Bit );
									rFile += (sal_Unicode) cEnc;
								}
							}
							else
								rFile.AppendAscii( "<EMPTY MAC-LONG-VOLUME>" );

							rFile.AppendAscii( ":" );
						}
						else
							rFile.AppendAscii( "<ERROR IN MAC-LONG-VOLUME>" );
					}
					break;
					case 0x0006:	rFile.AppendAscii( "<Startup Dir>:" );		break;
					case 0x0007:	rFile.AppendAscii( "<Alt Startup Dir>:" );	break;
					case 0x0008:	rFile.AppendAscii( "<Library>" );			break;
					case '[':		eState = S_FILENAME;						break;	// start of file name
					default:		rFile += (sal_Unicode) cEnc;
				}
			}
			break;
			case S_FILENAME:
			{
				switch( cEnc )
				{
					case ']':	eState = S_TABNAME;				break;	// end of file name
					default:	rFile += (sal_Unicode) cEnc;
				}
			}
			break;
			case S_TABNAME:		rTable += (sal_Unicode) cEnc;	break;
            case S_READRAW:     rFile += (sal_Unicode) cEnc;    break;
		}
	}
}

void XclImpHelper::DecodeExternsheetUni(
        XclImpStream& rStrm, String& rFile, String& rTable,
        BOOL& rbSameWb, UINT16 nChars, UINT8 nFlags )
{
	rStrm.PushPosition();
	BOOL b16Bit;
    rStrm.ReadUniStringExtHeader( b16Bit, nFlags );
    DecodeExternsheetImpl( rStrm, rFile, rTable, rbSameWb, nChars, b16Bit );
	rStrm.PopPosition();
	rStrm.IgnoreUniString( nChars, nFlags );
}

