/*************************************************************************
 *
 *  $RCSfile: XclExpHelper.hxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/30 12:39:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SC_XCLEXPHELPER_HXX
#define _SC_XCLEXPHELPER_HXX

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#ifndef _SC_XCLEXPSTREAM_HXX
#include "XclExpStream.hxx"
#endif
#ifndef _EXCDEFS_HXX
#include "excdefs.hxx"
#endif

class ScDocument;
class ScMatrix;
class XclExpStream;
class ExcRichStr;

//___________________________________________________________________
// static helper routines for Excel export

class XclExpHelper
{
public:
	static BOOL					GetRKFromDouble( double fValue, INT32& rRKValue );
	static UINT16				GetRotation( long nRot );

								// write as 8 bit/16 bit
	static void					WriteUnicodeBuffer(
									SvStream& rStrm,
									const UINT16* pBuffer,
									ULONG nChars,
									BOOL b16Bit );
};


//___________________________________________________________________
// stores a byte string or unicode string

class XclExpUniString
{
private:
	UINT16*						pBuffer;
	UINT16						nLen;
	BOOL						b16Bit;

	void						Build( const sal_Unicode* pSource, UINT16 nCount, BOOL bForce16Bit = FALSE );

	void						PrepareWrite( XclExpStream& rStrm, ULONG nBytes ) const;

protected:
	void						WriteRawHeader(
									XclExpStream& rStrm,
									ULONG nAddLen,
									BOOL b16BitCount,
									BOOL bForceFlags ) const;

public:
								XclExpUniString();
								XclExpUniString( const XclExpUniString& rCopy );

								XclExpUniString(
									const String& rString,
									UINT16 nMaxLen = 0xFFFF,
									BOOL bForce16Bit = FALSE );
								XclExpUniString(
									const ::rtl::OUString& rString,
									UINT16 nMaxLen = 0xFFFF,
									BOOL bForce16Bit = FALSE );

	virtual						~XclExpUniString();

	void						Assign(
									const String& rString,
									UINT16 nMaxLen = 0xFFFF,
									BOOL bForce16Bit = FALSE );
	void						Assign(
									const ::rtl::OUString& rString,
									UINT16 nMaxLen = 0xFFFF,
									BOOL bForce16Bit = FALSE );

	inline UINT16				GetLen() const;				// character count
	virtual UINT8				GetGrbit() const;			// Excel grbit field

	inline ULONG				GetBufferByteCount() const;	// byte count of character buffer
	virtual ULONG				GetByteCount() const;		// byte count of entire string (assuming 3 byte header!)

								// write buffer to memory (always little endian or 8 bit)
	void						WriteBuffer( void* pDest ) const;

								// write 8 bit flags only
	virtual void				WriteFlags( XclExpStream& rStrm ) const;
								// write 8/16 bit char count, 8 bit flags
	virtual void				WriteHeader(
									XclExpStream& rStrm,
									BOOL b16BitCount = TRUE,
									BOOL bForceFlags = TRUE ) const;
								// write characters only
	virtual void				WriteBuffer( XclExpStream& rStrm ) const;

								// write 8/16 bit length, 8 bit flags, characters
	virtual void				Write(
									XclExpStream& rStrm,
									BOOL b16BitCount = TRUE,
									BOOL bForceFlags = TRUE ) const;
};

inline UINT16 XclExpUniString::GetLen() const
{
	return nLen;
}

inline ULONG XclExpUniString::GetBufferByteCount() const
{
	return GetLen() * (b16Bit ? 2 : 1);
}


//___________________________________________________________________
// stores a XclExpUniString with format infos

class XclExpRichString : public XclExpUniString
{
private:
	ExcRichStr*					pRichData;		// format infos

public:
								XclExpRichString();
								XclExpRichString( const XclExpRichString& rCopy );

								XclExpRichString(
									const String& rString,
									UINT16 nMaxLen = 0xFFFF,
									BOOL bForce16Bit = FALSE );
								XclExpRichString(
									const ::rtl::OUString& rString,
									UINT16 nMaxLen = 0xFFFF,
									BOOL bForce16Bit = FALSE );

	virtual						~XclExpRichString();

	void						SetRichData( ExcRichStr* pNewData );

	virtual UINT8				GetGrbit() const;			// Excel grbit field
	virtual ULONG				GetByteCount() const;		// byte count of entire string + format info

								// write 8/16 bit char count, 8 bit flags
	virtual void				WriteHeader(
									XclExpStream& rStrm,
									BOOL b16BitCount = TRUE,
									BOOL bForceFlags = TRUE ) const;

								// write 8/16 bit length, 8 bit flags, characters
	virtual void				Write(
									XclExpStream& rStrm,
									BOOL b16BitCount = TRUE,
									BOOL bForceFlags = TRUE ) const;
};


//___________________________________________________________________
// list of cached double values or strings (for EXTERNNAME, ptgArray, ...)
// (1 byte) column count - 1
// (2 byte) row count - 1
// (x byte) list of values (double or string)
// - structure of double value: (XclExpCachedValueDbl)
//   (1 byte) 0x01 identifier
//   (8 byte) double
// - structure of string value: (XclExpCachedValueStr)
//   (1 byte) 0x02 identifier
//   (1 byte) or (2 byte) length
//   (1 byte) unicode flags (always!)
//   (x byte) character array

class XclExpCachedValue
{
public:
    virtual                     ~XclExpCachedValue();
    virtual void                Save( XclExpStream& rStrm ) = 0;
};


class XclExpCachedValueDbl : public XclExpCachedValue
{
private:
    double                      fVal;
public:
    inline                      XclExpCachedValueDbl( double _fVal ) : fVal( _fVal ) {}
    virtual void                Save( XclExpStream& rStrm );
};


class XclExpCachedValueStr : public XclExpCachedValue
{
private:
    XclExpUniString             aStr;
    BOOL                        b16BitCnt;
public:
                                XclExpCachedValueStr( const String& rStr, BOOL b16BitCount );
    inline ULONG                GetLen();
    virtual void                Save( XclExpStream& rStrm );
};

inline ULONG XclExpCachedValueStr::GetLen()
{
    return aStr.GetBufferByteCount() + 3 + (b16BitCnt ? 1 : 0);
}



class XclExpCachedValueList : private List
{
private:
    ULONG                       nLen;           // record size
    USHORT                      nCols;          // matrix size
    USHORT                      nRows;          // matrix size
    BOOL                        b16BitCnt;      // string length write mode

    inline XclExpCachedValue*   _First()    { return (XclExpCachedValue*) List::First(); }
    inline XclExpCachedValue*   _Next()     { return (XclExpCachedValue*) List::Next(); }

    void                        Append( double fVal );
    void                        Append( const String& rStr );

public:
                                XclExpCachedValueList(
                                    ScDocument& rDoc,
                                    USHORT _nCols, USHORT _nRows,
                                    const ScMatrix* pMatrix,
                                    BOOL b16BitCount );
    virtual                     ~XclExpCachedValueList();

    inline ULONG                GetLen() const  { return nLen; }
    void                        Save( XclExpStream& rStrm );
};


//___________________________________________________________________

#endif

