/*************************************************************************
*
*  $RCSfile: XclImpHelper.hxx,v $
*
*  $Revision: 1.7 $
*
*  last change: $Author: dr $ $Date: 2001/07/30 11:25:46 $
*
*  The Contents of this file are made available subject to the terms of
*  either of the following licenses
*
*         - GNU Lesser General Public License Version 2.1
*         - Sun Industry Standards Source License Version 1.1
*
*  Sun Microsystems Inc., October, 2000
*
*  GNU Lesser General Public License Version 2.1
*  =============================================
*  Copyright 2000 by Sun Microsystems, Inc.
*  901 San Antonio Road, Palo Alto, CA 94303, USA
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Lesser General Public
*  License version 2.1, as published by the Free Software Foundation.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with this library; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA  02111-1307  USA
*
*
*  Sun Industry Standards Source License Version 1.1
*  =================================================
*  The contents of this file are subject to the Sun Industry Standards
*  Source License Version 1.1 (the "License"); You may not use this file
*  except in compliance with the License. You may obtain a copy of the
*  License at http://www.openoffice.org/license.html.
*
*  Software provided under this License is provided on an "AS IS" basis,
*  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
*  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
*  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
*  See the License for the specific provisions governing your rights and
*  obligations concerning the Software.
*
*  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
*
*  Copyright: 2000 by Sun Microsystems, Inc.
*
*  All Rights Reserved.
*
*  Contributor(s): _______________________________________
*
*
************************************************************************/

#ifndef _SC_XCLIMPHELPER_HXX
#define _SC_XCLIMPHELPER_HXX

//___________________________________________________________________

#ifndef _SC_XCLIMPSTREAM_HXX
#include "XclImpStream.hxx"
#endif

#ifndef _FORMEL_HXX
#include "formel.hxx"
#endif

class ScDocument;
class ShStrTabEntry;

//___________________________________________________________________
// static helper routines for Excel import

class XclImpHelper
{
private:
                                // read a 8 or 16 bit char
    static UINT16               ReadExternsheetChar( XclImpStream& rStrm, UINT16& nCharsLeft, BOOL b16Bit );
                                // read encoded externsheet name
    static void                 DecodeExternsheetImpl(
                                    XclImpStream& rStrm,
                                    String& rFile,
                                    String& rTable,
                                    BOOL& rbSameWb,
                                    UINT16 nChars,
                                    BOOL b16Bit );

public:
    // *** numeric operations ***

    static double               GetDoubleFromRK( UINT32 nRKValue );
    static sal_Int32            GetRotation( UINT16 nExcRot );
    static BoolError            ErrorToEnum( BYTE bErrOrVal, BYTE nError, double& rfVal );

                                // Excel-Anchor -> Calc-Twips
    static long                 CalcX( UINT16 nT, UINT16 nC, UINT16 nOC, double f, ScDocument* pDoc );
    static long                 CalcY( UINT16 nT, UINT16 nR, UINT16 nOR, double f, ScDocument* pDoc );

    // *** string operations ***

                                // create (formatted) shared string table entry
    static ShStrTabEntry*       CreateUnicodeEntry( XclImpStream& rStrm, UINT16 nChars );
    static inline ShStrTabEntry* CreateUnicodeEntry( XclImpStream& rStrm )
                                    { return CreateUnicodeEntry( rStrm, rStrm.ReaduInt16() ); }

                                // read encoded externsheet name (unicode)
    static void                 DecodeExternsheetUni(
                                    XclImpStream& rStrm,
                                    String& rFile,
                                    String& rTable,
                                    BOOL& rbSameWb,
                                    UINT16 nChars,
                                    UINT8 nFlags );
    static inline void          DecodeExternsheetUni(
                                    XclImpStream& rStrm,
                                    String& rFile,
                                    String& rTable,
                                    BOOL& rbSameWb,
                                    UINT16 nChars );
    static inline void          DecodeExternsheetUni(
                                    XclImpStream& rStrm,
                                    String& rFile,
                                    String& rTable,
                                    BOOL& rbSameWb );

                                // read encoded externsheet name (byte string)
    static inline void          DecodeExternsheetByte(
                                    XclImpStream& rStrm,
                                    String& rFile,
                                    String& rTable,
                                    BOOL& rbSameWb );
};

inline void XclImpHelper::DecodeExternsheetUni(
        XclImpStream& rStrm, String& rFile, String& rTable,
        BOOL& rbSameWb, UINT16 nChars )
{
    DecodeExternsheetUni( rStrm, rFile, rTable, rbSameWb, nChars, rStrm.ReaduInt8() );
}

inline void XclImpHelper::DecodeExternsheetUni(
        XclImpStream& rStrm, String& rFile, String& rTable, BOOL& rbSameWb )
{
    DecodeExternsheetUni( rStrm, rFile, rTable, rbSameWb, rStrm.ReaduInt16() );
}

inline void XclImpHelper::DecodeExternsheetByte(
        XclImpStream& rStrm, String& rFile, String& rTable, BOOL& rbSameWb )
{
    DecodeExternsheetImpl( rStrm, rFile, rTable, rbSameWb, rStrm.ReaduInt8(), FALSE );
}

//___________________________________________________________________

#endif

