/*************************************************************************
 *
 *  $RCSfile: XclImpObjects.hxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/24 13:51:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _SC_XCLIMPOBJECTS_HXX
#define _SC_XCLIMPOBJECTS_HXX

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
#ifndef _MSDFFIMP_HXX
#include <svx/msdffimp.hxx>
#endif

#ifndef _EXCDEFS_HXX
#include "excdefs.hxx"
#endif
#ifndef _ROOT_HXX
#include "root.hxx"
#endif

class XclImpStream;
class ExcChart;
class Biff8MSDffManager;


//___________________________________________________________________

struct XclImpStreamNode
{
    sal_uInt32                  nPos;
    sal_uInt32                  nSize;
    XclImpStreamNode*           pPrev;
};


//___________________________________________________________________

class XclImpStreamConsumer
{
private:
    SvMemoryStream              aStrm;
    DffRecordHeader             aHd;
    XclImpStreamNode*           pNode;
    sal_uInt32                  nBytesLeft;

    void                        UpdateNode( const DffRecordHeader& rHd );
    void                        RemoveNode();

public:
                                XclImpStreamConsumer();
                                ~XclImpStreamConsumer();

    const DffRecordHeader*      ConsumeRecord( XclImpStream& rSrcStrm );
    sal_Bool                    AppendData( sal_Char* pBuf, sal_uInt32 nLen );

    inline sal_Bool             HasData() const { return aStrm.Tell() > 0; }
    inline ULONG                Tell() const    { return aStrm.Tell(); }
    inline SvStream&            GetStream()     { return aStrm; }
};


//___________________________________________________________________
// object type

enum XclImpObjectType
{
    otUnknown = 0,
    otChart,
    otPicture,
    otTxo,
    otNote,
    otOle,
    otCtrl
};


//___________________________________________________________________
// base class for Escher objects

class XclImpEscherObj : public ExcRoot
{
protected:
	Rectangle*					pAnchor;
	SdrObject*					pSdrObj;
    XclImpObjectType            eType;
    ULONG                       nStrmStart;
    ULONG                       nStrmEnd;
	UINT16						nTab;
	UINT16						nId;

public:
                                XclImpEscherObj(
                                    ULONG _nStrmStart, ULONG _nStrmEnd,
                                    RootData& rRootData,
                                    XclImpObjectType _eType = otUnknown );
                                // rCopy.pSdrObj is copied to this and becomes NULL!
                                XclImpEscherObj(
                                    XclImpEscherObj& rCopy,
                                    XclImpObjectType _eType = otUnknown );

    virtual                     ~XclImpEscherObj();

    void                        SetAnchor( const Rectangle& rAnchor );
    virtual void                SetSdrObj( SdrObject* pNewSdrObj );
    inline void                 SetType( XclImpObjectType _eType )  { eType = _eType; }
    inline void                 SetId( UINT16 nNewId )              { nId = nNewId; }

    inline const Rectangle*     GetAnchor() const           { return pAnchor; }
    inline const SdrObject*     GetSdrObj() const           { return pSdrObj; }
    inline XclImpObjectType     GetObjType() const          { return eType; }
    inline ULONG                GetStrmStart() const        { return nStrmStart; }
    inline ULONG                GetStrmEnd() const          { return nStrmEnd; }
    inline UINT16               GetTab() const              { return nTab; }
    inline UINT16               GetId() const               { return nId; }

    virtual void                Apply();

    void                        MorpheFrom( XclImpEscherObj*& rpMAD );
};


//___________________________________________________________________

class XclImpEscherChart : public XclImpEscherObj
{
protected:
	ExcChart*					pChrtData;

public:
                                XclImpEscherChart( XclImpEscherObj*& rpCAD );
    virtual                     ~XclImpEscherChart();

    inline ExcChart*            GetChartData()      { return pChrtData; }
    void                        SetChartData( ExcChart* pNewChart );

    virtual void                Apply();
};


//___________________________________________________________________

class XclImpEscherOle : public XclImpEscherObj
{
private:
	String						aStorageName;
	UINT32						nBlipId;
	BOOL						bAsSymbol	: 1;
	BOOL						bLinked		: 1;

public:
                                XclImpEscherOle( XclImpEscherObj*& rpCAD );
    virtual                     ~XclImpEscherOle();

    virtual void                Apply();

    inline void                 SetAsSymbol( BOOL bVal )    { bAsSymbol = (bVal != 0); }
    inline void                 SetLinked( BOOL bVal )      { bLinked = (bVal != 0); }
    inline void                 SetBlipId( UINT32 nVal )    { nBlipId = nVal; }

    void                        ReadPictFmla( XclImpStream& rIn, UINT16 nLen );
    void                        CreateSdrOle( Biff8MSDffManager& rDffMan, UINT32 nOLEImpFlags = 0 );
};


//___________________________________________________________________

class XclImpEscherDrwObj : public XclImpEscherObj
{
public:
                                XclImpEscherDrwObj( XclImpEscherObj*& rpCAD );
    virtual                     ~XclImpEscherDrwObj();
};


//___________________________________________________________________

class XclImpEscherTxo : public XclImpEscherObj
{
protected:
	String*						pText;
	EditTextObject*				pFormText;
	BOOL						bIsApplied;

    void                        ApplyTextOnObject( SdrObject* pTextObj = NULL );

public:
                                XclImpEscherTxo( XclImpEscherObj*& rpCAD );
    virtual                     ~XclImpEscherTxo();

    virtual void                SetSdrObj( SdrObject* pNewSdrObj );

    void                        SetText( const String& rText );
    inline const String*        GetText() const     { return pText; }

    void                        TakeText( String* pNewText, EditTextObject* pNewFormText );

    virtual void                Apply();
    void                        Apply( SdrObject* pTextObj );
								// also set bIsApplied, but doesn't put on drawing layer!
};


//___________________________________________________________________

class XclImpEscherNote : public XclImpEscherTxo
{
public:
                                XclImpEscherNote( XclImpEscherObj*& rpCAP );

    virtual void                Apply();
};


//___________________________________________________________________
// client anchor data

struct XclImpAnchorData
{
	UINT32						nObjNum;
	UINT16						nTab;

    UINT16                      nCol;
	UINT16						nX;
	UINT16						nRow;
	UINT16						nY;
	UINT16						nDCol;
	UINT16						nDX;
	UINT16						nDRow;
	UINT16						nDY;

    inline                      XclImpAnchorData();
    inline                      XclImpAnchorData( const XclImpAnchorData& rCopy );
};

inline XclImpAnchorData::XclImpAnchorData()
{
    memset( this, 0x00, sizeof( XclImpAnchorData ) );
}

inline XclImpAnchorData::XclImpAnchorData( const XclImpAnchorData& rCopy )
{
    memcpy( this, &rCopy, sizeof( XclImpAnchorData ) );
}

SvStream& operator>>( SvStream& rStrm, XclImpAnchorData& rAnchor );


//___________________________________________________________________
// stores Escher stream position and client anchor data of an object

class XclImpObjData
{
public:
    XclImpEscherObj*            pEscherObj;
    XclImpAnchorData*           pAnchorData;
    ULONG                       nStart;
    ULONG                       nEnd;
    UINT32                      nObjNum;
    UINT16                      nTabNum;

    inline                      XclImpObjData(
                                    XclImpEscherObj* pObj,
                                    UINT32 _nObjNum );
    inline                      ~XclImpObjData();

    inline XclImpAnchorData*    GetAnchorDataAccess();
    inline BOOL                 IsInRange( ULONG nStrmPos ) const;
};

inline XclImpObjData::XclImpObjData( XclImpEscherObj* pObj, UINT32 _nObjNum ) :
    pEscherObj( pObj ),
    pAnchorData( NULL ),
    nObjNum( _nObjNum ),
    nStart( pObj->GetStrmStart() ),
    nEnd( pObj->GetStrmEnd() ),
    nTabNum( pObj->GetTab() )
{
}

inline XclImpObjData::~XclImpObjData()
{
    if( pEscherObj )
        delete pEscherObj;
    if( pAnchorData )
        delete pAnchorData;
}

inline XclImpAnchorData* XclImpObjData::GetAnchorDataAccess()
{
    if( !pAnchorData )
        pAnchorData = new XclImpAnchorData;
    return pAnchorData;
}

inline BOOL XclImpObjData::IsInRange( ULONG nStrmPos ) const
{
    return (nStrmPos >= nStart) && (nStrmPos <= nEnd);
}


//___________________________________________________________________
// list of all read Escher objects

class XclImpEscherObjList : protected List, protected ExcRoot
{
private:
	UINT32						nLastReqTabStart;
	UINT16						nLastReqTab;

    inline XclImpObjData*       _First()    { return (XclImpObjData*) List::First(); }
    inline XclImpObjData*       _Next()     { return (XclImpObjData*) List::Next(); }
    inline XclImpObjData*       _Last()     { return (XclImpObjData*) List::Last(); }
    inline XclImpObjData*       _Get( ULONG nIndex ) const
                                    { return (XclImpObjData*) List::GetObject( nIndex ); }

    XclImpObjData*              GetFromStream( ULONG nStrmPos ) const;

    BOOL                        SetTabStart( UINT16 nNewTab );

public:
                                XclImpEscherObjList( RootData& rRootData );
    virtual                     ~XclImpEscherObjList();

    void                        Append( XclImpEscherObj* pObj );
    void                        MorpheLastObj( XclImpEscherObj* pObj );

    BOOL                        GetObjNum( ULONG nStrmPos, UINT32& rObjNum ) const;

    XclImpAnchorData*           GetAnchorDataAccess( ULONG nStrmPos );
    const XclImpAnchorData*     GetAnchorData( ULONG nStrmPos ) const;

    void                        SetData( UINT32 nObjNum, SdrObject* pDrawObj );
    void                        SetData( UINT32 nObjNum, const Rectangle& rAnchor );
    void                        SetData( UINT32 nObjNum, const Rectangle& rAnchor, SdrObject* pDrawObj );

    inline                      GetObjCount() const     { return List::Count(); }
    XclImpEscherObj*            GetObj( UINT32 nObjNum, UINT16 nTab = EXC_TAB_INVALID ) const;
    XclImpEscherObj*            GetObjFromStream( ULONG nStrmPos );

    void                        Apply();
};


//___________________________________________________________________
// stores all Escher objects and additional data

class XclImpObjectManager : private ExcRoot
{
private:
    XclImpStreamConsumer        aStreamConsumer;
    XclImpEscherObjList         aEscherObjList;

    XclImpEscherObj*            pCurrEscherObj;

    BOOL                        bLeadingTxo;
    BOOL                        bMaybeTxo;
    BOOL                        bStartWithDummy;

    static inline BOOL          IsType( const XclImpEscherObj* pObj, XclImpObjectType eType );

                                // OBJ subrecords
    XclImpEscherObj*            ReadObjFtCmo( XclImpStream& rStrm );
    void                        ReadObjFtPioGrbit( XclImpStream& rStrm, XclImpEscherObj* pObj );
    void                        ReadObjFtPictFmla( XclImpStream& rStrm, XclImpEscherObj* pObj, UINT16 nLen );

public:
                                XclImpObjectManager( RootData& rRootData );
                                ~XclImpObjectManager();

    inline BOOL                 HasEscherStream()   { return aStreamConsumer.HasData(); }
    inline SvStream&            GetEscherStream()   { return aStreamConsumer.GetStream(); }

    inline UINT32               GetObjCount() const;
    inline const XclImpEscherObj* GetObj( UINT32 nObjNum, UINT16 nTab ) const;
    const XclImpEscherNote*     GetObjNote( UINT32 nObjNum, UINT16 nTab ) const;

    inline XclImpEscherObj*     GetObjFromStream( ULONG nStrmPos );
    XclImpEscherTxo*            GetTxoFromStream( ULONG nStrmPos );

    inline XclImpAnchorData*    GetAnchorDataAccess( ULONG nStrmPos );
    inline const XclImpAnchorData* GetAnchorData( ULONG nStrmPos ) const;

    inline XclImpEscherObj*     GetCurrObj() const  { return pCurrEscherObj; }
    inline XclImpEscherChart*   GetCurrObjChart() const;

    void                        SetCurrObjChart();
    void                        AppendCurrObjToList();

    inline void                 SetCurrChartData( ExcChart* pNewData );
    inline ExcChart*            GetCurrChartData() const;

    void                        ReadMsodrawinggroup( XclImpStream& rStrm );
    void                        ReadMsodrawing( XclImpStream& rStrm );
    void                        ReadMsodrawingselection( XclImpStream& rStrm );

    void                        ReadObj( XclImpStream& rStrm );
    void                        ReadTxo( XclImpStream& rStrm );

    inline void                 InsertDummyObj()        { bStartWithDummy = TRUE; }

    inline void                 Apply()     { aEscherObjList.Apply(); }
};

inline BOOL XclImpObjectManager::IsType( const XclImpEscherObj* pObj, XclImpObjectType eType )
{
    return pObj && (pObj->GetObjType() == eType);
}

inline UINT32 XclImpObjectManager::GetObjCount() const
{
    return aEscherObjList.GetObjCount();
}

inline const XclImpEscherObj* XclImpObjectManager::GetObj( UINT32 nObjNum, UINT16 nTab ) const
{
    return aEscherObjList.GetObj( nObjNum, nTab );
}

inline XclImpEscherObj* XclImpObjectManager::GetObjFromStream( ULONG nStrmPos )
{
    return aEscherObjList.GetObjFromStream( nStrmPos );
}

inline XclImpAnchorData* XclImpObjectManager::GetAnchorDataAccess( ULONG nStrmPos )
{
    return aEscherObjList.GetAnchorDataAccess( nStrmPos );
}

inline const XclImpAnchorData* XclImpObjectManager::GetAnchorData( ULONG nStrmPos ) const
{
    return aEscherObjList.GetAnchorData( nStrmPos );
}

inline XclImpEscherChart* XclImpObjectManager::GetCurrObjChart() const
{
    return IsType( pCurrEscherObj, otChart ) ? (XclImpEscherChart*)pCurrEscherObj : NULL;
}

inline void XclImpObjectManager::SetCurrChartData( ExcChart* pNewData )
{
    if( IsType( pCurrEscherObj, otChart ) )
        ((XclImpEscherChart*)pCurrEscherObj)->SetChartData( pNewData );
}

inline ExcChart* XclImpObjectManager::GetCurrChartData() const
{
    return IsType( pCurrEscherObj, otChart ) ? ((XclImpEscherChart*)pCurrEscherObj)->GetChartData() : NULL;
}



//___________________________________________________________________

class Biff8MSDffManager : public SvxMSDffManager, protected ExcRoot
{
private:
    XclImpObjectManager&        rObjManager;

protected:
    virtual void                ProcessClientAnchor2( SvStream&, DffRecordHeader&, void*, DffObjData& );
    virtual SdrObject*          ProcessObj(
                                    SvStream&,
                                    DffObjData&,
                                    void* pData,
                                    Rectangle& rTextRect,
                                    SdrObject* pObj = NULL );
    virtual ULONG               Calc_nBLIPPos( ULONG nOrgVal, ULONG nStreamPos ) const;
    virtual FASTBOOL            GetColorFromPalette( USHORT nIndex, Color& rColor ) const;
    virtual BOOL                ShapeHasText( ULONG nShapeId, ULONG nFilePos ) const;

public:
                                Biff8MSDffManager(
                                    RootData&               rRootData,
                                    XclImpObjectManager&    rObjMan,
                                    long                    nOffsDgg,
                                    SvStream*               pStData,
                                    SdrModel*               pSdrModel_          = 0,
                                    long                    nApplicationScale   = 0,
                                    ColorData               mnDefaultColor_     = COL_DEFAULT,
                                    ULONG                   nDefaultFontHeight_ = 24,
                                    SvStream*               pStData2_           = 0 );

    virtual                     ~Biff8MSDffManager();

    void                        SetSdrObject(
                                    XclImpEscherObj* pEscherObj,
                                    ULONG nId,
                                    SvxMSDffImportData& rData );
};

#endif

