/*************************************************************************
 *
 *  $RCSfile: flttools.hxx,v $
 *
 *  $Revision: 1.11 $
 *
 *  last change: $Author: dr $ $Date: 2001/06/13 12:38:01 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _FLTTOOLS_HXX
#define _FLTTOOLS_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif


enum IMPEXC_MARGINSIDE { IMPEXC_MARGINSIDE_TOP, IMPEXC_MARGINSIDE_RIGHT,
	IMPEXC_MARGINSIDE_BOTTOM, IMPEXC_MARGINSIDE_LEFT };


#define TRUEBOOL(expr)		((expr)!=0)
#define _STRING(p)			String::CreateFromAscii(p)
#define _STRINGCONST(s)		String(RTL_CONSTASCII_USTRINGPARAM(s))


//___________________________________________________________________

class ScFilterTools
{
public:
	static double				ReadLongDouble( SvStream& rStr );
	static CharSet				GetSystemCharSet();
	static String				GetHexStr( UINT16 nValue );

	static void					AddToken( String& rTokenList, const String& rToken, sal_Unicode cSep = ';' );
    static void                 AddQuotes( String& rString, sal_Unicode cQuote = '"' );
    static void                 EraseQuotes( String& rString, sal_Unicode cQuote = '"' );

								// Excel built in names
	static const sal_Char*		GetBuiltInName( sal_Unicode nIndex );
    static void                 GetBuiltInName( String& rName, sal_Unicode nIndex, UINT16 nSheet );
    static BOOL                 IsBuiltInName( UINT16& rnSheet, const String& rName, sal_Unicode nIndex );

								// convert to valid calc name
	static void					ConvertName( String& rName );

								// string import operations
	static ByteString			ReadCString( SvStream& rStrm );
	static inline String		ReadCString( SvStream& rStrm, CharSet eSrc )
									{ return String( ReadCString( rStrm ), eSrc ); }

	static ByteString			ReadCString( SvStream& rStrm, INT32& rBytesLeft );
	static inline String		ReadCString( SvStream& rStrm, INT32& rBytesLeft, CharSet eSrc )
									{ return String( ReadCString( rStrm, rBytesLeft ), eSrc ); }

	static void					AppendCString( SvStream& rStrm, ByteString& rString );
	static void					AppendCString( SvStream& rStrm, String& rString, CharSet eSrc );

								// HTML table names <-> named range names
	static String				GetHTMLDocName();
	static String				GetHTMLTablesName();
	static String				GetNameFromHTMLIndex( ULONG nIndex );
	static String				GetNameFromHTMLName( const String& rTabName );

	static BOOL					IsHTMLDocName( const String& rSource );
	static BOOL					IsHTMLTablesName( const String& rSource );
	static BOOL					GetHTMLNameFromName( const String& rSource, String& rName );
};

//___________________________________________________________________

class UINT16List : protected List
{
public:
	inline						UINT16List() : List() {}
	inline						UINT16List( const UINT16List& rCopy ) : List( rCopy ) {}

								List::Count;
								List::Clear;

	inline UINT16				First()	{ return (UINT16)(ULONG) List::First(); }
	inline UINT16				Next()	{ return (UINT16)(ULONG) List::Next(); }
	inline UINT16				Get( ULONG nIndex ) const
										{ return (UINT16)(ULONG) List::GetObject( nIndex ); }

	inline void					Append( UINT16 nVal )
										{ List::Insert( (void*) nVal, LIST_APPEND ); }

};

//___________________________________________________________________

class UINT32List : protected List
{
public:
	inline						UINT32List() : List() {}
	inline						UINT32List( const UINT32List& rCopy ) : List( rCopy ) {}

								List::Count;
								List::Clear;

	inline UINT32				First()	{ return (UINT32) List::First(); }
	inline UINT32				Next()	{ return (UINT32) List::Next(); }
	inline UINT32				Get( ULONG nIndex ) const
										{ return (UINT32) List::GetObject( nIndex ); }

	inline void					Append( UINT32 nVal )
										{ List::Insert( (void*) nVal, LIST_APPEND ); }

};


#endif

