/*************************************************************************
 *
 *  $RCSfile: XclImpCharts.cxx,v $
 *
 *  $Revision: 1.8 $
 *
 *  last change: $Author: dr $ $Date: 2001/07/16 10:49:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//___________________________________________________________________

#ifndef _SC_XCLIMPCHARTS_HXX
#include "XclImpCharts.hxx"
#endif

#ifndef _SVX_COLRITEM_HXX
#include <svx/colritem.hxx>
#endif

#ifndef _FLTTOOLS_HXX
#include "flttools.hxx"
#endif

using namespace ::com::sun::star;

//___________________________________________________________________

template< class Class > inline void lcl_Exchange( Class& rData1, Class& rData2 )
{
	Class aTemp( rData1 );
	rData1 = rData2;
	rData2 = aTemp;
}

template< class Class > inline void lcl_CopyContent( Class*& rpDestPtr, const Class* pSourcePtr )
{
	rpDestPtr = pSourcePtr ? new Class( *pSourcePtr ) : NULL;
}

template< class Class > inline void lcl_MovePtr( Class*& rpDestPtr, Class*& rpSourcePtr )
{
	rpDestPtr = rpSourcePtr;
	rpSourcePtr = NULL;
}

template< class Class > inline void lcl_Delete( Class*& rpDelPtr )
{
	if( rpDelPtr )
		delete rpDelPtr;
	rpDelPtr = NULL;
}


//___________________________________________________________________

ColorData lcl_ReadRGB( XclImpStream& rIn )
{
	UINT8 nR, nG, nB;
	rIn >> nR >> nG >> nB;
	rIn.Ignore( 1 );
	return RGB_COLORDATA( nR, nG, nB );
}


//___________________________________________________________________

XclImpStream& operator>>( XclImpStream& rIn, ExcChart_Pos& rPos )
{
	rIn >> rPos.nPosX >> rPos.nPosY >> rPos.nWidth >> rPos.nHeight;
	return rIn;
}


//___________________________________________________________________

ExcChart_Lineformat::ExcChart_Lineformat( XclImpStream& rIn )
{
	UINT16 nFlags;

	nLineColor = lcl_ReadRGB( rIn );
	rIn >> nPattern >> nWeight >> nFlags;

	bAuto		= TRUEBOOL( nFlags & EXC_CHART_LINEF_AUTO );
	bDrawTick	= TRUEBOOL( nFlags & EXC_CHART_LINEF_TICK );
}


//___________________________________________________________________

ExcChart_FillData::ExcChart_FillData( XclImpStream& rIn, RootData* pRootData ) :
	eStyle( awt::GradientStyle_LINEAR ),
	nForeColor( COL_BLACK ),
	nBackColor( COL_BLACK ),
	nAngle( 0 ),
	nXOffset( 0 ),
	nYOffset( 0 ),
	bIsValid( FALSE )
{
	rIn.Ignore( 10 );
	UINT32 nFillMode;
	rIn >> nFillMode;
	if(	(nFillMode != EXC_CHART_GEL_CORNER) &&
		(nFillMode != EXC_CHART_GEL_CENTER) &&
		(nFillMode != EXC_CHART_GEL_LINEAR) )
		return;

	rIn.Ignore( 2 );
	UINT32 nColor1;
	rIn >> nColor1;

	rIn.Ignore( 8 );
	UINT32 nColor2;
	rIn >> nColor2;

	rIn.Ignore( 46 );
	INT16 nRot;
	rIn >> nRot;

	rIn.Ignore( 2 );
	INT32 nCol2Pos;
	rIn >> nCol2Pos;

	UINT32 nFlags;
	rIn.Ignore( 2 );
	rIn >> nFlags;
	BOOL bCorner1 = TRUEBOOL( nFlags & EXC_CHART_GEL_CORNERFLAG );
	rIn.Ignore( 2 );
	rIn >> nFlags;
	BOOL bCorner2 = TRUEBOOL( nFlags & EXC_CHART_GEL_CORNERFLAG );

	rIn.Ignore( 44 );
	UINT32 nPreset;
	rIn >> nPreset;

	rIn.Ignore( 2 );
	UINT32 nAddSize;
	rIn >> nAddSize;

	rIn.Ignore( 26 );
	UINT32 nColorMode;
	rIn >> nColorMode;

	rIn.Ignore( 6 );
	if( nPreset && (nAddSize >= 14) )
	{
		UINT16 nColCount;
		rIn >> nColCount;
		rIn.Ignore( 4 );
		nForeColor = lcl_ReadRGB( rIn );
		rIn.Ignore( (ULONG)(8 * nColCount) - 12 );
		nBackColor = lcl_ReadRGB( rIn );
	}
	else if( (nColor1 & EXC_CHART_GEL_COLTYPEMASK) == EXC_CHART_GEL_TYPECOLOR )
	{
		nForeColor = GetColor( pRootData, nColor1, COL_BLACK );
		nBackColor = GetColor( pRootData, nColor2, nForeColor );
	}
	else
	{
		DBG_ERROR( "ExcChart_FillData::ExcChart_FillData - unknown color data" );
		return;
	}

	switch( nFillMode )
	{
		case EXC_CHART_GEL_CORNER:
		{
			eStyle = awt::GradientStyle_RECT;
			nXOffset = bCorner1 ? 100 : 0;
			nYOffset = bCorner2 ? 100 : 0;
			lcl_Exchange( nForeColor, nBackColor );
			bIsValid = TRUE;
		}
		break;
		case EXC_CHART_GEL_CENTER:
		{
			eStyle = awt::GradientStyle_RECT;
			nXOffset = nYOffset = 50;
			if( nCol2Pos > 50 )
				lcl_Exchange( nForeColor, nBackColor );
			bIsValid = TRUE;
		}
		break;
		case EXC_CHART_GEL_LINEAR:
		{
			eStyle = ((Abs( nCol2Pos ) >= 25) && (Abs( nCol2Pos ) < 75)) ?
				awt::GradientStyle_AXIAL : awt::GradientStyle_LINEAR;

			if( ((nCol2Pos > -25) && (nCol2Pos < 75)) == (nRot >= 0) )
				lcl_Exchange( nForeColor, nBackColor );

			nRot %= 360;
			nAngle = ((nRot < 0) ? nRot + 360 : nRot) * 10;

			bIsValid = TRUE;
		}
		break;
	}
}

// static
ColorData ExcChart_FillData::GetColor( RootData* pRootData, UINT32 nData, ColorData nFirstColor )
{
	ColorData nColData = COL_BLACK;
	switch( nData & EXC_CHART_GEL_COLTYPEMASK )
	{
		case EXC_CHART_GEL_TYPECOLOR:
		{
			UINT16 nIndex = EXC_CHART_GEL_GETINDEX( nData );
            const SvxColorItem* pItem = pRootData->pColor->GetColor( nIndex );
            nColData = pItem ? pItem->GetValue().GetColor() : COL_WHITE;
		}
		break;
		case EXC_CHART_GEL_TYPEBRIGHTSAT:
		{
			Color aColor( nFirstColor );
			UINT8 nValue = EXC_CHART_GEL_GETBRIGHT( nData );
			switch( nData & EXC_CHART_GEL_BRIGHTTYPEMASK )
			{
				case EXC_CHART_GEL_TYPEBRIGHT:
				{
					aColor.SetRed( ChangeBrightness( aColor.GetRed(), nValue ) );
					aColor.SetGreen( ChangeBrightness( aColor.GetGreen(), nValue ) );
					aColor.SetBlue( ChangeBrightness( aColor.GetBlue(), nValue ) );
				}
				break;
				case EXC_CHART_GEL_TYPESATUR:
				{
					aColor.SetRed( ChangeSaturation( aColor.GetRed(), nValue ) );
					aColor.SetGreen( ChangeSaturation( aColor.GetGreen(), nValue ) );
					aColor.SetBlue( ChangeSaturation( aColor.GetBlue(), nValue ) );
				}
				break;
			}
			nColData = aColor.GetColor();
		}
		break;
	}
	return nColData;
}


//___________________________________________________________________

ExcChart_Areaformat::ExcChart_Areaformat( XclImpStream& rIn ) :
	pFillData( NULL )
{
	UINT16 nFlags;

	nForeColor = lcl_ReadRGB( rIn );
	nBackColor = lcl_ReadRGB( rIn );
	rIn >> nPattern >> nFlags;

	bAuto			= TRUEBOOL( nFlags & EXC_CHART_AREAF_AUTO );
	bInvertIfNeg	= TRUEBOOL( nFlags & EXC_CHART_AREAF_INVERT );
}

ExcChart_Areaformat::~ExcChart_Areaformat()
{
	lcl_Delete( pFillData );
}


//___________________________________________________________________

ExcChart_MarkerFormat::ExcChart_MarkerFormat( XclImpStream& rIn )
{
	UINT16 nFlags;

	nForeColor = lcl_ReadRGB( rIn );
	nBackColor = lcl_ReadRGB( rIn );
	rIn >> nMarkerType >> nFlags;
	rIn.Ignore( 4 );
	rIn >> nMarkerSize;
}


//___________________________________________________________________

ExcChart_Text::ExcChart_Text() :
		pString( NULL ), pFont( NULL ), pLine( NULL ), pArea( NULL ),
		eRot( ETO_NoRot ), nRot( 0 ),
		nAlignHor( EXC_CHART_TEXT_ALIGNCENTER ),
		nAlignVert( EXC_CHART_TEXT_ALIGNCENTER ),
		nMode( EXC_CHART_TEXT_BGTRANS ),
		nDataLabelPlace( EXC_CHART_TEXT_POSDEFAULT ),
		nTextColor( EXC_COLIND_AUTOTEXT ),
		nLinkObj( EXC_CHART_OLINK_NONE ),
		nLinkSer( 0 ), nLinkPoint( 0 )
{
	bAutoColor = bAutoText = bGenerated = bAutoBackgr = TRUE;
	bShowKey = bShowVal = bShowLabPrct = bShowPrct = bShowBSize = bShowLabel = FALSE;
	bVertical = bDeleted = FALSE;
}

ExcChart_Text::ExcChart_Text( XclImpStream& rIn ) :
		pString( NULL ), pFont( NULL ),	pLine( NULL ), pArea( NULL ),
		nLinkObj( EXC_CHART_OLINK_NONE ),
		nLinkSer( 0 ), nLinkPoint( 0 )
{
	UINT16 nFlags, nPlace;

	rIn >> nAlignHor >> nAlignVert >> nMode;
	nTextColor = lcl_ReadRGB( rIn );
	rIn >> aPos >> nFlags;
	rIn.Ignore( 2 );
	rIn >> nPlace >> nRot;

	bAutoColor		= TRUEBOOL( nFlags & EXC_CHART_TEXT_AUTOCOLOR );
	bShowKey		= TRUEBOOL( nFlags & EXC_CHART_TEXT_SHOWKEY );
	bShowVal		= TRUEBOOL( nFlags & EXC_CHART_TEXT_SHOWVALUE );
	bVertical		= TRUEBOOL( nFlags & EXC_CHART_TEXT_VERTICAL );
	bAutoText		= TRUEBOOL( nFlags & EXC_CHART_TEXT_AUTOTEXT );
	bGenerated		= TRUEBOOL( nFlags & EXC_CHART_TEXT_GENERATED );
	bDeleted		= TRUEBOOL( nFlags & EXC_CHART_TEXT_DELETED );
	bAutoBackgr		= TRUEBOOL( nFlags & EXC_CHART_TEXT_AUTOMODE );
	bShowLabPrct	= TRUEBOOL( nFlags & EXC_CHART_TEXT_SHOWLABELPERC );
	bShowPrct		= TRUEBOOL( nFlags & EXC_CHART_TEXT_SHOWPERCENT );
	bShowBSize		= TRUEBOOL( nFlags & EXC_CHART_TEXT_SHOWBSIZE );
	bShowLabel		= TRUEBOOL( nFlags & EXC_CHART_TEXT_SHOWLABEL );
	eRot			= (ExcTextOrient) EXC_CHART_TEXT_GETROTMODE( nFlags );
	nDataLabelPlace = (UINT8)(nPlace & 0x000F);
}

ExcChart_Text::~ExcChart_Text()
{
	lcl_Delete( pString );
	lcl_Delete( pLine );
	lcl_Delete( pArea );
}


//___________________________________________________________________

ExcChart_LinkedData::ExcChart_LinkedData( const ScRange& rRange, BOOL bGuessDir, const BOOL bIsTitle )
{
	bBaeh = FALSE;
	eState = dsNull;
	nSerCnt = 0;

	if( bGuessDir )
		GetDir( rRange );

	xValList = new ScRangeList;
	xTitleList = new ScRangeList;

	if( bIsTitle )
	{
		xTitleList->Append( rRange );
		nMinColVal = nMinRowVal = 0xFFFF;
	}
	else
	{
		xValList->Append( rRange );
		nMinColVal = rRange.aStart.Col();
		nMinRowVal = rRange.aStart.Row();
	}
}

void ExcChart_LinkedData::AppendData( const ScRange& rRange, const BOOL bGuessDir, const BOOL bIsTitle )
{
	if( bGuessDir )
		GetDir( rRange );

	if( IsBaeh( rRange, bIsTitle ) )
		bBaeh = TRUE;

	if( bIsTitle )
		xTitleList->Append( rRange );
	else
	{
		xValList->Append( rRange );

		const ScAddress& rAddr = rRange.aStart;

		if( rAddr.Col() < nMinColVal )
			nMinColVal = rAddr.Col();
		if( rAddr.Row() < nMinRowVal )
			nMinRowVal = rAddr.Row();
	}
}

void ExcChart_LinkedData::GetDir( const ScRange& rRange )
{
	if( nSerCnt )
	{
		if( eState == dsFirst )
		{
			const ScRange* pLast = xValList->Last();
			if( pLast )
				eState = (pLast->aStart.Row() == pLast->aEnd.Row()) ? dsRows : dsCols;
		}
	}
	else
	{
		if( eState == dsNull )
		{
			if( rRange.aStart.Col() == rRange.aEnd.Col() )
				eState = (rRange.aStart.Row() == rRange.aEnd.Row()) ? dsFirst : dsCols;
			else if( rRange.aStart.Row() == rRange.aEnd.Row() )
				eState = dsRows;
			else if( (rRange.aEnd.Row() - rRange.aStart.Row()) >= (rRange.aEnd.Col() - rRange.aStart.Col()) )
				eState = dsCols;
			else
				eState = dsRows;
		}
		else if( eState == dsFirst )
		{
			const ScRange*	pLast = xValList->Last();
			if( pLast )
				eState = (pLast->aStart.Row() == rRange.aStart.Row()) ? dsRows : dsCols;
		}
	}
}

BOOL ExcChart_LinkedData::Close()
{
	BOOL			bFlag	= FALSE;
	const ScRange*	pRange	= xTitleList->First();

	while( pRange )
	{
		if( (pRange->aEnd.Col() < nMinColVal) || (pRange->aEnd.Row() < nMinRowVal) )
			pRange = xTitleList->Next();
		else
		{
			bFlag = TRUE;
			pRange = NULL;
		}
	}

	if( !bFlag )
		for( pRange = xTitleList->First(); pRange; pRange = xTitleList->Next() )
			xValList->Append( *pRange );

	return bFlag;
}


//___________________________________________________________________

ExcChart_Point::ExcChart_Point( UINT32 nIndex ) :
		nPointNum( nIndex ),
		pLine( NULL ),
		pArea( NULL ),
		pText( NULL ),
		pMarker( NULL ),
		p3DData( NULL ),
		pPieDist( NULL ),
		pLabel( NULL )
{	}

ExcChart_Point::~ExcChart_Point()
{
	lcl_Delete( pLine );
	lcl_Delete( pArea );
	lcl_Delete( pText );
	lcl_Delete( pMarker );
	lcl_Delete( p3DData );
	lcl_Delete( pPieDist );
	lcl_Delete( pLabel );
}

void ExcChart_Point::Update( const ExcChart_Point& rPoint )
{
	if( !pLine )
		lcl_CopyContent( pLine, rPoint.pLine );
	if( !pArea )
		lcl_CopyContent( pArea, rPoint.pArea );
	if( !pText )
		lcl_CopyContent( pText, rPoint.pText );
	if( !pMarker )
		lcl_CopyContent( pMarker, rPoint.pMarker );
	if( !p3DData )
		lcl_CopyContent( p3DData, rPoint.p3DData );
	if( !pPieDist )
		lcl_CopyContent( pPieDist, rPoint.pPieDist );
	if( !pLabel )
		lcl_CopyContent( pLabel, rPoint.pLabel );
}

BOOL ExcChart_Point::HasAttachedLabel()
{
	return TRUEBOOL( (pLabel ? *pLabel : 0) & EXC_CHART_ATLAB_ISATTACHED );
}


//___________________________________________________________________

ExcChart_PointList::~ExcChart_PointList()
{
	for( ExcChart_Point* pPoint = First(); pPoint; pPoint = Next() )
		delete pPoint;
}

ExcChart_Point* ExcChart_PointList::Find( UINT16 nPoint )
{
	for( ExcChart_Point* pPoint = First(); pPoint; pPoint = Next() )
		if( pPoint->nPointNum == nPoint )
			return pPoint;
	return NULL;
}

ExcChart_Point& ExcChart_PointList::GetPoint( UINT16 nPoint )
{
	ExcChart_Point* pPoint = Find( nPoint );
	if( !pPoint )
	{
		pPoint = new ExcChart_Point( nPoint );
		Append( pPoint );
	}
	return *pPoint;
}

BOOL ExcChart_PointList::HasAttachedLabel()
{
	for( ExcChart_Point* pPoint = First(); pPoint; pPoint = Next() )
		if( pPoint->HasAttachedLabel() )
			return TRUE;
	return FALSE;
}


//___________________________________________________________________

ExcChart_Series::ExcChart_Series( UINT16 nSerI ) :
		nSerInd( nSerI ),
		nSerNum( 0xFFFF ),
		pAll( NULL ),
		pSpec( NULL ),
		bSecondary( FALSE )
{	}

ExcChart_Series::~ExcChart_Series()
{
	lcl_Delete( pAll );
	lcl_Delete( pSpec );
}

ExcChart_Point& ExcChart_Series::GetPoint( UINT16 nPointNum )
{
	if( nPointNum == EXC_CHART_DATAF_SERIES )
	{
		if( !pAll )
			pAll = new ExcChart_Point( nPointNum );
		return *pAll;
	}

	if( !pSpec )
		pSpec = new ExcChart_PointList;
	return pSpec->GetPoint( nPointNum );
}

BOOL ExcChart_Series::HasAttachedLabel()
{
	return (pAll ? pAll->HasAttachedLabel() : FALSE) || (pSpec ? pSpec->HasAttachedLabel() : FALSE);
}


//___________________________________________________________________

ExcChart_SeriesList::ExcChart_SeriesList() :
		pCurrSer( NULL ),
		pAll( NULL )
{	}

ExcChart_SeriesList::~ExcChart_SeriesList()
{
	for( ExcChart_Series* pSer = First(); pSer;	pSer = Next() )
		delete pSer;
	lcl_Delete( pAll );
}

ExcChart_Series* ExcChart_SeriesList::Find( UINT16 nSerInd )
{
	if( pCurrSer && (pCurrSer->nSerInd == nSerInd) )
		return pCurrSer;

	for( ExcChart_Series* pSer = First(); pSer; pSer = Next() )
		if( pSer->nSerInd == nSerInd )
			return pSer;

	return NULL;
}

const ExcChart_Series* ExcChart_SeriesList::FindBySerNum( UINT32 nSerNum ) const
{
	if( pCurrSer && (pCurrSer->nSerNum == nSerNum) )
		return pCurrSer;

	for( UINT32 nIndex = 0; nIndex < Count(); nIndex++ )
	{
		const ExcChart_Series* pSer = Get( nIndex );
		if( pSer && (pSer->nSerNum == nSerNum) )
			return pSer;
	}

	return NULL;
}

ExcChart_Series& ExcChart_SeriesList::GetSeries( UINT16 nSerInd )
{
	ExcChart_Series* pSer = Find( nSerInd );
	if( !pSer )
	{
		pSer = new ExcChart_Series( nSerInd );
		Append( pSer );
	}
	return *pSer;
}

ExcChart_Point& ExcChart_SeriesList::GetPoint( UINT16 nSerInd, UINT16 nPointNum )
{
	ExcChart_Series& rSer = GetSeries( nSerInd );
	return rSer.GetPoint( nPointNum );
}

ExcChart_Point& ExcChart_SeriesList::GetGlobalFormat()
{
	if( !pAll )
		pAll = new ExcChart_Point( 0 );
	return *pAll;
}

BOOL ExcChart_SeriesList::HasAttachedLabel()
{
	if( pAll && pAll->HasAttachedLabel() )
		return TRUE;
	for( ExcChart_Series* pSer = First(); pSer; pSer = Next() )
		if( pSer->HasAttachedLabel() )
			return TRUE;
	return FALSE;
}

void ExcChart_SeriesList::SetSeriesNumber( UINT16 nSerInd, UINT16 nSerNum )
{
	ExcChart_Series& rSer = GetSeries( nSerInd );
	rSer.nSerNum = nSerNum;
}

void ExcChart_SeriesList::SetSecondaryFlag( UINT16 nSerInd )
{
	ExcChart_Series& rSer = GetSeries( nSerInd );
	rSer.bSecondary = TRUE;
}


//___________________________________________________________________

ExcChart_ValueRange::ExcChart_ValueRange( XclImpStream& rIn )
{
	UINT16 nFlags;
	rIn >> fMin >> fMax >> fMajorInc >> fMinorInc >> fCross >> nFlags;

	bAutoMin	= TRUEBOOL( nFlags & EXC_CHART_VALR_AUTOMIN );
	bAutoMax	= TRUEBOOL( nFlags & EXC_CHART_VALR_AUTOMAX );
	bAutoMajor	= TRUEBOOL( nFlags & EXC_CHART_VALR_AUTOMAJOR );
	bAutoMinor	= TRUEBOOL( nFlags & EXC_CHART_VALR_AUTOMINOR );
	bAutoCross	= TRUEBOOL( nFlags & EXC_CHART_VALR_AUTOCROSS );
	bLogScale	= TRUEBOOL( nFlags & EXC_CHART_VALR_LOGSCALE );
	bReverse	= TRUEBOOL( nFlags & EXC_CHART_VALR_REVERSE );
	bMaxCross	= TRUEBOOL( nFlags & EXC_CHART_VALR_MAXCROSS );
}


//___________________________________________________________________

ExcChart_Tick::ExcChart_Tick( XclImpStream& rIn )
{
	UINT8	nBkgMode;
	UINT16	nFlags;

	rIn >> nMajor >> nMinor >> nLabel >> nBkgMode;
	nTextColor = lcl_ReadRGB( rIn );
	rIn.Ignore( 16 );
	rIn >> nFlags;
	rIn.Ignore( 2 );
	rIn >> nRot;

	bTransparent	= (nBkgMode != EXC_CHART_TICK_OPAQUE);
	bAutoColor		= TRUEBOOL( nFlags & EXC_CHART_TICK_AUTO );
	bAutoBack		= TRUEBOOL( nFlags & EXC_CHART_TICK_AUTOMODE );
	bAutoRot		= TRUEBOOL( nFlags & EXC_CHART_TICK_AUTOROT );
	eRot			= (ExcTextOrient) EXC_CHART_TICK_GETROTMODE( nFlags );
}


//___________________________________________________________________

ExcChart_Axis::ExcChart_Axis()
{
	pValueRange = NULL;
	pTick = NULL;

	pAxisLine = pMajorLine = pMinorLine = pWallLine = NULL;
	pWallArea = NULL;

	pTitle = NULL;
	pCaptionText = new ExcChart_Text;

	nNumForm = EXC_CHART_IFMT_INVALID;
	bBreakText = TRUE;
}

ExcChart_Axis::~ExcChart_Axis()
{
	lcl_Delete( pValueRange );
	lcl_Delete( pTick );
	lcl_Delete( pAxisLine );
	lcl_Delete( pMajorLine );
	lcl_Delete( pMinorLine );
	lcl_Delete( pWallLine );
	lcl_Delete( pWallArea );
	lcl_Delete( pTitle );
	lcl_Delete( pCaptionText );
}


//___________________________________________________________________

ExcChart_AxesSet::~ExcChart_AxesSet()
{
	lcl_Delete( pCatAxis );
	lcl_Delete( pValueAxis );
	lcl_Delete( pSeriesAxis );
}


//___________________________________________________________________

ExcChart_Data3D::ExcChart_Data3D( XclImpStream& rIn )
{
	UINT16 nFlags;
	rIn >> nRot >> nElev >> nDist >> nHeight >> nDepth >> nGap >> nFlags;

	bPersp		= TRUEBOOL( nFlags & EXC_CHART_3D_PERSP );
	bCluster	= TRUEBOOL( nFlags & EXC_CHART_3D_CLUSTER );
	b3DScaling	= TRUEBOOL( nFlags & EXC_CHART_3D_AUTOSCALE );
	b2DWalls	= TRUEBOOL( nFlags & EXC_CHART_3D_2DWALLS );
}


//___________________________________________________________________

ExcChart_Dropbar::ExcChart_Dropbar( XclImpStream& rIn ) :
		pLine( NULL ), pArea( NULL )
{
	rIn >> nGap;
}

ExcChart_Dropbar::~ExcChart_Dropbar()
{
	lcl_Delete( pLine );
	lcl_Delete( pArea );
}


//___________________________________________________________________

ExcChart_Legend::ExcChart_Legend( XclImpStream& rIn ) :
		pLine( NULL ), pArea( NULL ), pText( NULL )
{
	UINT16 nFlags;
	rIn >> aPos >> nType >> nSpacing >> nFlags;

	bDocked			= TRUEBOOL( nFlags & EXC_CHART_LEGEND_DOCKED );
	bAutoSeries		= TRUEBOOL( nFlags & EXC_CHART_LEGEND_AUTOSERIES );
	bAutoPosX		= TRUEBOOL( nFlags & EXC_CHART_LEGEND_AUTOPOSX );
	bAutoPosY		= TRUEBOOL( nFlags & EXC_CHART_LEGEND_AUTOPOSY );
	bVert			= TRUEBOOL( nFlags & EXC_CHART_LEGEND_VERT );
	bWasDataTable	= TRUEBOOL( nFlags & EXC_CHART_LEGEND_DATATABLE );
}

ExcChart_Legend::~ExcChart_Legend()
{
	lcl_Delete( pLine );
	lcl_Delete( pArea );
	lcl_Delete( pText );
}


//___________________________________________________________________
// class ExcChart

ExcChart::ExcChart( RootData* pRoot ) : ExcRoot( pRoot )
{
	pStateStack = new ExcStateStack;
	pStateStack->Push( csGlobal );

	pSourceData = NULL;

	pLine = NULL;
	pArea = NULL;

	pPlotLine = NULL;
	pPlotArea = NULL;

	pChartTitle = NULL;

	pSeries = NULL;
	nCurSeries = 0;
	pLastDataPoint = NULL;

	pPrimAS = pSecAS = NULL;

	p3D = NULL;
	pLegend = NULL;
	pUpBar = pDownBar = NULL;
	pHiLoLine = NULL;

	nCurrDefText = -1;
	pDefText0 = pDefText1 = pDefText2 = pNewText = NULL;

	ppCurrLine = NULL;
	ppCurrArea = NULL;
	ppCurrText = NULL;
	pCurrAS = NULL;
	pCurrAxis = NULL;
	pCurrBar = NULL;

	nBaseTab = 0;
	bSecondary = FALSE;
	bHasSeriesNames = bHasCategoryNames = FALSE;
	bStock = bSpline = FALSE;

	eChartType = ctUnknown;
	nMaxPieDist = 0;
	nRowOffset = 0;
}

ExcChart::ExcChart( ExcChart& rCpy ) :
		ExcRoot( rCpy.pExcRoot ),
		nBaseTab( rCpy.nBaseTab ),
		nCurSeries( rCpy.nCurSeries ),
		nCurrDefText( rCpy.nCurrDefText ),
		bSecondary( rCpy.bSecondary ),
		bHasSeriesNames( rCpy.bHasSeriesNames ),
		bHasCategoryNames( rCpy.bHasCategoryNames ),
		bStock( rCpy.bStock ),
		bSpline( rCpy.bSpline ),
		eChartType( rCpy.eChartType ),
		nMaxPieDist( rCpy.nMaxPieDist ),
		nRowOffset( rCpy.nRowOffset )
{
	lcl_MovePtr( pStateStack, rCpy.pStateStack );

	lcl_MovePtr( pSourceData, rCpy.pSourceData );

	lcl_MovePtr( pLine, rCpy.pLine );
	lcl_MovePtr( pArea, rCpy.pArea );

	lcl_MovePtr( pPlotLine, rCpy.pPlotLine );
	lcl_MovePtr( pPlotArea, rCpy.pPlotArea );

	lcl_MovePtr( pChartTitle, rCpy.pChartTitle );

	lcl_MovePtr( pSeries, rCpy.pSeries );
	lcl_MovePtr( pLastDataPoint, rCpy.pLastDataPoint );

	lcl_MovePtr( pPrimAS, rCpy.pPrimAS );
	lcl_MovePtr( pSecAS, rCpy.pSecAS );

	lcl_MovePtr( p3D, rCpy.p3D );
	lcl_MovePtr( pLegend, rCpy.pLegend );
	lcl_MovePtr( pUpBar, rCpy.pUpBar );
	lcl_MovePtr( pDownBar, rCpy.pDownBar );
	lcl_MovePtr( pHiLoLine, rCpy.pHiLoLine );

	lcl_MovePtr( pDefText0, rCpy.pDefText0 );
	lcl_MovePtr( pDefText1, rCpy.pDefText1 );
	lcl_MovePtr( pDefText2, rCpy.pDefText2 );
	lcl_MovePtr( pNewText, rCpy.pNewText );

	lcl_MovePtr( ppCurrLine, rCpy.ppCurrLine );
	lcl_MovePtr( ppCurrArea, rCpy.ppCurrArea );
	lcl_MovePtr( ppCurrText, rCpy.ppCurrText );
	lcl_MovePtr( pCurrAS, rCpy.pCurrAS );
	lcl_MovePtr( pCurrAxis, rCpy.pCurrAxis );
	lcl_MovePtr( pCurrBar, rCpy.pCurrBar );
}

ExcChart::~ExcChart()
{
	Reset();
}

void ExcChart::Reset()
{
	nBaseTab = 0;

	lcl_Delete( pStateStack );

	lcl_Delete( pSourceData );

	lcl_Delete( pLine );
	lcl_Delete( pArea );

	lcl_Delete( pPlotLine );
	lcl_Delete( pPlotArea );

	lcl_Delete( pChartTitle );

	lcl_Delete( pSeries );
	nCurSeries = 0;
	pLastDataPoint = NULL;

	lcl_Delete( pPrimAS );
	lcl_Delete( pSecAS );

	lcl_Delete( p3D );
	lcl_Delete( pLegend );
	lcl_Delete( pUpBar );
	lcl_Delete( pDownBar );
	lcl_Delete( pHiLoLine );

	nCurrDefText = -1;
	lcl_Delete( pDefText0 );
	lcl_Delete( pDefText1 );
	lcl_Delete( pDefText2 );
	lcl_Delete( pNewText );

	ppCurrLine = NULL;
	ppCurrArea = NULL;
	ppCurrText = NULL;
	pCurrAS = NULL;
	pCurrAxis = NULL;
	pCurrBar = NULL;

	bSecondary = FALSE;
	bHasSeriesNames = bHasCategoryNames = FALSE;
	bStock = bSpline = FALSE;

	eChartType = ctUnknown;
	nMaxPieDist = 0;
	nRowOffset = 0;
}

// for derived classes

void ExcChart::ApplyExt( uno::Reference< lang::XComponent > xComponent ) const
{	}

void ExcChart::ApplyExtPost( uno::Reference< lang::XComponent > xComponent ) const
{	}

// internal chart data

void ExcChart::UpdateState( ExcChartState eState )
{
	ppCurrLine = NULL;
	ppCurrArea = NULL;
	switch( eState )
	{
		case csChartFrame:
			ppCurrLine = &pLine;
			ppCurrArea = &pArea;
		break;
		case csPointFormat:
		case csDataFormat:
			if( pLastDataPoint )
			{
				ppCurrLine = &pLastDataPoint->pLine;
				ppCurrArea = &pLastDataPoint->pArea;
			}
		break;
		case csAxis:
			ppCurrText = pCurrAxis ? &pCurrAxis->pCaptionText : NULL;
		break;
		case csDropbar:
			if( pCurrBar )
			{
				ppCurrLine = &pCurrBar->pLine;
				ppCurrArea = &pCurrBar->pArea;
			}
		break;
		case csLegendText:
			if( pLegend )
				MoveNewTextTo( pLegend->pText );
		break;
		case csLegendFrame:
			if( pLegend )
			{
				ppCurrLine = &pLegend->pLine;
				ppCurrArea = &pLegend->pArea;
			}
		break;
		case csDefText:
		{
			switch( nCurrDefText )
			{
				case 0:	MoveNewTextTo( pDefText0 );	break;
				case 1:	MoveNewTextTo( pDefText1 );	break;
				case 2:	MoveNewTextTo( pDefText2 );	break;
			}
		}
		break;
		case csText:
			ppCurrText = &pNewText;
		break;
		case csTextFrame:
			if( ppCurrText && *ppCurrText )
			{
				ppCurrLine = &(*ppCurrText)->pLine;
				ppCurrArea = &(*ppCurrText)->pArea;
			}
		break;
		case csPlotarea:
			ppCurrLine = &pPlotLine;
			ppCurrArea = &pPlotArea;
		break;
		default:
			ppCurrText = NULL;
	}

	nCurrDefText = -1;
}

void ExcChart::EndAxis()
{
	if( !pCurrAxis ) return;

	if( pCurrAxis->pCaptionText && pCurrAxis->pTick )
		pCurrAxis->pCaptionText->nRot = pCurrAxis->pTick->nRot;
	pCurrAxis = NULL;
}

void ExcChart::MoveNewTextTo( ExcChart_Text*& rpDest )
{
	lcl_Delete( rpDest );
	rpDest = pNewText;
	pNewText = NULL;
	ppCurrText = &rpDest;
}

void ExcChart::LinkNewText()
{
	if( !pNewText ) return;

	switch( pNewText->nLinkObj )
	{
		case EXC_CHART_OLINK_TITLE:
			MoveNewTextTo( pChartTitle );
		break;
		case EXC_CHART_OLINK_XAXIS:
			if( pPrimAS && pPrimAS->pCatAxis )
				MoveNewTextTo( pPrimAS->pCatAxis->pTitle );
		break;
		case EXC_CHART_OLINK_YAXIS:
			if( pPrimAS && pPrimAS->pValueAxis )
				MoveNewTextTo( pPrimAS->pValueAxis->pTitle );
		break;
		case EXC_CHART_OLINK_ZAXIS:
			if( pPrimAS && pPrimAS->pSeriesAxis )
				MoveNewTextTo( pPrimAS->pSeriesAxis->pTitle );
		break;
		case EXC_CHART_OLINK_DATA:
		{
			ExcChart_Point& rPoint = GetPoint( pNewText->nLinkSer, pNewText->nLinkPoint );
			MoveNewTextTo( rPoint.pText );
		}
		break;
		default:
			lcl_Delete( pNewText );
			ppCurrText = NULL;
	}
}

ExcChart_Point& ExcChart::GetPoint( UINT16 nSerInd, UINT16 nPointNum )
{
	if( !pSeries )
		pSeries = new ExcChart_SeriesList;
	return pSeries->GetPoint( nSerInd, nPointNum );
}

ExcChart_Point& ExcChart::GetGlobalFormat()
{
	if( !pSeries )
		pSeries = new ExcChart_SeriesList;
	return pSeries->GetGlobalFormat();
}

// CHART common recs

void ExcChart::ReadBegin()
{
	pStateStack->Push( eNewState );
	UpdateState( eNewState );
	eNewState = csUnknown;
}

void ExcChart::ReadEnd()
{
	switch( pStateStack->Pop() )
	{
		case csText:		LinkNewText();		break;
		case csAxis:		pCurrAxis = NULL;	break;
		case csAxisParent:	pCurrAS = NULL;		break;
		case csDropbar: 	pCurrBar = NULL;	break;
		case csSeries:		nCurSeries++;		break;
	}
	UpdateState( GetState() );
}

void ExcChart::ReadChart()
{
	eNewState = csChart;
}

// FRAME group

void ExcChart::ReadFrame()
{
	if( eNewState != csUnknown ) return;	// for csPlotarea
	switch( GetState() )
	{
		case csChart:	eNewState = csChartFrame;		break;
		case csDefText:
		case csText:	eNewState = csTextFrame;		break;
		case csLegend:	eNewState = csLegendFrame;		break;
		default:		eNewState = csUnknown;
	}
}

void ExcChart::ReadLineformat( XclImpStream& rIn )
{
	if( ppCurrLine )
	{
		lcl_Delete( *ppCurrLine );
		*ppCurrLine = new ExcChart_Lineformat( rIn );
	}
}

void ExcChart::ReadAreaformat( XclImpStream& rIn )
{
	if( ppCurrArea )
	{
		lcl_Delete( *ppCurrArea );
		*ppCurrArea = new ExcChart_Areaformat( rIn );
	}
}

void ExcChart::ReadGelframe( XclImpStream& rIn )
{
	if( ppCurrArea && *ppCurrArea )
	{
		lcl_Delete( (*ppCurrArea)->pFillData );
		(*ppCurrArea)->pFillData = new ExcChart_FillData( rIn, pExcRoot );
	}
}

// TEXT group

void ExcChart::ReadText( XclImpStream& rIn )
{
	lcl_Delete( pNewText );
	pNewText = new ExcChart_Text( rIn );
	ppCurrText = &pNewText;

	if( (nCurrDefText >= 0) && (nCurrDefText <= 2) )
		eNewState = csDefText;
	else
	{
		switch( GetState() )
		{
			case csLegend:	eNewState = csLegendText;	break;
			default:		eNewState = csText;
		}
	}
}

void ExcChart::ReadFontx( XclImpStream& rIn )
{
	if( ppCurrText && *ppCurrText )
	{
		UINT16 nFontx;
		rIn >> nFontx;

		(*ppCurrText)->pFont = pExcRoot->pFontBuffer->GetFont( nFontx );
	}
}

void ExcChart::SetSeriestext( const String& rString )
{
	if( ppCurrText && *ppCurrText )
	{
		lcl_Delete( (*ppCurrText)->pString );
		(*ppCurrText)->pString = new String( rString );
	}
}

void ExcChart::ReadObjectlink( XclImpStream& rIn )
{
	if( ppCurrText && *ppCurrText )
	{
		rIn >> (*ppCurrText)->nLinkObj
			>> (*ppCurrText)->nLinkSer
			>> (*ppCurrText)->nLinkPoint;
	}
}

void ExcChart::ReadDefaulttext( XclImpStream& rIn )
{
	rIn >> nCurrDefText;
}

// SERIES group

void ExcChart::ReadSeries()
{
	eNewState = csSeries;
}

void ExcChart::ReadSertocrt( XclImpStream& rIn )
{
	UINT16 nGroup;
	rIn >> nGroup;

	if( (nGroup == EXC_CHART_SECONDGROUP) && pSeries )
		pSeries->SetSecondaryFlag( nCurSeries );
}

void ExcChart::AddSourceData( const ScRange& rRange, BOOL bValues, BOOL bIsTitle )
{
	if( pSourceData )
		pSourceData->AppendData( rRange, bValues, bIsTitle );
	else
		pSourceData = new ExcChart_LinkedData( rRange, bValues, bIsTitle );
}

void ExcChart::EndSerValues()
{
	if( pSourceData )
		pSourceData->EndSer();
}

void ExcChart::CloseSourceData( void )
{
	if( pSourceData )
	{
		if( pSourceData->Close() )
			bHasSeriesNames = bHasCategoryNames = FALSE;
	}
}

// DATAFORMAT group

void ExcChart::ReadDataformat( XclImpStream& rIn )
{
	UINT16 nPoint, nSerInd, nSerNum;
	rIn >> nPoint >> nSerInd >> nSerNum;

	if( nSerNum == EXC_CHART_DATAF_CHART )
	{
		ExcChart_Point& rPoint = GetGlobalFormat();
		pLastDataPoint = &rPoint;
	}
	else
	{
		ExcChart_Point& rPoint = GetPoint( nSerInd, nPoint );
		pLastDataPoint = &rPoint;
		pSeries->SetSeriesNumber( nSerInd, nSerNum );
	}

	switch( GetState() )
	{
		case csSeries:	eNewState = csPointFormat;	break;
		case csFormat:	eNewState = csDataFormat;	break;
		default:		eNewState = csUnknown;
	}
}

void ExcChart::ReadMarkerformat( XclImpStream& rIn )
{
	if( pLastDataPoint )
	{
		if( !pLastDataPoint->pMarker )
			pLastDataPoint->pMarker = new ExcChart_MarkerFormat( rIn );
	}
}

void ExcChart::Read3DDataformat( XclImpStream& rIn )
{
	if( pLastDataPoint )
	{
		UINT8 nBase, nTop;
		rIn >> nBase >> nTop;

		if( !pLastDataPoint->p3DData )
			pLastDataPoint->p3DData = new ExcChart_3DDataFormat( nBase, nTop );
	}
}

void ExcChart::ReadPieformat( XclImpStream& rIn )
{
	if( pLastDataPoint )
	{
		UINT16 nDist;
		rIn >> nDist;

		if( !pLastDataPoint->pPieDist )
			pLastDataPoint->pPieDist = new UINT16( nDist );
		nMaxPieDist = Max( nMaxPieDist, nDist );
	}
}

void ExcChart::ReadSerfmt( XclImpStream& rIn )
{
	UINT16 nFlags;
	rIn >> nFlags;
	bSpline |= TRUEBOOL( nFlags & EXC_CHART_SERF_SMOOTHED );
}

void ExcChart::ReadAttachedlabel( XclImpStream& rIn )
{
	if( pLastDataPoint )
	{
		UINT16 nLabel;
		rIn >> nLabel;

		if( !pLastDataPoint->pLabel )
			pLastDataPoint->pLabel = new UINT16( nLabel );
	}
}

// AXIS group

void ExcChart::ReadAxesused( XclImpStream& rIn )
{
	UINT16 nUsed;
	rIn >> nUsed;
	bSecondary = (nUsed > 1);
}

void ExcChart::ReadAxisparent( XclImpStream& rIn )
{
	UINT16 nAxesSet;
	rIn >> nAxesSet;

	if( nAxesSet == EXC_CHART_FIRSTAXES )
	{
		if( !pPrimAS )
			pPrimAS = new ExcChart_AxesSet;
		pCurrAS = pPrimAS;
	}
	else if( nAxesSet == EXC_CHART_SECONDAXES )
	{
		if( !pSecAS )
			pSecAS = new ExcChart_AxesSet;
		pCurrAS = pSecAS;
	}
	else
		pCurrAS = NULL;

	if( (nAxesSet == EXC_CHART_FIRSTAXES) && pCurrAS )
		rIn >> pCurrAS->aPos;

	eNewState = csAxisParent;
}

void ExcChart::ReadAxis( XclImpStream& rIn )
{
	pCurrAxis = NULL;

	UINT16 nAxis;
	rIn >> nAxis;

	if( pCurrAS )
	{
		switch( nAxis )
		{
			case EXC_CHART_XAXIS:
				if( !pCurrAS->pCatAxis )
					pCurrAS->pCatAxis = new ExcChart_Axis;
				pCurrAxis = pCurrAS->pCatAxis;
			break;
			case EXC_CHART_YAXIS:
				if( !pCurrAS->pValueAxis )
					pCurrAS->pValueAxis = new ExcChart_Axis;
				pCurrAxis = pCurrAS->pValueAxis;
			break;
			case EXC_CHART_ZAXIS:
				if( !pCurrAS->pSeriesAxis )
					pCurrAS->pSeriesAxis = new ExcChart_Axis;
				pCurrAxis = pCurrAS->pSeriesAxis;
			break;
		}
	}
	eNewState = csAxis;
}

void ExcChart::ReadCatserrange( XclImpStream& rIn )
{
	if( pCurrAxis )
	{
		rIn.Ignore( 2 );
		UINT16 nLabelFreq;
		rIn >> nLabelFreq;
		pCurrAxis->bBreakText = (nLabelFreq == 1);
	}
}

void ExcChart::ReadValuerange( XclImpStream& rIn )
{
	if( pCurrAxis )
	{
		lcl_Delete( pCurrAxis->pValueRange );
		pCurrAxis->pValueRange = new ExcChart_ValueRange( rIn );
	}
}

void ExcChart::ReadTick( XclImpStream& rIn )
{
	if( pCurrAxis )
	{
		lcl_Delete( pCurrAxis->pTick );
		pCurrAxis->pTick = new ExcChart_Tick( rIn );
		if( pCurrAxis->pCaptionText )
			pCurrAxis->pCaptionText->nRot = pCurrAxis->pTick->nRot;
	}
}

void ExcChart::SetIfmt( UINT32 nFmt )
{
	if( pCurrAxis )
		pCurrAxis->nNumForm = nFmt;
}

void ExcChart::ReadAxislineformat( XclImpStream& rIn )
{
	ppCurrLine = NULL;
	ppCurrArea = NULL;

	if( pCurrAxis )
	{
		UINT16 nLineType;
		rIn >> nLineType;

		switch( nLineType )
		{
			case EXC_CHART_AXLINEF_AXISLINE:	ppCurrLine = &pCurrAxis->pAxisLine;		break;
			case EXC_CHART_AXLINEF_MAJORGRID:	ppCurrLine = &pCurrAxis->pMajorLine;	break;
			case EXC_CHART_AXLINEF_MINORGRID:	ppCurrLine = &pCurrAxis->pMinorLine;	break;
			case EXC_CHART_AXLINEF_WALLS:
				ppCurrLine = &pCurrAxis->pWallLine;
				ppCurrArea = &pCurrAxis->pWallArea;
			break;
		}
	}
}

void ExcChart::ReadPlotarea()
{
	eNewState = csPlotarea;
}

// CHARTFORMAT group

void ExcChart::ReadChartformat()
{
	eNewState = csFormat;
}

void ExcChart::Read3D( XclImpStream& rIn )
{
	lcl_Delete( p3D );
	p3D = new ExcChart_Data3D( rIn );
}

void ExcChart::ReadLegend( XclImpStream& rIn )
{
	lcl_Delete( pLegend );
	pLegend = new ExcChart_Legend( rIn );
	eNewState = csLegend;
}

void ExcChart::ReadDropbar( XclImpStream& rIn )
{
	if( !pUpBar )
		pCurrBar = pUpBar = new ExcChart_Dropbar( rIn );
	else if( !pDownBar )
		pCurrBar = pDownBar = new ExcChart_Dropbar( rIn );
	else
		pCurrBar = NULL;

	bStock = TRUE;
	eNewState = csDropbar;
}

void ExcChart::ReadChartline( XclImpStream& rIn )
{
	UINT16 nId;
	rIn >> nId;
	switch( nId )
	{
		case EXC_CHART_CHLINE_HILO:		ppCurrLine = &pHiLoLine;	break;
		default:						ppCurrLine = NULL;
	}

	bStock = TRUE;
}


//___________________________________________________________________
// line charts

ExcChartLine::ExcChartLine( ExcChart& rChart, XclImpStream& rIn, BOOL bArea ) :
		ExcChart( rChart )
{
	UINT16 nFlags;
	rIn >> nFlags;

	bStacked		= TRUEBOOL( nFlags & EXC_CHART_LINE_STACKED );
	bCatAsPercent	= TRUEBOOL( nFlags & EXC_CHART_LINE_PERCENT );
	bHasShadow		= TRUEBOOL( nFlags & EXC_CHART_LINE_SHADOW );

	eChartType = bArea ? ctArea : ctLine;
}

ExcChartLine::~ExcChartLine()
{	}


//___________________________________________________________________
// bar & column charts

ExcChartBar::ExcChartBar( ExcChart& rChart, XclImpStream& rIn ) :
		ExcChart( rChart )
{
	UINT16 nFlags;
	rIn >> nSpaceBeetwBar >> nSpaceBeetwCat >> nFlags;

	bHorizontal		= TRUEBOOL( nFlags & EXC_CHART_BAR_TRANS );
	bStacked		= TRUEBOOL( nFlags & EXC_CHART_BAR_STACKED );
	bCatAsPercent	= TRUEBOOL( nFlags & EXC_CHART_BAR_PERCENT );
	bHasShadow		= TRUEBOOL( nFlags & EXC_CHART_BAR_SHADOW );

	eChartType = bHorizontal ? ctBar : ctColumn;
}

ExcChartBar::~ExcChartBar()
{	}


//___________________________________________________________________
// pie & donut charts

ExcChartPie::ExcChartPie( ExcChart& rChart, XclImpStream& rIn ) :
		ExcChart( rChart )
{
	UINT16 nFlags;
	rIn >> nAngleFirstSlice >> nHoleSize >> nFlags;

	bHasShadow			= TRUEBOOL( nFlags & EXC_CHART_PIE_SHADOW );
	bShowLeaderLines	= TRUEBOOL( nFlags & EXC_CHART_PIE_LINES );

	eChartType = nHoleSize ? ctDonut : ctPie;
}

ExcChartPie::~ExcChartPie()
{	}


//___________________________________________________________________
// radar charts

ExcChartRadar::ExcChartRadar( ExcChart& rChart ) :
		ExcChart( rChart )
{
	eChartType = ctNet;
}

ExcChartRadar::~ExcChartRadar()
{	}


//___________________________________________________________________
// surface charts

ExcChartSurface::ExcChartSurface( ExcChart& rChart ) :
		ExcChart( rChart )
{	}

ExcChartSurface::~ExcChartSurface()
{	}


//___________________________________________________________________
// scatter charts

ExcChartScatter::ExcChartScatter( ExcChart& rChart, XclImpStream& rIn ) :
		ExcChart( rChart )
{
	UINT16 nFlags;
		rIn >> nBubbleSizeRatio >> nBubbleSize >> nFlags;

	bBubbles		= TRUEBOOL( nFlags & EXC_CHART_SCAT_BUBBLES );
	bShowNegBubbles	= TRUEBOOL( nFlags & EXC_CHART_SCAT_NEGBUBBLES );
	bHasShadows		= TRUEBOOL( nFlags & EXC_CHART_SCAT_SHADOW );

	bHasCategoryNames = FALSE;
	nRowOffset = 1;

	eChartType = ctScatter;
}

ExcChartScatter::~ExcChartScatter()
{	}

