/*************************************************************************
 *
 *  $RCSfile: InteractionHandler.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: mh $ $Date: 2000/10/23 16:22:19 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_TASK_XINTERACTIONHANDLER_HPP_
#include <com/sun/star/task/XInteractionHandler.hpp>
#endif
#ifndef _COM_SUN_STAR_UCB_INTERACTIVEWRONGMEDIUMEXCEPTION_HPP_
#include <com/sun/star/ucb/InteractiveWrongMediumException.hpp>
#endif
#ifndef _COM_SUN_STAR_TASK_XINTERACTIONAPPROVE_HPP_
#include <com/sun/star/task/XInteractionApprove.hpp>
#endif
#ifndef _COM_SUN_STAR_TASK_XINTERACTIONDISAPPROVE_HPP_
#include <com/sun/star/task/XInteractionDisapprove.hpp>
#endif
#ifndef _COM_SUN_STAR_TASK_XINTERACTIONABORT_HPP_
#include <com/sun/star/task/XInteractionAbort.hpp>
#endif
#ifndef _COM_SUN_STAR_TASK_XINTERACTIONRETRY_HPP_
#include <com/sun/star/task/XInteractionRetry.hpp>
#endif
#ifndef _COM_SUN_STAR_UCB_INTERACTIVEFILEIOEXCEPTION_HPP_
#include <com/sun/star/ucb/InteractiveFileIOException.hpp>
#endif
#ifndef _COM_SUN_STAR_UCB_IOERRORCODE_HPP_
#include <com/sun/star/ucb/IOErrorCode.hpp>
#endif

#ifndef __RSC
#include <tools/errinf.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _CPPUHELPER_IMPLBASE1_HXX_
#include <cppuhelper/implbase1.hxx>
#endif
#ifndef _SV_MSGBOX_HXX
#include <vcl/msgbox.hxx>
#endif

#include "sdresid.hxx"
#include "strings.hrc"

using namespace ::rtl;
//using namespace ::vos;
using namespace ::com::sun::star;

class PackerErrorContext : public ErrorContext
{
private:
	const String& maContext;

public:
					PackerErrorContext(const String& rContext);
					~PackerErrorContext() {};

	virtual sal_Bool	GetString( sal_uInt32 nErrId, String& rCtxStr );
};

PackerErrorContext::PackerErrorContext(const String& rContext)
: maContext( rContext )
{
}

sal_Bool PackerErrorContext::GetString( sal_uInt32 nErrId, String& rCtxStr )
{
	rCtxStr = maContext;

	return sal_True;
}

class InteractionHandler : public ::cppu::WeakAggImplHelper1< task::XInteractionHandler >
{
public:
	virtual void SAL_CALL handle(const uno::Reference< task::XInteractionRequest > & Request) throw( uno::RuntimeException );
};

void InteractionHandler::handle(const uno::Reference< task::XInteractionRequest > & Request) throw( uno::RuntimeException )
{
	uno::Any aAny( Request->getRequest() );
	if( !aAny.hasValue() || aAny.getValueTypeClass() != uno::TypeClass_EXCEPTION )
	{
		DBG_ERROR( "Illegal task::XInteractionRequest!" );
		return;
	}

	sal_uInt32 nErrorID = ERRCODE_NONE;
	String aCustomErrorString;
	String aErrorContext;
	task::InteractionClassification eClassification = task::InteractionClassification_ERROR;
	sal_Bool bErrorFlags = sal_True;
	uno::Sequence<uno::Reference< task::XInteractionContinuation >  > aContinuations( Request->getContinuations() );
	sal_uInt16 nErrorFlags = 0;
	sal_uInt32 nBoxFlags = WB_OK;


/*	uno::Reference< reflection::XIdlClass >  xReflectionClass( aAny.getReflection()->getIdlClass() );
	if( ::getCppuType((const ClassifiedInteractionRequest*)0)->getIdlClass()->
		isAssignableFrom( xReflectionClass ) )
	{
		ClassifiedInteractionRequest aClassifiedInteractionRequest =
			*(ClassifiedInteractionRequest*)aAny.get();

		eClassification = aClassifiedInteractionRequest.Classification;
	}
*/
	if( aAny.getValueType() == ::getCppuType((const ucb::InteractiveWrongMediumException*)0) )
	{
		ucb::InteractiveWrongMediumException aWMException(
			*(ucb::InteractiveWrongMediumException*)aAny.getValue() );

		aCustomErrorString = String( SdResId(STR_PACKNGO_NEWMEDIUM) );
		sal_Int32 nMedium;
		aWMException.Medium >>= nMedium;
		String aDiskNumber( String::CreateFromInt32( nMedium + 1 ) );
		aCustomErrorString.SearchAndReplaceAscii( "$$", aDiskNumber );
		eClassification = task::InteractionClassification_QUERY;
	}
	else if( aAny.getValueType() == ::getCppuType((const ucb::InteractiveFileIOException*)0) )
	{
		ucb::InteractiveFileIOException aFileIOException(
			*(ucb::InteractiveFileIOException*) aAny.getValue() );

		static const sal_uInt32 aID[ucb::IOErrorCode_WRONG_VERSION + 1]
			= { ERRCODE_IO_ABORT, // ABORT
				ERRCODE_IO_ACCESSDENIED, // ACCESS_DENIED
				ERRCODE_IO_ALREADYEXISTS, // ALREADY_EXISTING
				ERRCODE_IO_BADCRC, // BAD_CRC
				ERRCODE_IO_CANTCREATE, // CANT_CREATE
				ERRCODE_IO_CANTREAD, // CANT_READ
				ERRCODE_IO_CANTSEEK, // CANT_SEEK
				ERRCODE_IO_CANTTELL, // CANT_TELL
				ERRCODE_IO_CANTWRITE, // CANT_WRITE
				ERRCODE_IO_CURRENTDIR, // CURRENT_DIRECTORY
				ERRCODE_IO_DEVICENOTREADY, // DEVICE_NOT_READY
				ERRCODE_IO_NOTSAMEDEVICE, // DIFFERENT_DEVICES
				ERRCODE_IO_GENERAL, // GENERAL
				ERRCODE_IO_INVALIDACCESS, // INVALID_ACCESS
				ERRCODE_IO_INVALIDCHAR, // INVALID_CHARACTER
				ERRCODE_IO_INVALIDDEVICE, // INVALID_DEVICE
				ERRCODE_IO_INVALIDLENGTH, // INVALID_LENGTH
				ERRCODE_IO_INVALIDPARAMETER, // INVALID_PARAMETER
				ERRCODE_IO_ISWILDCARD, // IS_WILDCARD
				ERRCODE_IO_LOCKVIOLATION, // LOCKING_VIOLATION
				ERRCODE_IO_MISPLACEDCHAR, // MISPLACED_CHARACTER
				ERRCODE_IO_NAMETOOLONG, // NAME_TOO_LONG
				ERRCODE_IO_NOTEXISTS, // NOT_EXISTING
				ERRCODE_IO_NOTEXISTSPATH, // NOT_EXISTING_PATH
				ERRCODE_IO_NOTSUPPORTED, // NOT_SUPPORTED
				ERRCODE_IO_NOTADIRECTORY, // NO_DIRECTORY
				ERRCODE_IO_NOTAFILE, // NO_FILE
				ERRCODE_IO_OUTOFSPACE, // OUT_OF_DISK_SPACE
				ERRCODE_IO_TOOMANYOPENFILES, // OUT_OF_FILE_HANDLES
				ERRCODE_IO_OUTOFMEMORY, // OUT_OF_MEMORY
				ERRCODE_IO_PENDING, // PENDING
				ERRCODE_IO_RECURSIVE, // RECURSIVE
				ERRCODE_IO_UNKNOWN, // UNKNOWN
				ERRCODE_IO_WRITEPROTECTED, // WRITE_PROTECTED
				ERRCODE_IO_WRONGFORMAT, // WRONG_FORMAT
				ERRCODE_IO_WRONGVERSION }; // WRONG_VERSION
		nErrorID = aID[aFileIOException.Code];
		bErrorFlags = sal_True;

		aErrorContext = String(SdResId(STR_FILEIOERROR));
		const String aReplace( RTL_CONSTASCII_USTRINGPARAM("($URL1)") );
		aErrorContext.SearchAndReplace( aReplace, aFileIOException.FileName );
	}

	if (bErrorFlags)
	{
		sal_Bool bCanApprove = sal_False;
		sal_Bool bCanDisapprove = sal_False;
		sal_Bool bCanAbort = sal_False;
		sal_Bool bCanRetry = sal_False;
		uno::Reference< uno::XInterface >  xRef;
		for (sal_Int32 i = 0; i < aContinuations.getLength(); ++i)
		{
			uno::Reference< task::XInteractionContinuation > xRef((aContinuations.getArray())[i]);

			uno::Any aAny(  );
			if( xRef->queryInterface( ::getCppuType((const uno::Reference< task::XInteractionApprove >*)0)).hasValue() )
			{
				bCanApprove = sal_True;
			}
			else if( xRef->queryInterface( ::getCppuType((const uno::Reference< task::XInteractionDisapprove >*)0)).hasValue() )
			{
				nErrorFlags |= ERRCODE_BUTTON_NO;
				bCanDisapprove = sal_True;
			}
			else if( xRef->queryInterface( ::getCppuType((const uno::Reference< task::XInteractionAbort >*)0)).hasValue() )
			{
				nErrorFlags |= ERRCODE_BUTTON_CANCEL;
				bCanAbort = sal_True;
			}
			else if( xRef->queryInterface( ::getCppuType((const uno::Reference< task::XInteractionRetry >*)0) ).hasValue() )
			{
				nErrorFlags |= ERRCODE_BUTTON_RETRY;
				bCanRetry = sal_True;
			}
		}
		if (bCanApprove)
			nErrorFlags |= (nErrorFlags & ERRCODE_BUTTON_NO) == 0 ? ERRCODE_BUTTON_OK :
			                                              ERRCODE_BUTTON_YES;
		if (bCanApprove && !bCanAbort )
		{
			nBoxFlags = WB_OK;
			nErrorFlags |= WB_RETRY_CANCEL;
		}
		else if( bCanApprove && bCanAbort )
		{
			nBoxFlags = WB_OK_CANCEL;
			nErrorFlags |= ERRCODE_BUTTON_OK_CANCEL;
		}
		else if( bCanApprove && bCanDisapprove && !bCanAbort )
		{
			nBoxFlags = WB_YES_NO;
			nErrorFlags |= ERRCODE_BUTTON_YES_NO;
		}
		else if( bCanApprove && bCanDisapprove && bCanAbort )
		{
			nBoxFlags = WB_YES_NO_CANCEL;
			nErrorFlags |= ERRCODE_BUTTON_YES_NO_CANCEL;
		}
		else if( bCanAbort && bCanRetry )
		{
			nBoxFlags = WB_RETRY_CANCEL;
			nErrorFlags |= ERRCODE_BUTTON_OK_RETRY_CANCEL;
		}

		switch (eClassification)
		{
			case task::InteractionClassification_ERROR:
				nErrorFlags |= ERRCODE_MSG_ERROR;
				break;

			case task::InteractionClassification_WARNING:
				nErrorFlags |= ERRCODE_MSG_WARNING;
				break;

			case task::InteractionClassification_INFO:
				nErrorFlags |= ERRCODE_MSG_INFO;
				break;

			case task::InteractionClassification_QUERY:
				nErrorFlags |= ERRCODE_MSG_QUERY;
				break;
		}
	}
	
	sal_uInt16 nRC = 0;

	if( aCustomErrorString.Len() != 0 )
	{
		nBoxFlags |= WB_STDDIALOG;
		MessBox* pBox = NULL;
		switch (eClassification)
		{
		case task::InteractionClassification_WARNING:
			pBox = new WarningBox( NULL, nBoxFlags, aCustomErrorString );
			break;	
		case task::InteractionClassification_INFO:
			pBox = new InfoBox( NULL, aCustomErrorString );
			break;	
		case task::InteractionClassification_QUERY:
			pBox = new QueryBox( NULL, nBoxFlags, aCustomErrorString );
			break;	
		case task::InteractionClassification_ERROR:
		default:							
			pBox = new ErrorBox( NULL, nBoxFlags, aCustomErrorString );
		}

		nRC = pBox->Execute();
		delete pBox;
	}
	else
	{
		PackerErrorContext* pContext = NULL;
		if( aErrorContext.Len() != 0 )
			pContext = new PackerErrorContext(aErrorContext);

		nRC = ErrorHandler::HandleError(nErrorID,
									  bErrorFlags ? nErrorFlags : USHRT_MAX);

		delete pContext;
	}

	switch(nRC)
	{
		case ERRCODE_BUTTON_OK:
		case ERRCODE_BUTTON_YES:
			{for (sal_Int32 i = 0; i < aContinuations.getLength(); ++i)
			{
				uno::Reference< task::XInteractionApprove >  xApprove( aContinuations.getArray()[i], uno::UNO_QUERY);
				if (xApprove.is())
				{
					xApprove->select();
					break;
				}
			}}
			break;

		case ERRCODE_BUTTON_CANCEL:
			{for (sal_Int32 i = 0; i < aContinuations.getLength(); ++i)
			{
				uno::Reference< task::XInteractionAbort >  xAbort(aContinuations.getArray()[i], uno::UNO_QUERY);
				if (xAbort.is())
				{
					xAbort->select();
					break;
				}
			}}
			break;

		case ERRCODE_BUTTON_RETRY:
			{for (sal_Int32 i = 0; i < aContinuations.getLength(); ++i)
			{
				uno::Reference< task::XInteractionRetry >  xRetry(aContinuations.getArray()[i], uno::UNO_QUERY);
				if (xRetry.is())
				{
					xRetry->select();
					break;
				}
			}}
			break;

		case ERRCODE_BUTTON_NO:
			{for (sal_Int32 i = 0; i < aContinuations.getLength(); ++i)
			{
				uno::Reference< task::XInteractionDisapprove >  xDisapprove(aContinuations.getArray()[i], uno::UNO_QUERY);
				if (xDisapprove.is())
				{
					xDisapprove->select();
					break;
				}
			}}
			break;
	}

};

uno::Reference< task::XInteractionHandler > InteractionHandler_createInstance()
{
	return new InteractionHandler();
}

