/*************************************************************************
 *
 *  $RCSfile: view.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: aw $ $Date: 2000/10/31 17:51:46 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SVDPAGE_HXX //autogen wg. SdrPaintProcRec
#include <svx/svdpage.hxx>
#endif
#ifndef _SVDPAGV_HXX //autogen wg. SdrPageView
#include <svx/svdpagv.hxx>
#endif
#ifndef _XOUTX_HXX //autogen wg. ExtOutputDevice
#include <svx/xoutx.hxx>
#endif
#ifndef _SVDORECT_HXX //autogen wg. SdrRectObj
#include <svx/svdorect.hxx>
#endif
#ifndef _SDPAGE_HXX //autogen wg. SdPage
#include <sd/sdpage.hxx>
#endif
#ifndef _DRAWDOC_HXX //autogen wg. SdDrawDocument
#include <sd/drawdoc.hxx>
#endif
#ifndef _SVDOGRP_HXX //autogen wg. SdrObjGroup
#include <svx/svdogrp.hxx>
#endif
#ifndef _SVDOGRAF_HXX //autogen wg. SdrGrafObj
#include <svx/svdograf.hxx>
#endif
#ifndef _SVDOOLE2_HXX
#include <svx/svdoole2.hxx>
#endif

#include <toolkit/unohlp.hxx>

#include "view.hxx"
#include "showwin.hxx"
#include "fuslshow.hxx"

using namespace ::com::sun::star;

SipView::SipView( FuSlideShow* pFuSlideShow, SdDrawDocument* pDoc, Window* pWin ) :
	E3dView( pDoc, pWin ),
	m_nPresPaintSmph(0),
	m_pFuSlideShow( pFuSlideShow ),
	m_pDoc(pDoc)
{
}

void  SipView::AllowPresPaint(BOOL bAllowed)
{
	if (bAllowed)
	{
		DBG_ASSERT(m_nPresPaintSmph != 0, "Unterlauf im PaintSemaphor");
		m_nPresPaintSmph--;
	}
	else
	{
		DBG_ASSERT(m_nPresPaintSmph + 1 != 0, "Ueberlauf im PaintSemaphor");
		m_nPresPaintSmph++;
	}
}

void __EXPORT SipView::InitRedraw(OutputDevice* pOutDev, const Region& rReg)
{
	if (m_pFuSlideShow)
	{
		OutputDevice* pShowWindow = (OutputDevice*)m_pFuSlideShow->GetShowWindow();
		if (pShowWindow == pOutDev )
			PresPaint(rReg);
	}
}

void SipView::PresPaint(const Region& rRegion)
{
	if(m_nPresPaintSmph == 0)
	{
		SdShowWindow* pWindow = (SdShowWindow*)GetWin(0);
		MapMode aOldMode( pWindow->GetMapMode() );

		if(m_pFuSlideShow)
			pWindow->SetMapMode( m_pFuSlideShow->GetMasterMapModeWin() );

		// Schwarzer Hintergrund
		Point aPos(pWindow->PixelToLogic(Point(0,0)));
		Size aSize(pWindow->GetOutputSize());
		Rectangle aRect(aPos, aSize);
		Color aBlack(COL_BLACK);
		Color aOldFillColor( pWindow->GetFillColor() );
		ULONG nOldDrawMode( pWindow->GetDrawMode() );

		pWindow->SetDrawMode( DRAWMODE_DEFAULT );
		pWindow->SetFillColor( aBlack );
		pWindow->DrawRect(aRect);
		pWindow->SetFillColor( aOldFillColor );
		pWindow->SetDrawMode( nOldDrawMode );
		
		// Clipping auf angezeigten Seitenbereich
		SdPage*   pPage 		= m_pDoc->GetSdPage(0, PK_STANDARD);
		BOOL	  bIsClipRegion = pWindow->IsClipRegion();
		Region	  aOldClipRegion(pWindow->GetClipRegion());

		Point aUpperLeft(pPage->GetLftBorder(),
						 pPage->GetUppBorder());
		Point aLowerRight(pPage->GetSize().Width() -
						  pPage->GetRgtBorder(),
						  pPage->GetSize().Height() -
						  pPage->GetLwrBorder());
		Rectangle aClipRect(aUpperLeft, aLowerRight);

		// untere und rechte Pixelreihe nicht darstellen
		Size aPixelSize(pWindow->PixelToLogic(Size(1,1)));
		aClipRect.Bottom() -= aPixelSize.Height();
		aClipRect.Right()  -= aPixelSize.Width();

		Region	  aClipRegion(aClipRect);
		pWindow->SetClipRegion(aClipRegion);

		Link aPaintProcLink = LINK(this, SipView, PaintProc);
		SdrPageView* pPageView = GetPageViewPvNum(0);

		if (pPageView)
			pPageView->InitRedraw((USHORT)0, rRegion, 0, &aPaintProcLink);

		// altes Clipping restaurieren
		if (bIsClipRegion)
			pWindow->SetClipRegion(aOldClipRegion);
		else
			pWindow->SetClipRegion();

		pWindow->SetMapMode( aOldMode );
	}
}

/*************************************************************************
|*
|* Paint-Proc filtert fuer die Praesentation Objekte heraus, die unsichtbar
|* sind oder anders dargestellt werden muessen.
|*
\************************************************************************/

IMPL_LINK( SipView, PaintProc, SdrPaintProcRec *, pRecord )
{
	SdAnimationInfo* pInfo = m_pDoc->GetAnimationInfo(pRecord->pObj);

	if( !pRecord->pObj->IsEmptyPresObj() )
	{
		BOOL		bDrawn = FALSE;

		if( pInfo && pInfo->bIsShown )
		{
			const BOOL bDimmed = pInfo->bDimmed;

			if( pRecord->pObj == m_pFuSlideShow->GetLayoutText() )
			{
				m_pFuSlideShow->PaintLayoutClone(pRecord->rOut.GetOutDev());
				bDrawn = TRUE;
			}
			else if( pInfo->bIsMovie )
			{
				if( bDimmed )
				{
					m_pFuSlideShow->PaintDimmedObject(((SdrAttrObj*) pRecord->pObj), pRecord->rOut.GetOutDev(), pInfo->aDimColor, TRUE);
				}
				else
				{
					// nur das letzte Bild
					SdrObjList* pObjList = ((SdrObjGroup*)pRecord->pObj)->GetSubList();
					SdrObject* pLast = (SdrObject*)pObjList->GetObj(pObjList->GetObjCount() - 1);
					pLast->Paint(pRecord->rOut, pRecord->rInfoRec);
				}

				bDrawn = TRUE;
			}
			else if( bDimmed )
			{
				m_pFuSlideShow->PaintDimmedObject(((SdrAttrObj*)pRecord->pObj), pRecord->rOut.GetOutDev(), pInfo->aDimColor, TRUE);
				bDrawn = TRUE;
			}
		}
		else if( pInfo && !pInfo->bIsShown )
			bDrawn = TRUE;

		if( !bDrawn )
		{
			const BOOL bGrafAnim = OBJIS_GRAFANIM( pRecord->pObj );
			const BOOL bTextAnim = OBJIS_TEXTANIM( pRecord->pObj );

			// falls es sich um ein animiertes Objekt handelt,
			// das nicht animiert dargestellt werden soll, muessen
			// wir ein nicht animiertes Clone-Objekt ausgeben
			if( !m_pFuSlideShow->IsAnimationAllowed() && ( bGrafAnim || bTextAnim ) )
			{
				SdrObject* pClone = pRecord->pObj->Clone();

				if( bGrafAnim )
					( (SdrGrafObj*) pClone )->SetGraphic( ( (SdrGrafObj*) pRecord->pObj )->GetGraphic().GetBitmapEx() );

				if( bTextAnim )
				{
//-/					SfxItemSet aTempAttr( m_pDoc->GetPool(), SDRATTR_TEXT_ANIKIND, SDRATTR_TEXT_ANIKIND );
//-/					aTempAttr.InvalidateItem( SDRATTR_TEXT_ANIKIND );
//-/					aTempAttr.Put( SdrTextAniKindItem() );
//-/					pClone->NbcSetAttributes( aTempAttr, FALSE );
					pClone->ClearItem(SDRATTR_TEXT_ANIKIND);
					pClone->SetItem(SdrTextAniKindItem());
				}

				pClone->Paint( pRecord->rOut, pRecord->rInfoRec );
				delete pClone;
			}
			else
				pRecord->pObj->Paint( pRecord->rOut, pRecord->rInfoRec );
		}
	}
	// das Hintergrundrechteck gibt sich faelschlicherweise als EmptyPresObj
	// aus, das kann auch nicht geaendert werden, da es sonst von anderen
	// Methoden nicht erkannt wird und neue Hintergrundrechtecke erzeugt
	// werden --> sieht aus wie n-faches Redraw des Hintergrunds
	// (siehe auch FuSlideShow::PaintProc)
	else
	{
		SdPage* pPage = (SdPage*)pRecord->pObj->GetPage();
		if (pPage->GetPresObj(PRESOBJ_BACKGROUND) == pRecord->pObj)
			   pRecord->pObj->Paint(pRecord->rOut, pRecord->rInfoRec);
	}


	return 0;
}

void SipView::DoConnect(SdrOle2Obj* pOleObj)
{
	if( pOleObj == NULL )
		return;

	SvInPlaceObjectRef xIPO = pOleObj->GetObjRef();

	uno::Reference< awt::XWindowPeer > xPeer( VCLUnoHelper::GetInterface( (Window*)GetWin(0) ), uno::UNO_QUERY );
	if( xPeer.is() )
		xIPO->CreatePlugin( xPeer->getToolkit(), xPeer );
}


