/*************************************************************************
 *
 *  $RCSfile: filepicker.cxx,v $
 *
 *  $Revision: 1.6 $
 *
 *  last change: $Author: dv $ $Date: 2001/07/25 15:41:55 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "filepicker.hxx"
#include "iodlg.hxx"

#include <list>
#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

#define _SVSTDARR_STRINGSDTOR
#include "svstdarr.hxx"

#ifndef  _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif

#ifndef  _COM_SUN_STAR_CONTAINER_XCONTENTENUMERATIONACCESS_HPP_
#include <com/sun/star/container/XContentEnumerationAccess.hpp>
#endif
#ifndef  _COM_SUN_STAR_CONTAINER_XSET_HPP_
#include <com/sun/star/container/XSet.hpp>
#endif

#ifndef  _COM_SUN_STAR_UI_DIALOGS_FILEPICKEREVENT_HPP_
#include <com/sun/star/ui/dialogs/FilePickerEvent.hpp>
#endif
#ifndef  _COM_SUN_STAR_UI_DIALOGS_FILEPREVIEWIMAGEFORMATS_HPP_
#include <com/sun/star/ui/dialogs/FilePreviewImageFormats.hpp>
#endif
#ifndef  _COM_SUN_STAR_UI_DIALOGS_TEMPLATEDESCRIPTION_HPP_
#include <com/sun/star/ui/dialogs/TemplateDescription.hpp>
#endif

#ifndef  _CPPUHELPER_FACTORY_HXX_
#include <cppuhelper/factory.hxx>
#endif

#ifndef _UNOTOOLS_UCBHELPER_HXX
#include <unotools/ucbhelper.hxx>
#endif

#ifndef INCLUDED_SVTOOLS_PATHOPTIONS_HXX
#include <pathoptions.hxx>
#endif
#ifndef INCLUDED_SVTOOLS_MISCOPT_HXX
#include <miscopt.hxx>
#endif

// define ----------------------------------------------------------------

#define MAKE_ANY    ::com::sun::star::uno::makeAny

// using ----------------------------------------------------------------

using namespace     ::com::sun::star::container;
using namespace     ::com::sun::star::lang;
using namespace     ::com::sun::star::ui::dialogs;
using namespace     ::com::sun::star::uno;
using namespace     ::rtl;
using namespace     ::utl;

// struct FilterEntry_Impl -----------------------------------------------

struct FilterEntry_Impl
{
	UNOOUSTRING			m_aTitle;
	UNOOUSTRING			m_aFilter;

	FilterEntry_Impl( const UNOOUSTRING& rTitle, const UNOOUSTRING& rFilter ) :
		m_aTitle( rTitle ), m_aFilter( rFilter ) {}
};

typedef std::list < FilterEntry_Impl > FilterList;

// struct ElementEntry_Impl ----------------------------------------------

struct ElementEntry_Impl
{
	sal_Int16		m_nElementID;
	sal_Int16		m_nControlAction;
	UNOANY			m_aValue;
	UNOOUSTRING		m_aLabel;
    sal_Bool        m_bEnabled      : 1;

    sal_Bool        m_bHasValue     : 1;
    sal_Bool        m_bHasLabel     : 1;
    sal_Bool        m_bHasEnabled   : 1;

                    ElementEntry_Impl( sal_Int16 nId );

    void            setValue( const UNOANY& rVal ) { m_aValue = rVal; m_bHasValue = sal_True; }
    void            setAction( sal_Int16 nAction ) { m_nControlAction = nAction; }
    void            setLabel( const UNOOUSTRING& rVal ) { m_aLabel = rVal; m_bHasLabel = sal_True; }
    void            setEnabled( sal_Bool bEnabled ) { m_bEnabled = bEnabled; m_bHasEnabled = sal_True; }
};

typedef std::list < ElementEntry_Impl > ElementList;

ElementEntry_Impl::ElementEntry_Impl( sal_Int16 nId )
    : m_nElementID( nId )
    , m_nControlAction( 0 )
    , m_bEnabled( sal_False )
    , m_bHasValue( sal_False )
    , m_bHasLabel( sal_False )
    , m_bHasEnabled( sal_False )
{}

//------------------------------------------------------------------------------------
// SvtFilePicker_Impl
//------------------------------------------------------------------------------------

class SvtFilePicker_Impl
{
public:
	SvtFileDialog_Impl*	m_pDlg;
	FilterList*			m_pFilterList;
	ElementList*		m_pElemList;
	SvtFilePicker*		m_pOwner;

	sal_Bool			m_bMultiSelection;
    sal_Int16           m_nServiceType;
	UNOOUSTRING			m_aTitle;
	UNOOUSTRING			m_aDefaultName;
	UNOOUSTRING			m_aDisplayDirectory;
	UNOOUSTRING			m_aCurrentFilter;

	Reference< XFilePickerListener >	m_xListener;

public:
						SvtFilePicker_Impl( SvtFilePicker* pOwner );
						~SvtFilePicker_Impl();

	sal_Int16			execute();
	ULONG				getWinBits( ULONG& rExtraBits );
	void				notify( short nEventId, short nControlId );
	void				clearOwner() { m_pOwner = NULL; }

    sal_Bool            FilterNameExists( const UNOOUSTRING& rTitle );
};

//-----------------------------------------------------------------------------
SvtFilePicker_Impl::SvtFilePicker_Impl( SvtFilePicker* pOwner ) :
	m_pDlg				( NULL ),
	m_pFilterList		( NULL ),
	m_pElemList			( NULL ),
	m_bMultiSelection	( sal_False ),
	m_pOwner			( pOwner )
{
}

//------------------------------------------------------------------------------------
SvtFilePicker_Impl::~SvtFilePicker_Impl()
{
	if ( m_pFilterList && !m_pFilterList->empty() )
		m_pFilterList->erase( m_pFilterList->begin(), m_pFilterList->end() );
	delete m_pFilterList;

	if ( m_pElemList && !m_pElemList->empty() )
		m_pElemList->erase( m_pElemList->begin(), m_pElemList->end() );
	delete m_pElemList;

	delete m_pDlg;
}

//------------------------------------------------------------------------------------
ULONG SvtFilePicker_Impl::getWinBits( ULONG& rExtraBits )
{
	// set the winbits for creating the filedialog
	ULONG nBits = 0L;
    rExtraBits = 0L;

	// set the standard bits acording to the service name
    if ( m_nServiceType == TemplateDescription::FILEOPEN_SIMPLE )
	{
		nBits = WB_OPEN;
	}
	else if ( m_nServiceType == TemplateDescription::FILESAVE_SIMPLE )
	{
		nBits = WB_SAVEAS;
	}
	else if ( m_nServiceType == TemplateDescription::FILESAVE_AUTOEXTENSION_PASSWORD )
	{
		nBits = WB_SAVEAS | SFXWB_PASSWORD;
        rExtraBits = SFX_EXTRA_AUTOEXTENSION;
	}
    else if ( m_nServiceType == TemplateDescription::FILESAVE_AUTOEXTENSION_PASSWORD_FILTEROPTIONS )
	{
		nBits = WB_SAVEAS | SFXWB_PASSWORD;
        rExtraBits = SFX_EXTRA_AUTOEXTENSION | SFX_EXTRA_FILTEROPTIONS;
	}
    else if ( m_nServiceType == TemplateDescription::FILESAVE_AUTOEXTENSION_TEMPLATE )
	{
		nBits = WB_SAVEAS;
        rExtraBits = SFX_EXTRA_AUTOEXTENSION | SFX_EXTRA_TEMPLATES;
	}
    else if ( m_nServiceType == TemplateDescription::FILESAVE_AUTOEXTENSION_SELECTION )
	{
		nBits = WB_SAVEAS;
        rExtraBits = SFX_EXTRA_AUTOEXTENSION | SFX_EXTRA_SELECTION;
    }

    else if ( m_nServiceType == TemplateDescription::FILEOPEN_LINK_PREVIEW_IMAGE_TEMPLATE )
	{
		nBits = WB_OPEN;
        rExtraBits = SFX_EXTRA_INSERTASLINK | SFX_EXTRA_SHOWPREVIEW | SFX_EXTRA_IMAGE_TEMPLATE;
	}
    else if ( m_nServiceType == TemplateDescription::FILEOPEN_PLAY )
	{
		nBits = WB_OPEN;
        rExtraBits = SFX_EXTRA_PLAYBUTTON;
    }
    else if ( m_nServiceType == TemplateDescription::FILEOPEN_READONLY_VERSION )
	{
		nBits = WB_OPEN | SFXWB_READONLY;
        rExtraBits = SFX_EXTRA_SHOWVERSIONS;
	}
    else if ( m_nServiceType == TemplateDescription::FILEOPEN_LINK_PREVIEW )
	{
		nBits = WB_OPEN;
        rExtraBits = SFX_EXTRA_INSERTASLINK | SFX_EXTRA_SHOWPREVIEW;
	}
	if ( m_bMultiSelection && ( ( nBits & WB_OPEN ) == WB_OPEN ) )
		nBits |= SFXWB_MULTISELECTION;

	return nBits;
}

//------------------------------------------------------------------------------------
sal_Int16 SvtFilePicker_Impl::execute()
{
	ULONG               nExtraBits;
    ULONG				nBits = getWinBits( nExtraBits );

	if ( ! m_pDlg )
        m_pDlg = new SvtFileDialog_Impl( nBits, nExtraBits );

	FileNotifyHelper_Impl *pNotifier = new FileNotifyHelper_Impl( this );
	m_pDlg->SetFileCallback( pNotifier );

	// set the title
	if ( m_aTitle.getLength() > 0 )
		m_pDlg->SetText( m_aTitle );

	// set the default directory
	// --**-- doesn't match the spec yet
	if ( m_aDisplayDirectory.getLength() > 0 || m_aDefaultName.getLength() > 0 )
	{
		if ( m_aDisplayDirectory.getLength() > 0 )
		{

			INetURLObject aPath( m_aDisplayDirectory );
			if ( m_aDefaultName.getLength() > 0 )
				aPath.insertName( m_aDefaultName );
			m_pDlg->SetPath( aPath.GetMainURL( INetURLObject::NO_DECODE ) );
		}
		else if ( m_aDefaultName.getLength() > 0 )
			m_pDlg->SetPath( m_aDefaultName );
	}
	else
	{
		// Default-Standard-Dir setzen
		INetURLObject aStdDirObj( SvtPathOptions().GetWorkPath() );
		m_pDlg->SetPath( aStdDirObj.GetMainURL( INetURLObject::NO_DECODE ) );
	}

	// set the control values and set the control labels, too
    if ( m_pElemList && !m_pElemList->empty() )
    {
		ElementList::iterator aListIter;
		for ( aListIter = m_pElemList->begin();
			  aListIter != m_pElemList->end(); ++aListIter )
		{
			ElementEntry_Impl& rEntry = *aListIter;
            if ( rEntry.m_bHasValue )
                m_pDlg->SetValue( rEntry.m_nElementID, rEntry.m_nControlAction, rEntry.m_aValue );
            if ( rEntry.m_bHasLabel )
                m_pDlg->SetLabel( rEntry.m_nElementID, rEntry.m_aLabel );
            if ( rEntry.m_bHasEnabled )
                m_pDlg->EnableControl( rEntry.m_nElementID, rEntry.m_bEnabled );
		}
    }
    
    if ( m_pFilterList && !m_pFilterList->empty() )
	{
		FilterList::iterator aListIter;
		for ( aListIter = m_pFilterList->begin();
			  aListIter != m_pFilterList->end(); ++aListIter )
		{
			FilterEntry_Impl& rEntry = *aListIter;
			m_pDlg->AddFilter( rEntry.m_aTitle, rEntry.m_aFilter );
		}
	}

	// set the default filter
	if ( m_aCurrentFilter.getLength() > 0 )
		m_pDlg->SetCurFilter( m_aCurrentFilter );

	// now we are ready to execute the dialog
	sal_Int16 nRet = m_pDlg->Execute();

	m_pDlg->SetFileCallback( NULL );
	delete pNotifier;

	return nRet;
}

//------------------------------------------------------------------------------------
void SvtFilePicker_Impl::notify( short nEventId, short nControlId )
{
	if ( !m_xListener.is() || !m_pOwner )
		return;

    FilePickerEvent aEvent( *m_pOwner, nControlId );

	switch ( nEventId ) {
	case FILE_SELECTION_CHANGED:
		m_xListener->fileSelectionChanged( aEvent );
		break;
	case DIRECTORY_CHANGED:
		m_xListener->directoryChanged( aEvent );
		break;
	case HELP_REQUESTED:
		m_xListener->helpRequested( aEvent );
		break;
	case CTRL_STATE_CHANGED:
		m_xListener->controlStateChanged( aEvent );
		break;
	case DIALOG_SIZE_CHANGED:
		m_xListener->dialogSizeChanged();
		break;
	default:
		DBG_ERRORFILE( "SvtFilePicker_Impl::notify(): Unknown event id!" );
	}
}

//------------------------------------------------------------------------------------
sal_Bool SvtFilePicker_Impl::FilterNameExists( const UNOOUSTRING& rTitle )
{
    sal_Bool bRet = sal_False;

    if ( m_pFilterList && !m_pFilterList->empty() )
	{
		FilterList::iterator aListIter;
		for ( aListIter = m_pFilterList->begin();
			  aListIter != m_pFilterList->end(); ++aListIter )
		{
			FilterEntry_Impl& rEntry = *aListIter;
			if ( rEntry.m_aTitle == rTitle )
            {
                bRet = sal_True;
                break;
            }
		}
	}

    return bRet;
}


//------------------------------------------------------------------------------------
// class SvtFilePicker
//------------------------------------------------------------------------------------

SvtFilePicker::SvtFilePicker( const Reference < XMultiServiceFactory >& xFactory )
{
	m_pImpl = new SvtFilePicker_Impl( this ); 
}

SvtFilePicker::~SvtFilePicker()
{
	m_pImpl->clearOwner();

	delete m_pImpl;
}

//------------------------------------------------------------------------------------
// XExecutableDialog functions
//------------------------------------------------------------------------------------

void SAL_CALL SvtFilePicker::setTitle( const UNOOUSTRING& aTitle ) throw( UNORUNTIMEEXCEPTION )
{
	m_pImpl->m_aTitle = aTitle;
}

//------------------------------------------------------------------------------------
sal_Int16 SAL_CALL SvtFilePicker::execute() throw( UNORUNTIMEEXCEPTION )
{
	return m_pImpl->execute();
}

//------------------------------------------------------------------------------------
// XFilePicker functions
//------------------------------------------------------------------------------------

void SAL_CALL SvtFilePicker::setMultiSelectionMode( sal_Bool bMode ) throw( UNORUNTIMEEXCEPTION )
{
	m_pImpl->m_bMultiSelection = bMode;
}

void SAL_CALL SvtFilePicker::setDefaultName( const UNOOUSTRING& aName ) throw( UNOILLEGALARGUMENTEXCEPTION, UNORUNTIMEEXCEPTION )
{
	m_pImpl->m_aDefaultName = aName;
}

void SAL_CALL SvtFilePicker::setDisplayDirectory( const UNOOUSTRING& aDirectory )
    throw( UNOILLEGALARGUMENTEXCEPTION, UNORUNTIMEEXCEPTION )
{
	m_pImpl->m_aDisplayDirectory = aDirectory;
}

UNOOUSTRING SAL_CALL SvtFilePicker::getDisplayDirectory() throw( UNORUNTIMEEXCEPTION )
{
	if ( m_pImpl->m_pDlg )
    {
        UNOOUSTRING aPath = m_pImpl->m_pDlg->GetPath();

        if( !UCBContentHelper::IsFolder( aPath ) )
        {
            INetURLObject aFolder( aPath );
            aFolder.CutLastName();
            aPath = aFolder.GetMainURL( INetURLObject::NO_DECODE );
        }
        return aPath;
    }
    else
        return m_pImpl->m_aDisplayDirectory;
}

UNOSEQUENCE< UNOOUSTRING > SAL_CALL SvtFilePicker::getFiles() throw( UNORUNTIMEEXCEPTION )
{
	if ( ! m_pImpl->m_pDlg )
	{
		UNOSEQUENCE< UNOOUSTRING > aEmpty;
		return aEmpty;
	}

	// if there is more than one path we have to return the path to the
	// files first and then the list of the selected entries

	SvStringsDtor* pPathList = m_pImpl->m_pDlg->GetPathList();
	USHORT i, nCount = pPathList->Count();
	USHORT nTotal = nCount > 1 ? nCount+1: nCount;

	UNOSEQUENCE< UNOOUSTRING > aPath( nTotal );

	if ( nCount == 1 )
		aPath[0] = UNOOUSTRING( *pPathList->GetObject( 0 ) );
	else if ( nCount > 1 )
	{
		INetURLObject aObj( *pPathList->GetObject( 0 ) );
		aObj.removeSegment();
		aPath[0] = aObj.GetMainURL( INetURLObject::NO_DECODE );
			
		for ( i = 0; i < nCount; /* i++ is done below */ )
		{
			aObj.SetURL( *pPathList->GetObject(i++) );
			aPath[i] = aObj.getName();
		}
	}

	delete pPathList;
	return aPath;
}

//------------------------------------------------------------------------------------
// XFilePickerControlAccess functions
//------------------------------------------------------------------------------------

void SAL_CALL SvtFilePicker::setValue( sal_Int16 nElementID,
                                       sal_Int16 nControlAction,
                                       const UNOANY& rValue )
    throw( UNORUNTIMEEXCEPTION )
{
	if ( m_pImpl->m_pDlg )
	{
		m_pImpl->m_pDlg->SetValue( nElementID, nControlAction, rValue );
	}
	else
	{
		if ( !m_pImpl->m_pElemList )
			m_pImpl->m_pElemList = new ElementList;
		
        sal_Bool bFound = sal_False;
		ElementList::iterator aListIter;

        for ( aListIter = m_pImpl->m_pElemList->begin();
			  aListIter != m_pImpl->m_pElemList->end(); ++aListIter )
		{
			ElementEntry_Impl& rEntry = *aListIter;
            if ( ( rEntry.m_nElementID == nElementID ) &&
                 ( !rEntry.m_bHasValue || ( rEntry.m_nControlAction == nControlAction ) ) )
            {
                rEntry.setAction( nControlAction );
                rEntry.setValue( rValue );
                bFound = sal_True;
            }
		}

        if ( !bFound )
        {
            ElementEntry_Impl aNew( nElementID );
            aNew.setAction( nControlAction );
            aNew.setValue( rValue );
            m_pImpl->m_pElemList->insert( m_pImpl->m_pElemList->end(), aNew );
        }
	}
}

//------------------------------------------------------------------------------------

UNOANY SAL_CALL SvtFilePicker::getValue( sal_Int16 nElementID, sal_Int16 nControlAction )
    throw( UNORUNTIMEEXCEPTION )
{
	UNOANY      aAny;

	// execute() called?
	if ( m_pImpl->m_pDlg )
	{
		aAny = m_pImpl->m_pDlg->GetValue( nElementID, nControlAction );
	}
    else if ( m_pImpl->m_pElemList && !m_pImpl->m_pElemList->empty() )
    {
		ElementList::iterator aListIter;
		for ( aListIter = m_pImpl->m_pElemList->begin();
			  aListIter != m_pImpl->m_pElemList->end(); ++aListIter )
		{
			ElementEntry_Impl& rEntry = *aListIter;
            if ( ( rEntry.m_nElementID == nElementID ) &&
                 ( rEntry.m_bHasValue ) &&
                 ( rEntry.m_nControlAction == nControlAction ) )
            {
                aAny = rEntry.m_aValue;
                break;
            }
		}
    }

    return aAny;
}


//------------------------------------------------------------------------------------
void SAL_CALL SvtFilePicker::setLabel( sal_Int16 nLabelID, const UNOOUSTRING& rValue )
    throw ( UNORUNTIMEEXCEPTION )
{
	if ( m_pImpl->m_pDlg )
	{
		m_pImpl->m_pDlg->SetLabel( nLabelID, rValue );
	}
	else
	{
		if ( !m_pImpl->m_pElemList )
			m_pImpl->m_pElemList = new ElementList;
		
        sal_Bool bFound = sal_False;
		ElementList::iterator aListIter;

        for ( aListIter = m_pImpl->m_pElemList->begin();
			  aListIter != m_pImpl->m_pElemList->end(); ++aListIter )
		{
			ElementEntry_Impl& rEntry = *aListIter;
            if ( rEntry.m_nElementID == nLabelID )
            {
                rEntry.setLabel( rValue );
                bFound = sal_True;
            }
		}

        if ( !bFound )
        {
            ElementEntry_Impl aNew( nLabelID );
            aNew.setLabel( rValue );
            m_pImpl->m_pElemList->insert( m_pImpl->m_pElemList->end(), aNew );
        }
	}
}

//------------------------------------------------------------------------------------
UNOOUSTRING SAL_CALL SvtFilePicker::getLabel( sal_Int16 nLabelID )
    throw ( UNORUNTIMEEXCEPTION )
{
	UNOOUSTRING aLabel;

    if ( m_pImpl->m_pDlg )
	{
		aLabel = m_pImpl->m_pDlg->GetLabel( nLabelID );
	}
    else if ( m_pImpl->m_pElemList && !m_pImpl->m_pElemList->empty() )
    {
		ElementList::iterator aListIter;
		for ( aListIter = m_pImpl->m_pElemList->begin();
			  aListIter != m_pImpl->m_pElemList->end(); ++aListIter )
		{
			ElementEntry_Impl& rEntry = *aListIter;
            if ( rEntry.m_nElementID == nLabelID )
            {
                if ( rEntry.m_bHasLabel )
                    aLabel = rEntry.m_aLabel;
                break;
            }
		}
    }

	return aLabel;
}

//------------------------------------------------------------------------------------
void SAL_CALL SvtFilePicker::enableControl( sal_Int16 nElementID, sal_Bool bEnable )
    throw( UNORUNTIMEEXCEPTION )
{
    if ( m_pImpl->m_pDlg )
	{
		m_pImpl->m_pDlg->EnableControl( nElementID, bEnable );
	}
    else
    {
		if ( !m_pImpl->m_pElemList )
			m_pImpl->m_pElemList = new ElementList;
		
        sal_Bool bFound = sal_False;
		ElementList::iterator aListIter;

        for ( aListIter = m_pImpl->m_pElemList->begin();
			  aListIter != m_pImpl->m_pElemList->end(); ++aListIter )
		{
			ElementEntry_Impl& rEntry = *aListIter;
            if ( rEntry.m_nElementID == nElementID )
            {
                rEntry.setEnabled( bEnable );
                bFound = sal_True;
            }
		}

        if ( !bFound )
        {
            ElementEntry_Impl aNew( nElementID );
            aNew.setEnabled( bEnable );
            m_pImpl->m_pElemList->insert( m_pImpl->m_pElemList->end(), aNew );
        }
    }
}

//------------------------------------------------------------------------------------
// XFilePickerNotifier functions
//------------------------------------------------------------------------------------

void SAL_CALL SvtFilePicker::addFilePickerListener( const Reference< XFilePickerListener >& xListener ) throw ( UNORUNTIMEEXCEPTION )
{
	m_pImpl->m_xListener = xListener;
}

//------------------------------------------------------------------------------------
void SAL_CALL SvtFilePicker::removeFilePickerListener( const Reference< XFilePickerListener >& xListener ) throw ( UNORUNTIMEEXCEPTION )
{
	m_pImpl->m_xListener.clear();
}

//------------------------------------------------------------------------------------
// XFilePreview functions
//------------------------------------------------------------------------------------

UNOSEQUENCE< sal_Int16 > SAL_CALL SvtFilePicker::getSupportedImageFormats()
    throw ( UNORUNTIMEEXCEPTION )
{
    UNOSEQUENCE< sal_Int16 > aFormats( 1 );

    aFormats[0] = FilePreviewImageFormats::BITMAP;

    return aFormats;
}

//------------------------------------------------------------------------------------
sal_Int32 SAL_CALL SvtFilePicker::getTargetColorDepth() throw ( UNORUNTIMEEXCEPTION )
{
    sal_Int32 nDepth = 0;
    
    if ( m_pImpl->m_pDlg )
        nDepth = m_pImpl->m_pDlg->getTargetColorDepth();

    return nDepth;
}

//------------------------------------------------------------------------------------
sal_Int32 SAL_CALL SvtFilePicker::getAvailableWidth() throw ( UNORUNTIMEEXCEPTION )
{
    sal_Int32 nWidth = 0;
    
    if ( m_pImpl->m_pDlg )
        nWidth = m_pImpl->m_pDlg->getAvailableWidth();

    return nWidth;
}

//------------------------------------------------------------------------------------
sal_Int32 SAL_CALL SvtFilePicker::getAvailableHeight() throw ( UNORUNTIMEEXCEPTION )
{
    sal_Int32 nHeigth = 0;
    
    if ( m_pImpl->m_pDlg )
        nHeigth = m_pImpl->m_pDlg->getAvailableHeight();

    return nHeigth;
}

//------------------------------------------------------------------------------------
void SAL_CALL SvtFilePicker::setImage( sal_Int16 aImageFormat, const UNOANY& rImage )
    throw ( UNOILLEGALARGUMENTEXCEPTION, UNORUNTIMEEXCEPTION )
{
    if ( m_pImpl->m_pDlg )
        m_pImpl->m_pDlg->setImage( aImageFormat, rImage );
}

//------------------------------------------------------------------------------------
sal_Bool SAL_CALL SvtFilePicker::setShowState( sal_Bool bShowState )
    throw ( UNORUNTIMEEXCEPTION )
{
    sal_Bool bRet = sal_False;

    if ( m_pImpl->m_pDlg )
        bRet = m_pImpl->m_pDlg->setShowState( bShowState );

    return bRet;
}

//------------------------------------------------------------------------------------
sal_Bool SAL_CALL SvtFilePicker::getShowState() throw ( UNORUNTIMEEXCEPTION )
{
    sal_Bool bRet = sal_False;

    if ( m_pImpl->m_pDlg )
        bRet = m_pImpl->m_pDlg->getShowState();

    return bRet;
}

//------------------------------------------------------------------------------------
// XFilterManager functions
//------------------------------------------------------------------------------------

void SAL_CALL SvtFilePicker::appendFilter( const UNOOUSTRING& aTitle,
                                           const UNOOUSTRING& aFilter )
    throw( UNOILLEGALARGUMENTEXCEPTION, UNORUNTIMEEXCEPTION )
{
	if ( !m_pImpl->m_pFilterList )
    {
		m_pImpl->m_pFilterList = new FilterList;
        
        // set the first filter to the current filter
        if ( ! m_pImpl->m_aCurrentFilter.getLength() )
            m_pImpl->m_aCurrentFilter = aTitle;
    }

    if ( m_pImpl->FilterNameExists( aTitle ) )
        throw UNOILLEGALARGUMENTEXCEPTION();

	m_pImpl->m_pFilterList->insert( m_pImpl->m_pFilterList->end(), FilterEntry_Impl( aTitle, aFilter ) );
}

//------------------------------------------------------------------------------------
void SAL_CALL SvtFilePicker::setCurrentFilter( const UNOOUSTRING& aTitle )
    throw( UNOILLEGALARGUMENTEXCEPTION, UNORUNTIMEEXCEPTION )
{
    if ( ! m_pImpl->FilterNameExists( aTitle ) )
        throw UNOILLEGALARGUMENTEXCEPTION();

	m_pImpl->m_aCurrentFilter = aTitle;

    if ( m_pImpl->m_pDlg )
        m_pImpl->m_pDlg->SetCurFilter( aTitle );
}

//------------------------------------------------------------------------------------
UNOOUSTRING SAL_CALL SvtFilePicker::getCurrentFilter()
    throw( UNORUNTIMEEXCEPTION )
{
	UNOOUSTRING aFilter = m_pImpl->m_pDlg ? UNOOUSTRING( m_pImpl->m_pDlg->GetCurFilter() ) :
                                            UNOOUSTRING( m_pImpl->m_aCurrentFilter );
	return aFilter;
}


//------------------------------------------------------------------------------------
// XInitialization functions
//------------------------------------------------------------------------------------

void SAL_CALL SvtFilePicker::initialize( const UNOSEQUENCE< UNOANY >& rArguments )
	throw ( UNOEXCEPTION, UNORUNTIMEEXCEPTION )
{
	DBG_ASSERT( rArguments.getLength() == 1, "SvtFilePicker::initialize() : Wrong number of arguments!" );
	
	if ( rArguments.getLength() )
	{
		sal_Int16 nType;

		if ( rArguments[0] >>= nType )
			m_pImpl->m_nServiceType = nType;
	}
}

// ------------------------------------------------------------------------
// misc. helpers
//------------------------------------------------------------------------------------
#define SYSTEM_FILE_OPEN_SERVICE_NAME "com.sun.star.ui.dialogs.SystemFilePicker"

sal_Bool SvtFilePicker::HasSystemFilePicker( const Reference< XMultiServiceFactory > xFactory )
{
    sal_Bool bRet = sal_False;
	
	Reference< XContentEnumerationAccess > xEnumAccess( xFactory, UNO_QUERY );
    Reference< XSet > xSet( xFactory, UNO_QUERY );

    if ( ! xEnumAccess.is() || ! xSet.is() )
        return bRet;
    
    try
    {
    	OUString aFileService( RTL_CONSTASCII_USTRINGPARAM( SYSTEM_FILE_OPEN_SERVICE_NAME ) );

        Reference< XEnumeration > xEnum = xEnumAccess->createContentEnumeration( aFileService );
        if ( xEnum.is() && xEnum->hasMoreElements() )
            bRet = sal_True;
    }

    catch( IllegalArgumentException ) {}
    catch( ElementExistException ) {}

    return bRet;
}

//------------------------------------------------------------------------------------
sal_Bool SvtFilePicker::UseSystemFilePicker()
{
    SvtMiscOptions aOptions;

    sal_Bool bRet = aOptions.UseSystemFileDialog();

    return bRet;
}

//------------------------------------------------------------------------------------
// XServiceInfo
//------------------------------------------------------------------------------------

/* XServiceInfo */
UNOOUSTRING SAL_CALL SvtFilePicker::getImplementationName() throw( UNORUNTIMEEXCEPTION )
{
	return impl_getStaticImplementationName();
}

/* XServiceInfo */
sal_Bool SAL_CALL SvtFilePicker::supportsService( const UNOOUSTRING& sServiceName ) throw( UNORUNTIMEEXCEPTION )
{
    UNOSEQUENCE< UNOOUSTRING > seqServiceNames = getSupportedServiceNames();
    const UNOOUSTRING* pArray = seqServiceNames.getConstArray();
    for ( sal_Int32 i = 0; i < seqServiceNames.getLength(); i++ )
	{
        if ( sServiceName == pArray[i] )
		{
            return sal_True ;
		}
	}
    return sal_False ;
}

/* XServiceInfo */
UNOSEQUENCE< UNOOUSTRING > SAL_CALL SvtFilePicker::getSupportedServiceNames() throw( UNORUNTIMEEXCEPTION )
{
	return impl_getStaticSupportedServiceNames();
}

/* Helper for XServiceInfo */
UNOSEQUENCE< UNOOUSTRING > SvtFilePicker::impl_getStaticSupportedServiceNames()
{
	UNOMUTEXGUARD aGuard( UNOMUTEX::getGlobalMutex() );
    UNOSEQUENCE< UNOOUSTRING > seqServiceNames( 1 );
    UNOOUSTRING* pArray = seqServiceNames.getArray();
    pArray[0] = UNOOUSTRING::createFromAscii( "com.sun.star.ui.dialogs.FilePicker" );
    return seqServiceNames ;
}

/* Helper for XServiceInfo */
UNOOUSTRING SvtFilePicker::impl_getStaticImplementationName()
{
	return UNOOUSTRING::createFromAscii( "com.sun.star.svtools.FileOpen" );
}

/* Helper for registry */
Reference< UNOXINTERFACE > SAL_CALL SvtFilePicker::impl_createInstance( const Reference< UNOXMULTISERVICEFACTORY >& xServiceManager ) throw( UNOEXCEPTION )
{
    static sal_Bool bCheckedSystemFilePicker = sal_False;
    static sal_Bool bHasSystemFilePicker = sal_True;

    Reference< UNOXINTERFACE > xRet;
    
    if ( ! bCheckedSystemFilePicker )
        bHasSystemFilePicker = HasSystemFilePicker( xServiceManager );

    if ( bHasSystemFilePicker && UseSystemFilePicker() )
    {
    	OUString aFileService( RTL_CONSTASCII_USTRINGPARAM( SYSTEM_FILE_OPEN_SERVICE_NAME ) );
        xRet = xServiceManager->createInstance( aFileService );
    }

    if ( !xRet.is() )
        xRet = Reference< UNOXINTERFACE >( *new SvtFilePicker( xServiceManager ) );

    return xRet;
}

//------------------------------------------------------------------------------------
// class FileNotifyHelper_Impl
//------------------------------------------------------------------------------------
void FileNotifyHelper_Impl::Notify( short nEventId, short nControlId )
{
	if ( mpParent )
        mpParent->notify( nEventId, nControlId );
}
