/*************************************************************************
 *
 *  $RCSfile: msvbasic.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: cmc $ $Date: 2001/06/15 14:39:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include <string.h> 	// memset(), ...
#ifndef UNX
#include <io.h> 		// access()
#endif

#ifndef _MSVBASIC_HXX
#include <msvbasic.hxx>
#endif

/*
A few urls which may in the future be of some use
http://www.virusbtn.com/vb2000/Programme/papers/bontchev.pdf
*/

/* class VBA_Impl:
 * The VBA class provides a set of methods to handle Visual Basic For
 * Applications streams, the constructor is given the root ole2 stream
 * of the document, Open reads the VBA project file and figures out
 * the number of VBA streams, and the offset of the data within them.
 * Decompress decompresses a particular numbered stream, NoStreams returns
 * this number, and StreamName can give you the streams name. Decompress
 * will call Output when it has a 4096 byte collection of data to output,
 * and also with the final remainder of data if there is still some left
 * at the end of compression. Output is virtual to allow custom handling
 * of each chunk of decompressed data. So inherit from this to do something
 * useful with the data.
 *
 * cmc
 * */

void VBA_Impl::SkipTrickyMac(SvStorageStreamRef &xVBAProject)
{
    //For no particularly good reason that I can see this occurs on
    //occasion in macintosh xls documents, it looks like an object id
    xVBAProject->SeekRel(2);
    UINT32 nIdA;
    UINT16 nIdB,nIdC;
    *xVBAProject >> nIdA >> nIdB >> nIdC;

    static const BYTE aKnownChunk[] = 
    {
        0x85, 0x2E, 0x02, 0x60, 0x8C, 0x4D, 0x0B, 0xB4
    };
    BYTE aSeq[8];
	xVBAProject->Read( aSeq, sizeof(aSeq) );
    xVBAProject->SeekRel(2);

	if (!(
        nIdA == 0x0D452EE1 && nIdB == 0xE08F && nIdC == 0x101A &&
        (0 == memcmp( aSeq, aKnownChunk, sizeof(aSeq)))
       ))
    {
        xVBAProject->SeekRel(-20);
    }
}


BYTE VBA_Impl::ReadPString(SvStorageStreamRef &xVBAProject, BOOL bIsUnicode)
{
	UINT16 nIdLen, nOut16;
	BYTE nType = 0, nOut8;

    SkipTrickyMac(xVBAProject);
	*xVBAProject >> nIdLen;

    if (nIdLen < 6) //Error recovery
	{
		xVBAProject->SeekRel(-2);
	}
	else
		for(UINT16 i=0; i < nIdLen / (bIsUnicode ? 2 : 1); i++)
		{
            if (bIsUnicode)
    			*xVBAProject >> nOut16;
            else
            {
                *xVBAProject >> nOut8;
                nOut16 = nOut8;
            }
			if (i==2)
			{
				if ((nOut16 == 'G') || (nOut16 == 'H') || (nOut16 == 'C') ||
                    nOut16 == 'D')
                {
					nType = static_cast<BYTE>(nOut16);
                }
				if ( nType == 0)
				{
					xVBAProject->SeekRel(-8);   //Error recovery
					break;
				}
			}
		}
	return nType;
}

void VBA_Impl::Output( int nLen, const BYTE *pData )
{
	/*
	Each StarBasic module is tragically limited to the maximum len of a 
	string and WordBasic is not, so each overlarge module must be split
	*/
	String sTemp((const sal_Char *)pData, (xub_StrLen)nLen,
		eCharSet);
	int nTmp = sTemp.GetTokenCount('\x0D');
	int nIndex = aVBAStrings.GetSize()-1;
	if (aVBAStrings.Get(nIndex)->Len() +
		nLen + ((nLines+nTmp) * sComment.Len()) >= STRING_MAXLEN)
	{
		//DBG_ASSERT(0,"New Module String\n");
		//we are too large for our boots, break out into another
		//string
		nLines=0;
		nIndex++;
		aVBAStrings.SetSize(nIndex+1);
		aVBAStrings.Put(nIndex,new String);
	}
	*(aVBAStrings.Get(nIndex)) += sTemp;
	nLines+=nTmp;
}


int VBA_Impl::ReadVBAProject(const SvStorageRef &rxVBAStorage)
{
	SvStorageStreamRef xVBAProject;
	xVBAProject = rxVBAStorage->OpenStream( String(
								RTL_CONSTASCII_STRINGPARAM( "_VBA_PROJECT" ),
								RTL_TEXTENCODING_MS_1252 ),
					STREAM_STD_READ | STREAM_NOCREATE );

	if( !xVBAProject.Is() || SVSTREAM_OK != xVBAProject->GetError() )
	{
		DBG_WARNING("Not able to find vba project, cannot find macros");
		return 0;
	}

    static const BYTE aKnownId[] = {0xCC, 0x61};
    BYTE aId[2];
	xVBAProject->Read( aId, sizeof(aId) );
	if (memcmp( aId, aKnownId, sizeof(aId)))
    {
        DBG_WARNING("unrecognized VBA macro project type");
        return 0;
    }
    
    static const BYTE aOffice2000LE[]   = {0x6D, 0x00, 0x00, 0x01, 0x00, 0xFF};
    static const BYTE aOffice98BE[]     = {0x60, 0x00, 0x00, 0x0E, 0x00, 0xFF};
    static const BYTE aOffice97LE[]     = {0x5E, 0x00, 0x00, 0x01, 0x00, 0xFF};
    BYTE aProduct[6];
	xVBAProject->Read( aProduct, sizeof(aProduct) );

    BOOL bIsUnicode;
    if (!(memcmp(aProduct, aOffice2000LE, sizeof(aProduct))))
    {
    	xVBAProject->SetNumberFormatInt( NUMBERFORMAT_INT_LITTLEENDIAN );
        bIsUnicode=TRUE;
    }
    else if (!(memcmp(aProduct, aOffice98BE, sizeof(aProduct))))
    {
    	xVBAProject->SetNumberFormatInt( NUMBERFORMAT_INT_BIGENDIAN );
        bIsUnicode=FALSE;
        eCharSet = RTL_TEXTENCODING_APPLE_ROMAN;
    }
    else if (!(memcmp(aProduct, aOffice97LE, sizeof(aProduct))))
    {
    	xVBAProject->SetNumberFormatInt( NUMBERFORMAT_INT_LITTLEENDIAN );
        bIsUnicode=TRUE;
    }
    else
    {
        DBG_WARNING("unrecognized VBA macro version");
        return 0;
    }

    UINT32 nLidA;  //Language identifiers
    UINT32 nLidB;
    UINT16 nUnknownA;
    UINT16 nLenA;
    UINT32 nUnknownB;
    UINT32 nUnknownC;
    UINT16 nLenB;
    UINT16 nLenC;
    UINT16 nLenD;

	*xVBAProject >> nLidA >> nLidB >> nUnknownA >> nLenA >> nUnknownB;
	*xVBAProject >> nUnknownC >> nLenB >> nLenC >> nLenD;

    if (nLenD != 0x02)
    {
        DBG_WARNING("Warning VBA number is different, please report");
        return 0;
    }

	/*
    There appears to be a number of strings, the total of nLenB and nLenC,
    most begin with *G , but sometimes with *C. If a string begins with C, it
    is really two (counts as one for the nI index), one right after the other.
    Each string then has a 12 bytes suffix
    */
    for (UINT16 nI = 0; nI < nLenB + nLenC; nI++)
    {
	    BYTE nType;
        nType = ReadPString(xVBAProject,bIsUnicode);
        if (nType == 'C' || nType == 'D')
        {
            nType = ReadPString(xVBAProject,bIsUnicode);
    	    DBG_ASSERT( nType == 'C' || nType == 'D', 
                "VBA: This must be a 'C' or 'D' string!" );
            if (nType != 'C' && nType != 'D')
                return 0;
        }
    	DBG_ASSERT( nType, "VBA: Bad String in VBA Project, panic!!" );
        if (!nType)
            return 0;
        xVBAProject->SeekRel(12);
    }

    SkipTrickyMac(xVBAProject);

    INT16 nInt16s;
    *xVBAProject >> nInt16s;
    DBG_ASSERT( nInt16s, "VBA: Bad String in VBA Project, panic!!" );
    if (!nInt16s)
        return 0;
    
    xVBAProject->SeekRel(2*nInt16s);

    INT16 nInt32s;
    *xVBAProject >> nInt32s;
    DBG_ASSERT( nInt32s, "VBA: Bad String in VBA Project, panic!!" );
    if (!nInt32s)
        return 0;
    xVBAProject->SeekRel(4*nInt32s);

    xVBAProject->SeekRel(2);
    for(int k=0;k<3;k++)
    {
        UINT16 nLen;
    	*xVBAProject >> nLen;
        if (nLen != 0xFFFF)
            xVBAProject->SeekRel(nLen);
    }
    xVBAProject->SeekRel(100); //Seems fixed len

	*xVBAProject >> nOffsets;
    DBG_ASSERT( nOffsets != 0xFFFF, "VBA: Bad nOffsets, panic!!" );
    if (nOffsets == 0xFFFF)
        return 0;
	pOffsets = new VBAOffset_Impl[ nOffsets ];

	int i, j;
	for( i=0; i < nOffsets; i++)
	{
		UINT16 nLen;
		*xVBAProject >> nLen;

        if (bIsUnicode)
        {
            sal_Unicode* pBuf = pOffsets[i].sName.AllocBuffer( nLen / 2 );
            xVBAProject->Read( (sal_Char*)pBuf, nLen  );

#ifdef __BIGENDIAN
            for( j = 0; j < nLen / 2; ++j, ++pBuf )
                *pBuf = SWAPLONG( *pBuf );
#endif // ifdef __BIGENDIAN
        }
        else
        {
            ByteString aByteStr;    
            sal_Char*  pByteData = aByteStr.AllocBuffer( nLen );
            sal_Size nWasRead = xVBAProject->Read( pByteData, nLen );
            if( nWasRead != nLen )
                aByteStr.ReleaseBufferAccess();
            pOffsets[i].sName += String( aByteStr, eCharSet);
        }

		*xVBAProject >> nLen;
		xVBAProject->SeekRel( nLen );

		//begin section, another problem area
		*xVBAProject >> nLen;
		if ( nLen == 0xFFFF)
		{
			xVBAProject->SeekRel(2);
			*xVBAProject >> nLen;
			xVBAProject->SeekRel( nLen );
		}
		else
			xVBAProject->SeekRel( nLen+2 );
		
		*xVBAProject >> nLen;
        DBG_ASSERT( nLen == 0xFFFF, "VBA: Bad field in VBA Project, panic!!" );
		if ( nLen != 0xFFFF)
            return 0;
			
		xVBAProject->SeekRel(6);
		UINT16 nOctects;
		*xVBAProject >> nOctects;
		for(j=0;j<nOctects;j++)
			xVBAProject->SeekRel(8);
		
		xVBAProject->SeekRel(5);
		//end section

		*xVBAProject >> pOffsets[i].nOffset;
		xVBAProject->SeekRel(2);
	}

	//*pOut << endl;
	return nOffsets;
}

BOOL VBA_Impl::Open( const String &rToplevel, const String &rSublevel )
{
	/* beginning test for vba stuff */
	BOOL bRet = FALSE;
	SvStorageRef xMacros= xStor->OpenStorage( rToplevel,
									STREAM_READWRITE | STREAM_NOCREATE |
									STREAM_SHARE_DENYALL );
	if( !xMacros.Is() || SVSTREAM_OK != xMacros->GetError() )
	{
		DBG_WARNING("No Macros Storage");
	}
	else
	{
		xVBA = xMacros->OpenStorage( rSublevel,
									STREAM_READWRITE | STREAM_NOCREATE |
									STREAM_SHARE_DENYALL );
		if( !xVBA.Is() || SVSTREAM_OK != xVBA->GetError() )
		{
			DBG_WARNING("No Visual Basic in Storage");
		}
		else
		{
			if (ReadVBAProject(xVBA))
				bRet = TRUE;
		}
	}
	/* end test for vba stuff */
	return bRet;
}

const StringArray &VBA_Impl::Decompress(UINT16 nIndex, int *pOverflow)
{
	DBG_ASSERT( nIndex < nOffsets, "Index out of range" );
	SvStorageStreamRef xVBAStream;
	aVBAStrings.SetSize(1);
	aVBAStrings.Put(0,new String);

	xVBAStream = xVBA->OpenStream( pOffsets[nIndex].sName,
						STREAM_STD_READ | STREAM_NOCREATE );
	if (pOverflow)
		*pOverflow=0;

	if( !xVBAStream.Is() || SVSTREAM_OK != xVBAStream->GetError() )
	{
		DBG_WARNING("Not able to open vb module ");
	}
	else
	{
		xVBAStream->SetNumberFormatInt( NUMBERFORMAT_INT_LITTLEENDIAN );
		DecompressVBA( nIndex, xVBAStream );
		/*
		 * if len was too big for a single string set that variable ?
		 *	if ((len > XX) && (pOverflow))
				*pOverflow=1;
		 */
		if (bCommented)
		{
            String sTempStringa;
            String sTempStringb;
            if (eCharSet == RTL_TEXTENCODING_MS_1252)
            {
                sTempStringa = String( RTL_CONSTASCII_STRINGPARAM("\x0D\x0A"), 
                    eCharSet);
                sTempStringb = String( RTL_CONSTASCII_STRINGPARAM("\x0D\x0A"), 
                    eCharSet);
            }
            else
            {
                sTempStringa = String( RTL_CONSTASCII_STRINGPARAM("\x0D"), 
    				eCharSet);
			    sTempStringb = String( RTL_CONSTASCII_STRINGPARAM("\x0D"), 
	    			eCharSet);
            }
			sTempStringb+=sComment;
			for(ULONG i=0;i<aVBAStrings.GetSize();i++)
			{
				aVBAStrings.Get(i)->SearchAndReplaceAll(
					sTempStringa,sTempStringb);
				aVBAStrings.Get(i)->Insert(sComment,0);
			}
		}
	}
	return aVBAStrings;
}


int VBA_Impl::DecompressVBA( int nIndex, SvStorageStreamRef &xVBAStream )
{
	BYTE nLeadbyte;
	UINT16 nToken;
	unsigned int nPos = 0;
	int nLen, nDistance, nShift, nClean=1;

	//*pOut << "jumping to " << hex << offsets[nIndex].offset << endl;
	xVBAStream->Seek( pOffsets[ nIndex ].nOffset + 3 );

	while(xVBAStream->Read(&nLeadbyte,1))
	{
//		*pOut << "reading 8 data unit block beginning with " << nLeadbyte
//			<< int(nLeadbyte) << " at pos " << xVBAStream->Tell() << " real pos "
//			<< nPos << endl;
		for(int nPosition=0x01;nPosition < 0x100;nPosition=nPosition<<1)
		{
			//we see if the leadbyte has flagged this location as a dataunit
			//which is actually a token which must be looked up in the history
			if (nLeadbyte & nPosition)
			{
				*xVBAStream >> nToken;

				if (nClean == 0)
					nClean=1;

				//For some reason the division of the token into the length
				//field of the data to be inserted, and the distance back into
				//the history differs depending on how full the history is
				int nPos2 = nPos%WINDOWLEN;
				if (nPos2 <= 0x10)
					nShift = 12;
				else if (nPos2 <= 0x20)
					nShift = 11;
				else if (nPos2 <= 0x40)
					nShift = 10;
				else if (nPos2 <= 0x80)
					nShift = 9;
				else if (nPos2 <= 0x100)
					nShift = 8;
				else if (nPos2 <= 0x200)
					nShift = 7;
				else if (nPos2 <= 0x400)
					nShift = 6;
				else if (nPos2 <= 0x800)
					nShift = 5;
				else
					nShift = 4;

				int i;
				nLen=0;
				for(i=0;i<nShift;i++)
					nLen |= nToken & (1<<i);

				//*pOut << endl << "match lookup token " << int(nToken) << "len " << int(nLen) << endl;

				nLen += 3;
				//*pOut << endl << "len is " << nLen << "shift is " << nShift << endl;

				nDistance = nToken >> nShift;
				//*pOut << "distance token shift is " << nDistance << " " << int(nToken) << " " << nShift << "pos is " << nPos << " " << xVBAStream->Tell() << endl;

				//read the len of data from the history, wrapping around the
				//WINDOWLEN boundary if necessary
				//data read from the history is also copied into the recent
				//part of the history as well.
				for (i = 0; i < nLen; i++)
				{
					unsigned char c;
					//*pOut << endl << (nPos%WINDOWLEN)-nDistance-1 << " " << nPos << " " << nDistance << endl;
					c = aHistory[(nPos-nDistance-1)%WINDOWLEN];
					aHistory[nPos%WINDOWLEN] = c;
					nPos++;
					//*pOut << "real pos is " << nPos << endl;
					//
					//temp removed
					//*pOut << c ;
				}
			}
			else
			{
				// special boundary case code, not guarantueed to be correct
				// seems to work though, there is something wrong with the
				// compression scheme (or maybe a feature) where when
				// the data ends on a WINDOWLEN boundary and the excess
				// bytes in the 8 dataunit list are discarded, and not
				// interpreted as tokens or normal data.
				if ((nPos != 0) && ((nPos%WINDOWLEN) == 0) && (nClean))
				{
					//*pOut << "at boundary position is " << nPosition << " " << xVBAStream->Tell() << " pos is " << nPos << endl;
					//if (nPosition != 0x01)
					//*pOut << "must restart by eating remainder single byte data units" << endl;
					xVBAStream->SeekRel(2);
					nClean=0;
					Output(WINDOWLEN,aHistory);
					break;
				}
				//This is the normal case for when the data unit is not a
				//token to be looked up, but instead some normal data which
				//can be output, and placed in the history.
				if (xVBAStream->Read(&aHistory[nPos%WINDOWLEN],1))
				{
					nPos++;
					//temp removed
					//*pOut << aHistory[nPos++%WINDOWLEN];
				}
				if (nClean == 0)
					nClean=1;
				//*pOut << "pos is " << nPos << " " << xVBAStream->Tell() << endl;
			}
		}
	}
	if (nPos%WINDOWLEN)
		Output(nPos%WINDOWLEN,aHistory);
	return(nPos);
}

