/*************************************************************************
 *
 *  $RCSfile: lngconv.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: obr $ $Date: 2001/07/26 14:41:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#if defined( LINUX ) || defined (SOLARIS)
#include <locale.h>
#include <langinfo.h>
#endif

#include <strings.h>
#include <osl/nlsupport.h>
#include <osl/profile.h>
#include <osl/file.h>
#include <osl/diagnose.h>
#include <rtl/tencinfo.h>

#ifdef UNX

#define stricmp strcasecmp
#define strnicmp strncasecmp 

typedef struct {
	const char				*key;
	const rtl_TextEncoding	 value;
} _pair;

static int _pair_compare (const char *key, const _pair *pair);
static const _pair* _pair_search (const char *key, const _pair *base, unsigned int member );

const char * delimiter = "/";

#elif defined WNT

#define stricmp _stricmp
#define strnicmp _strnicmp

const char * delimiter = "\\";

#endif

static sal_Char section_buf[2048];
static rtl_uString * value_arr[256];
static sal_uInt32 n_items = 0;

static const char * pAppName = "lngconv";


/************************************************************************
 * Usage
 ************************************************************************/

void Usage() 
{
    printf( "Usage: lngconv [-o <outpath>] <lng-file> <lang no> -CHARSET_<lng charset> <locale> [<locale2>] ...\n" );
    exit(-1);
}


/************************************************************************
 * ReadSourceFile
 ************************************************************************/

int ReadSourceFile( const char * sourceFile, long nLang, rtl_TextEncoding encoding )
{
	rtl_uString * pFile = NULL;
	oslProfile aProfile = NULL;

	const char * pcDefaultLang = "01";
	const char * pc;
	
	char  absFileName[2048];
	char  Lang[4];

	/* convert language no to string */	
	sprintf( Lang, "%d", nLang );

	/* make source file absolute if necessary */
	if( sourceFile[0] != '/' )
	{
		getcwd( absFileName, sizeof( absFileName ) );
		OSL_ASSERT( strlen( absFileName ) + strlen( sourceFile ) < sizeof( absFileName ) - 2 );
		strcat( absFileName, "/" );
		strcat( absFileName, sourceFile );
		sourceFile = absFileName;
	}
	
	/* convert file name to unicode file url */
	rtl_uString_newFromAscii( &pFile, sourceFile );
	osl_getFileURLFromSystemPath( pFile, &pFile );

	/* open .lng file - use old oslProfile implementation */
	aProfile = osl_openProfile( pFile, 0 );
	rtl_uString_release( pFile );
	
	if( NULL == aProfile )
	{
		fprintf( stderr, "%s: unable to open language file %s.\n\n", pAppName, pFile );
		exit( 1 );
	}

	/* read section names into buffer */
	osl_getProfileSections( aProfile, section_buf, sizeof( section_buf ) );
	
	/* iterate over all sections */
	for( pc = section_buf; *pc != '\0'; pc += strlen( pc ) + 1 )
	{
		char entry[2048];
		
		rtl_TextEncoding entry_enc = encoding;
		
		/* read string for selected language */
		osl_readProfileString( aProfile, pc, Lang, entry, sizeof(entry), "" );
		
		/* use default language if string is empty */
		if( 0 == strlen( entry ) )
		{
			osl_readProfileString( aProfile, pc, pcDefaultLang, entry, sizeof(entry), "" );
			entry_enc = RTL_TEXTENCODING_ASCII_US;
		}
		
		/* convert entry string to unicode - cut off "" */
		value_arr[n_items] = NULL;
		rtl_string2UString( value_arr + n_items, entry, strlen( entry ), entry_enc, OSTRING_TO_OUSTRING_CVTFLAGS );
		++n_items;
	}
	

	/* close source file */
	osl_closeProfile( aProfile );
	
	return 1;
}


/************************************************************************
 * WriteDestFile
 ************************************************************************/

int WriteDestFile( const char * destFile, rtl_TextEncoding encoding )
{
	FILE * pFile = fopen( destFile, "w" );
	
	if( NULL != pFile )
	{
		unsigned long n = 0;
		const char * pc;
		
		/* iterate over all sections */
		for( pc = section_buf; *pc != '\0'; pc += strlen( pc ) + 1 )
		{
			rtl_String  * value   = NULL;
			rtl_uString * u_value = value_arr[n++];
		
			if( NULL != u_value )
			{
				/* convert string to desired encoding */
				rtl_uString2String( &value, u_value->buffer, u_value->length, encoding, OUSTRING_TO_OSTRING_CVTFLAGS );
				
				if( NULL != value )
				{
					/* write key = value pairs to file */
					fprintf( pFile, "%s = %s\n", pc, value->buffer );
					rtl_string_release( value );
				}
			}
		}
					
		fclose( pFile );
	}

 	return 1;
}


/************************************************************************
 * main
 ************************************************************************/

int main( int argc, const char * argv[] )
{
    const char * pOutPath = NULL;
    const char * pLNGFile = NULL;
	
    rtl_TextEncoding nSourceCharSet = RTL_TEXTENCODING_DONTKNOW;
    
    long nLang = 0;
    int i = 0;

    /* default behavior without parameter */
    if( argc < 4 )
        Usage();

    /* iterate over command line arguments */
    for( i = 1; i < argc; i++ )
    {
        /* check if outpath is specified */
        if( 0 == strncmp(argv[i], "-o", 2) )
        {
            if(argc - i >= 2)
                pOutPath = argv[++i];
            else
                Usage();
        }

        /* the .lng file to read from name */
        else if( pLNGFile == NULL )
            pLNGFile = argv[i];

        /* the language no */
        else if( nLang == 0 )
            sscanf( argv[i], "%d", &nLang );

        /* the encoding used for the language in .lng file */            
        else if( nSourceCharSet == RTL_TEXTENCODING_DONTKNOW )
        {
            const char * pStr = argv[i];
            
            if( !strnicmp( pStr, "-CHARSET_", 8 ) )
			{
				if( !stricmp( pStr + 8, "ANSI" ) )
					 nSourceCharSet = RTL_TEXTENCODING_MS_1252;
				else if( !stricmp( pStr + 8, "MAC" ) )
					 nSourceCharSet = RTL_TEXTENCODING_APPLE_ROMAN;
				else if( !stricmp( pStr + 8, "IBMPC_437" ) )
					 nSourceCharSet = RTL_TEXTENCODING_IBM_437;
				else if( !stricmp( pStr + 8, "IBMPC_850" ) )
					 nSourceCharSet = RTL_TEXTENCODING_IBM_850;
				else if( !stricmp( pStr + 8, "IBMPC_860" ) )
					 nSourceCharSet = RTL_TEXTENCODING_IBM_860;
				else if( !stricmp( pStr + 8, "IBMPC_861" ) )
					 nSourceCharSet = RTL_TEXTENCODING_IBM_861;
				else if( !stricmp( pStr + 8, "IBMPC_863" ) )
					 nSourceCharSet = RTL_TEXTENCODING_IBM_863;
				else if( !stricmp( pStr + 8, "IBMPC_865" ) )
					 nSourceCharSet = RTL_TEXTENCODING_IBM_865;
				else if( !stricmp( pStr + 8, "IBMPC" ) )
					 nSourceCharSet = RTL_TEXTENCODING_IBM_850;
				else if( !stricmp( pStr + 8, "DONTKNOW" ) )
					 nSourceCharSet = RTL_TEXTENCODING_DONTKNOW;
				else
				{
					nSourceCharSet = rtl_getTextEncodingFromUnixCharset( pStr + 8 );
					if( nSourceCharSet == 0 )
						nSourceCharSet = RTL_TEXTENCODING_MS_1252;
				}
            }
        }
        
        /* all other arguments expected to be destination locals / charsets */
        else
            break;
    }
    
    if( pLNGFile == NULL )
    {
        fprintf( stderr, "%s: no language file specified.\n\n", pAppName );
        Usage();
    }
    
    if( nLang == 0 )
    {
        fprintf( stderr, "%s: no language specified.\n\n", pAppName );
        Usage();
    }
    
    if( nSourceCharSet == RTL_TEXTENCODING_DONTKNOW )
    {
        fprintf( stderr, "%s: no source encoding specified.\n\n", pAppName );
        Usage();
    }
    
    if( i >= argc )
    {
        fprintf( stderr, "%s: no destination locales/languages specified.\n\n", pAppName );
        Usage();
    }

	ReadSourceFile( pLNGFile, nLang, nSourceCharSet );

    /* iterate over languages / locales */
    while( i < argc )
    {
		rtl_TextEncoding nDestCharSet;

		rtl_uString * pLanguage = NULL;
		rtl_uString * pCountry = NULL;
		rtl_uString * pVariant = NULL;
		
		rtl_Locale * pLocale;

        char * pOutFile = NULL;
		char * locale_buf = NULL;
        char * pc;

        // ignore leading path
        if( pc = (char * ) strrchr( pLNGFile, *delimiter ) )
            pLNGFile = pc + 1;

        int nStrLen = strlen( pLNGFile ) + strlen( argv[i] ) + 1;
        
        if( pOutPath )
            nStrLen += strlen( pOutPath ) + 1;
            
        pOutFile = (char *) malloc( nStrLen + 1 );
        pOutFile[0] = '\0';
        
        /* construct output file name */
        if( pOutPath )
        {
            strcpy(pOutFile, pOutPath);
            strcat(pOutFile, delimiter);
        }
        
        /* copy file name and cut off .lng extension */
        strcat( pOutFile, pLNGFile );
        if( ( pc = strstr( pOutFile, ".lng") ) || ( pc = strstr( pOutFile, ".LNG") ) )
            *pc = '\0';
        
        /* append ".<locale>" to the file name */
        strcat( pOutFile, "." );
        strcat( pOutFile, argv[i] );
            
		/* scan and split locale string */
		locale_buf = strdup( argv[i++] );
		
		/* variants should be leaded by '.' */
		pc = strrchr( locale_buf, '.' );
		rtl_uString_newFromAscii( &pVariant, pc ? pc+1 : "" );
		
		if( pc )
			*pc = '\0';
		
		/* languages should be leaded by '_' */
		pc = strrchr( locale_buf, '_' );
		rtl_uString_newFromAscii( &pCountry, pc ? pc+1 : "" );
		
		if( pc )
			*pc = '\0';

		/* this should be the language code only */
		rtl_uString_newFromAscii( &pLanguage, locale_buf );
		free( locale_buf );
		
		pLocale = rtl_locale_register( pLanguage->buffer, pCountry->buffer, pVariant->buffer );
		
		rtl_uString_release( pLanguage );
		rtl_uString_release( pCountry );
		rtl_uString_release( pVariant );

	    /* get encoding for locale */
		nDestCharSet = osl_getTextEncodingFromLocale( pLocale );

		printf( "Writing output file %s.\n", pOutFile );
		WriteDestFile( pOutFile, nDestCharSet );
    }
	
	/* FIXME: destroy value array list */
    
    return 0;
}
