/*************************************************************************
#*
#*    $Workfile:   testmain.cxx  $
#*
#*    Ersterstellung    AB 10.04.2000
#*    Letzte Aenderung  $Author: kr $ $Date: 2001/05/04 07:05:18 $
#*
#*    $Logfile: $
#*
#*    Copyright (c) 2000, Star Office Entwicklungs GmbH
#*
#*************************************************************************

      Source Code Control System - Header

      $Header: /cvs/oo/udk/testtools/source/testframework/testimpl.cxx,v 1.1 2001/05/04 07:05:18 kr Exp $

*************************************************************************/

#include <stdio.h>
#include <string.h>

#include <cppuhelper/servicefactory.hxx>
#include <cppuhelper/implbase1.hxx>
#include <osl/mutex.hxx>

#include <osl/thread.hxx>

#include <com/sun/star/registry/XImplementationRegistration.hpp>
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#include <com/sun/star/beans/XPropertySet.hpp>
#include <com/sun/star/lang/XComponent.hpp>
#include <com/sun/star/test/XTest.hpp>
#include <com/sun/star/test/XSimpleTest.hpp>
#include <com/sun/star/test/XTestListener.hpp>
#include <com/sun/star/test/TestEvent.hpp>

//#include <com/sun/star/beans/XIntrospection.hpp>

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::registry;
using namespace ::com::sun::star::test;
using namespace cppu;
using namespace rtl;
using namespace osl;

static void usingTestImpl();
static void errout( const char* pMsg );
static void out( const char* pMsg );
static sal_Bool b_stderr = sal_True;
static sal_Bool b_stdout = sal_True;


enum ComponentType
{
	CP_UNDEFINED,
	CP_LIB,
	CP_JAVA
};

// Struct to keep command line data
struct TestInfo
{
	// General Data
	Reference< XMultiServiceFactory > xSMgr;

	// Command line data
	ComponentType eComponentType;
	OUString aCmdComponentName;
	OUString aCmdTestInterfaceName;
	OUString aCmdTestServiceName;
	OUString aTestComponentRoot;
	sal_Bool bRemote;
	sal_Bool bMultiThread;
	sal_Int32 nThreadCount;
	sal_Bool bTestInterface;
	sal_Bool bTestService;

	// Component data
	Sequence<OUString> aSupportedServices;
	Sequence< Sequence<OUString> > aImplNamesForServices;

	// Status data
	//sal_Bool bMultiThreadPassed;


	TestInfo( sal_Int16 argc, char * argv[] ) throw (RuntimeException);
	TestInfo( const Sequence< OUString > & rArgs ) throw (RuntimeException);
	init( const Sequence< OUString > & rArgs ) throw (RuntimeException);
	void analyseComponent( void );
	sal_Int32 getServiceIndex( const OUString& rServiceName );
	Reference< XInterface > createTestInstance( const OUString& rImplName ) const;
	Reference< XInterface > createTestComponent( const OUString& aTestName ) const;
};


// TestListener
class TestListenerImpl : public WeakImplHelper1< XTestListener >
{
public:
	TestListenerImpl( const TestInfo& rInfo );
	virtual ~TestListenerImpl();
	
	// XTestListener
    virtual void SAL_CALL notifyError( const TestEvent& evt ) throw(RuntimeException);
    virtual void SAL_CALL notifyWarning( const TestEvent& evt ) throw(RuntimeException);
    virtual void SAL_CALL protocol( const TestEvent& evt ) throw(RuntimeException);
};

void SAL_CALL TestListenerImpl::notifyError( const TestEvent& evt ) 
	throw(RuntimeException)
{
	OString aOStrMsg( "ERROR " );
	if( evt.Exception.hasValue() )
	{
		aOStrMsg + " (Exception ";
		OString aOExceptionTypeName = OUStringToOString
			( evt.Exception.getValueType().getTypeName(), RTL_TEXTENCODING_ASCII_US );
		aOStrMsg + aOExceptionTypeName;
		aOStrMsg + ")";
	}
	if( evt.Message.len() )
	{
		aOStrMsg + ": ";
		aOStrMsg += OUStringToOString( evt.Message, RTL_TEXTENCODING_ASCII_US );
	}
	errout( aOStrMsg.getStr() );
}


void SAL_CALL TestListenerImpl::notifyWarning( const TestEvent& evt ) 
	throw(RuntimeException)
{
	OString aOStrMsg( "WARNING: " );
	aOStrMsg += OUStringToOString( evt.Message, RTL_TEXTENCODING_ASCII_US );
	errout( aOStrMsg.getStr() );
}

void SAL_CALL TestListenerImpl::protocol( const TestEvent& evt ) 
	throw(RuntimeException)
{
	OString aOStrMsg = OUStringToOString( evt.Message, RTL_TEXTENCODING_ASCII_US );
	out( aOStrMsg.getStr() );
}


// Help functions
Reference< XSimpleRegistry > createEmptyRegistry( Reference< XMultiServiceFactory > xSMgr, OUString aRegFileName )
{
	Reference< XSimpleRegistry > xReg( xSMgr->createInstance( ::rtl::OUString::createFromAscii( "com.sun.star.registry.SimpleRegistry" ) ), UNO_QUERY);
	if( xReg.is() )
	{
		sal_Bool bSuccess = sal_True;

		// Does the file already exist? Then destroy
		try
		{
			xReg->open( aRegFileName, sal_False, sal_False );
			if( xReg->isValid() )
				xReg->destroy();
		}
		catch( InvalidRegistryException& )
		{
		}

		try
		{
			// Now (re)create registry file
			xReg->open( aRegFileName, sal_False, sal_True );
			if( !xReg->isValid() )
				bSuccess = sal_False;
		}
		catch( InvalidRegistryException& )
		{
			bSuccess = sal_False;
		}

		if( !bSuccess )
		{
			char aMsg[256];
			OString aORegFileName( OUStringToOString( aRegFileName, RTL_TEXTENCODING_ASCII_US ) );
			sprintf( aMsg, "Fatal error: Create registry \"%s\" failed!\n", aORegFileName.getStr() );
			errout( aMsg );
			throw RuntimeException( OUString(), Reference< XInterface >() );
		}
	}
	return xReg;
}

sal_Bool registerImplementation( Reference< XMultiServiceFactory > xSMgr, Reference< XSimpleRegistry > xReg,
	const OUString& rComponentName, ComponentType eComponentType, sal_Bool bShowFatal=sal_True )
{
	OUString aImplRegistrationName( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.registry.ImplementationRegistration") );
	Reference< XImplementationRegistration > xImplRegistration( xSMgr->createInstance( aImplRegistrationName ), UNO_QUERY );
	if( !xImplRegistration.is() )
	{
		if( bShowFatal )
		{
			char aMsg[256];
			OString aOImplRegistrationName( OUStringToOString( aImplRegistrationName, RTL_TEXTENCODING_ASCII_US ) );
			sprintf( aMsg, "\nFATAL ERROR: Could not create service\"%s\"", aOImplRegistrationName.getStr() );
			errout( aMsg );
			throw RuntimeException( OUString(), Reference< XInterface >() );
		}
		else
		{
			return sal_False;
		}
	}

	OUString aLoader;
	if( eComponentType == CP_LIB )
		aLoader = OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.loader.SharedLibrary") );
	else if( eComponentType == CP_JAVA )
		aLoader = OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.loader.Java2") );

	try
	{
		xImplRegistration->registerImplementation( aLoader, rComponentName, xReg );		
	}
	catch( CannotRegisterImplementationException& e)
	{
		if( bShowFatal )
		{
			char aMsg[256];
			OString aOCompName = OUStringToOString( rComponentName, RTL_TEXTENCODING_ASCII_US );
			OString aMessage( OUStringToOString( e.Message, RTL_TEXTENCODING_ASCII_US ) );
			sprintf( aMsg, "\nFATAL ERROR: Registration error for \"%s\": %s\n", aOCompName.getStr(), aMessage.getStr() );
			errout( aMsg );
			throw RuntimeException( OUString(), Reference< XInterface >() );
		}
		else
		{
			return sal_False;
		}
	}
	return sal_True;
}


//***************************************
//*** Implementatiom struct TestInfo  ***
//***************************************

// Analyse command line
TestInfo::TestInfo( const Sequence< OUString > & rArgs )
	throw (RuntimeException)
{
	init( rArgs );
}

// Analyse command line
TestInfo::init( const Sequence< OUString > & rArgs )
	throw (RuntimeException)
{
	const OUString* pArgs = rArgs.getConstArray();
	sal_Int32 nLen = rArgs.getLength();

	bRemote = sal_False;
	bMultiThread = sal_False;
	bTestInterface = sal_False;
	bTestService = sal_False;

	if( nLen < 1 )
	{
		usingTestImpl();
		throw RuntimeException( OUString(), Reference< XInterface >() );
	}

	// Component parameter
	aCmdComponentName = pArgs[0];
	OUString aDllExt( RTL_CONSTASCII_USTRINGPARAM("dll") );
	OUString aLibExt( RTL_CONSTASCII_USTRINGPARAM("so") );
	OUString aClassExt( RTL_CONSTASCII_USTRINGPARAM("class") );
	OUString aJarExt( RTL_CONSTASCII_USTRINGPARAM("jar") );

	eComponentType = CP_UNDEFINED;
	if( aCmdComponentName.indexOf( aDllExt ) != -1 || aCmdComponentName.indexOf( aLibExt ) != -1 )
	{
		eComponentType = CP_LIB;
	}
	else if( aCmdComponentName.indexOf( aClassExt ) != -1 || aCmdComponentName.indexOf( aJarExt ) != -1 )
	{
		eComponentType = CP_JAVA;
	}

	// Other parameter
	for( sal_Int16 i = 1 ; i < nLen ; i++ )
	{
		if( pArgs[i].compareToAscii( "-r" ) == 0 )
		{
			bRemote = sal_True;
		}
		else if( pArgs[i].compareToAscii( "-m" ) == 0 )
		{
			bMultiThread = sal_True;
			if( i + 1 >= nLen )
			{
				usingTestImpl();
				throw RuntimeException( OUString(), Reference< XInterface >() );
			}
			nThreadCount = pArgs[++i].toInt32();
		}
		else if( pArgs[i].compareToAscii( "-i" ) == 0 )
		{
			if( i + 1 >= nLen )
			{
				usingTestImpl();
				throw RuntimeException( OUString(), Reference< XInterface >() );
			}
			aCmdTestInterfaceName = pArgs[++i];
			bTestInterface = sal_True;
		}
		else if( pArgs[i].compareToAscii( "-s" ) == 0 )
		{
			if( i + 1 >= nLen )
			{
				usingTestImpl();
				throw RuntimeException( OUString(), Reference< XInterface >() );
			}
			aCmdTestServiceName = pArgs[++i];
			bTestService = sal_True;
		}
		else if( pArgs[i].compareToAscii( "-p" ) == 0 )
		{
			if( i + 1 >= nLen )
			{
				usingTestImpl();
				throw RuntimeException( OUString(), Reference< XInterface >() );
			}
			aTestComponentRoot = pArgs[++i];
		}
		else
		{
			usingTestImpl();
			throw RuntimeException( OUString(), Reference< XInterface >() );
		}
	}
}

// Analyse command line
TestInfo::TestInfo( sal_Int16 argc, char * argv[] )
	throw (RuntimeException)
{
	Sequence< OUString > aArgSeq;
	aArgSeq.realloc( argc - 1 );
	OUString* pArgs = aArgSeq.getArray();
	for( sal_Int16 i = 1 ; i < argc ; i++ )
	{
		pArgs[i-1] = OUString::createFromAscii( argv[i] );
	}
	init( aArgSeq );
}

// Analyse test componenent
void TestInfo::analyseComponent( void )
{
	// Bootstrap service factory, only to create registry and register test component
	Reference< XMultiServiceFactory > xSimpleMgr( createServiceFactory() );

	// Create an empty registry to register the test component in it for analysing
	OUString aAnalyseRegFile( RTL_CONSTASCII_USTRINGPARAM("analyse.rdb") );
	Reference< XSimpleRegistry > xReg = createEmptyRegistry( xSimpleMgr, aAnalyseRegFile );

	// Register component in new registry
	registerImplementation( xSimpleMgr, xReg, aCmdComponentName, eComponentType );

	// Analys registry
	Reference<XRegistryKey> xRootKey( xReg->getRootKey() );

	OUString aServicesBaseKey( RTL_CONSTASCII_USTRINGPARAM("SERVICES") );
	Reference<XRegistryKey> xServicesKey = xRootKey->openKey( aServicesBaseKey );
	sal_Int32 nPureServiceNameCopyLen = aServicesBaseKey.getLength() + 2;

	Sequence<OUString> aServiceKeyNames = xServicesKey->getKeyNames(); 
	const OUString* pNames = aServiceKeyNames.getConstArray();
	sal_Int32 nLen = aServiceKeyNames.getLength();

	aSupportedServices.realloc( nLen );
	OUString* pServiceNames = aSupportedServices.getArray();
	aImplNamesForServices.realloc( nLen );
	Sequence<OUString>* pImpls = aImplNamesForServices.getArray();

	for( sal_Int32 i = 0 ; i < nLen ; i++ )
	{
		OUString aServiceKeyName = pNames[i];
		OUString aServiceName = aServiceKeyName.copy( nPureServiceNameCopyLen );
		pServiceNames[i] = aServiceName;

		Reference<XRegistryKey> xSingleServiceKey = xRootKey->openKey( aServiceKeyName );
		RegistryValueType eValueType = xSingleServiceKey->getValueType();
		Sequence<OUString> aImplSeq;
		try
		{
			Sequence<OUString>& rImplSeq = pImpls[i] = xSingleServiceKey->getAsciiListValue();
			if( rImplSeq.getLength() > 1 )
				errout( "\nWARNING: Components supports several implementations for one service!\n" );

			// TEST
			const OUString* pImplNames = rImplSeq.getConstArray();
			sal_Int32 nImplsLen = rImplSeq.getLength();
			for( sal_Int32 j = 0 ; j < nImplsLen ; j++ )
			{
				OUString aImplName = pImplNames[j];
			}
		}
		catch( InvalidValueException& )
		{
			char aMsg[256];
			OString aOServiceName = OUStringToOString( aServiceKeyName, RTL_TEXTENCODING_ASCII_US );
			sprintf( aMsg, "\nRegistry error: ValueType of key \"%s\" != STRINGLIST\n", 
				aOServiceName.getStr() );
			errout( aMsg );
			throw RuntimeException( OUString(), Reference< XInterface >() );
		}
	}

	xServicesKey->closeKey();
	xReg->close();
}

sal_Int32 TestInfo::getServiceIndex( const OUString& rServiceName )
{
	const OUString* pNames = aSupportedServices.getConstArray();
	sal_Int32 nLen = aSupportedServices.getLength();
	for( sal_Int32 i = 0 ; i < nLen ; i++ )
	{
		OUString aName = pNames[i];
		if( aName == rServiceName )
			return i;
	}
	return -1;
}

// Create instance according to parameters (remote/not remote)
Reference< XInterface > TestInfo::createTestInstance( const OUString& rImplName ) const
{
	Reference< XInterface > xRet;

	if( bRemote )
	{
		// TODO
	}
	else
	{
		xRet = xSMgr->createInstance( rImplName );
	}
	if( !xRet.is() )
	{
		char aMsg[256];
		OString aOImplName( OUStringToOString( rImplName, RTL_TEXTENCODING_ASCII_US ) );
		sprintf( aMsg, "\nFATAL ERROR: Cannot instanciate implementation \"%s\"!\n", aOImplName.getStr() );
		errout( aMsg );
		throw RuntimeException( OUString(), Reference< XInterface >() );
	}
	return xRet;
}

Reference< XInterface > TestInfo::createTestComponent( const OUString& aTestName ) const
{
	// Try first to find registered test component 
	// Name of the corresponding test component 
	OUString aTestStr( RTL_CONSTASCII_USTRINGPARAM("test") );
	OUString aPointStr( RTL_CONSTASCII_USTRINGPARAM(".") );
	OUString aTestImplName = aTestStr;
	aTestImplName += aPointStr;
	aTestImplName += aTestName;

	Reference< XInterface > xRet = xSMgr->createInstance( aTestImplName );
	if( xRet.is() )
		return xRet;

	// Try to find test component by library name
	// TODO: osl file interface
	OUString aLibExtension;
	sal_Unicode cPoint = '.';
	sal_Unicode cDirDelimiter;
#ifdef UNX
	aLibExtension = OUString( RTL_CONSTASCII_USTRINGPARAM(".so") );
	cDirDelimiter = '/';
#else
	aLibExtension = OUString( RTL_CONSTASCII_USTRINGPARAM(".dll") );
	cDirDelimiter = '\\';
#endif
	OUString aJavaExtensionClass = OUString( RTL_CONSTASCII_USTRINGPARAM(".class") );
	OUString aJavaExtensionJar = OUString( RTL_CONSTASCII_USTRINGPARAM(".jar") );
	OUString aPureTestName = aTestName.copy( aTestName.lastIndexOf( (sal_Unicode)'.' ) + 1 );

	sal_Bool bResult;
	OUString aNameBase;
	if( aTestComponentRoot.len() )
		aNameBase = aTestComponentRoot;
	else
		aNameBase = aTestStr;
	aNameBase += aPointStr;
	aNameBase += aTestName;
	aNameBase += aPointStr;
	aNameBase += aPureTestName;
	aNameBase = aNameBase.replace( cPoint, cDirDelimiter );
	aNameBase += aTestStr;
	OUString aName;

	// Try to register lib by name
	aName = aNameBase;
	aName += aLibExtension;
	Reference< XSimpleRegistry > xDummyReg;
	bResult = registerImplementation( xSMgr, xDummyReg, aName, CP_LIB, sal_False );
	if( bResult )
	{
		xRet = xSMgr->createInstance( aTestImplName );
		if( xRet.is() )
			return xRet;
	}

	if( !xRet.is() )
	{
		char aMsg[256];
		OString aOTestName( OUStringToOString( aTestName, RTL_TEXTENCODING_ASCII_US ) );
		OString aOTestImplName( OUStringToOString( aTestImplName, RTL_TEXTENCODING_ASCII_US ) );
		sprintf( aMsg, "Can't instanciate test component \"%s\"\nfor service \"%s\"\n", 
			aOTestName.getStr(), aOTestImplName.getStr() );
		out( aMsg );
	}
	return xRet;
}




//===============================================================
//MultiThread

class TestThread : public Thread
{
	const TestInfo& aTestInfo;
	OUString aTestName;
	OUString aImplName;
	//Reference< XInterface > xToTest;
	sal_Int32 nId;

public:
	TestThread( const TestInfo& aTestInfo_, const OUString& aTestName_, 
		const OUString& rImplName, sal_Int32 nId_ )
			:aTestInfo( aTestInfo_ ), aTestName( aTestName_ ), 
			 aImplName( rImplName ), nId( nId_ ) {}
	virtual void SAL_CALL run();

};

static char pass[] = "Passed\n";
static char fail[] = "FAILED!\n";

void TestThread::run()
{
	char aMsg[256];
	Reference< XInterface > xTestIface;

	// Protect against other threads
	{
		MutexGuard guard( Mutex::getGlobalMutex() );

		// Create component that performs the test
		xTestIface = aTestInfo.createTestComponent( aTestName );
		if( !xTestIface.is() )
			return;

		sprintf( aMsg, "    Starting thread %ld\n", nId );
		out( aMsg );
	}

	Reference< XTest > xTest = Reference< XTest >::query( xTestIface );
	Reference< XSimpleTest > xSimpleTest = Reference< XSimpleTest >::query( xTestIface );
	if( xTest.is() )
	{
		// Perform MultiThread test
		sal_Int32 nTestHandle = 0;
		while( nTestHandle != -1 )
		{
			Reference< XInterface > xToTest = aTestInfo.createTestInstance( aImplName );
			nTestHandle = xTest->testMultiThread( aTestName, xToTest, nTestHandle );
		}
	}
	else if( xSimpleTest.is() )
	{
		// Perform invariant test
		Reference< XInterface > xToTest = aTestInfo.createTestInstance( aImplName );
		xSimpleTest->testInvariant( aTestName, xToTest );
		//sal_Bool bPassed = xTest->testPassed();
	}

	sprintf( aMsg, "    Thread %ld finished.", nId );
	out( aMsg );
}


sal_Bool doMultiThreadTestOnImpl( const TestInfo& aTestInfo,
	const OUString& aTestName, const OUString& rImplName )
{
	char aMsg[256];
	sal_Int32 nThreadCount = aTestInfo.nThreadCount;
	OString aOImplName( OUStringToOString( rImplName, RTL_TEXTENCODING_ASCII_US ) );
	sprintf( aMsg, "  Implementation: \"%s\"\n", aOImplName.getStr() );
	out( aMsg );
	sprintf( aMsg, "  Multi Threading test using %ld threads\n", nThreadCount );
	out( aMsg );

	// Instance to be tested
	//Reference< XInterface > xToTest = aTestInfo.createTestInstance( rImplName );

	// TODO: Other concept
	// Initialise result
	//aTestInfo.bMultiThreadPassed = sal_True;

	// Create and start Threads
	TestThread** pThreads = new TestThread*[ nThreadCount ];
	Sequence< Reference< XInterface > > aTestObjSeq;
	aTestObjSeq.realloc( nThreadCount );
	for( sal_Int32 i = 0 ; i < nThreadCount ; i++ )
	{
		pThreads[ i ] = new TestThread( aTestInfo, aTestName, rImplName, i+1 );
		pThreads[ i ]->create();
	}

	// Wait for threads
	for( i = 0 ; i < nThreadCount ; i++ )
	{
		pThreads[ i ]->join();
	}

	// Delete threads
	for( i = 0 ; i < nThreadCount ; i++ )
	{
		delete pThreads[ i ];
	}
	
	// TODO: Other concept
	//printf( "  Multi threading test %s\n", aTestInfo.bMultiThreadPassed ? pass : fail );
	//return aTestInfo.bMultiThreadPassed;
	return sal_True;
}

//===============================================================

sal_Bool doTestOnImpl( TestInfo aTestInfo, 
	const OUString& aTestName, const OUString& rImplName )
{
	// Create component that performs the test
	Reference< XInterface > xTestIface = aTestInfo.createTestComponent( aTestName );
	if( !xTestIface.is() )
		return sal_False;
	Reference< XTest > xTest = Reference< XTest >::query( xTestIface );
	Reference< XSimpleTest > xSimpleTest = Reference< XSimpleTest >::query( xTestIface );

	OString aOImplName( OUStringToOString( rImplName, RTL_TEXTENCODING_ASCII_US ) );
	char aMsg[256];
	sprintf( aMsg, "  Implementation: \"%s\"\n", aOImplName.getStr() );
	out( aMsg );

	// First instance for testInvariant
	Reference< XInterface > xToTest = aTestInfo.createTestInstance( rImplName );
	
	// Perform invariant test
	out( "    testInvariant() ... " );
	if( xTest.is() )
	{
		xTest->testInvariant( aTestName, xToTest );
		sal_Bool bPassed = xTest->testPassed();
		out( bPassed ? pass : fail );
	}
	else if( xSimpleTest.is() )
	{
		xSimpleTest->testInvariant( aTestName, xToTest );
	}

	// Perform full test
	out( "\n    test() ... " );
	sal_Int32 nTestHandle = 0;
	sal_Int32 nPass = 0;
	sal_Bool bPrintPasses = sal_False;
	while( nTestHandle != -1 )
	{
		xToTest = aTestInfo.createTestInstance( rImplName );
		if( xTest.is() )
		{
			nTestHandle = xTest->test( aTestName, xToTest, nTestHandle );
		}
		else if( xSimpleTest.is() )
		{
			nTestHandle = xSimpleTest->test( aTestName, xToTest, nTestHandle );
		}

		if( nTestHandle != -1 )
			bPrintPasses = sal_True;

		if( bPrintPasses )
		{
			sprintf( aMsg, "\n      Pass %ld", nPass );
			out( aMsg );
			nPass++;
		}
		if( nTestHandle == -1 )
		{
			out( "\n    " );
		}
	}
	if( !xTest.is() && xSimpleTest.is() )
	{
		sal_Bool bPassed = xSimpleTest->testPassed();
		out( bPassed ? pass : fail );
	}
	return sal_True;
}


//*** Testing a service ***
sal_Bool doTest( TestInfo aTestInfo, OUString aTestName )
{
	char aMsg[256];
	OString aOServiceName( OUStringToOString( aTestName, RTL_TEXTENCODING_ASCII_US ) );

	// Get the right index in the component's list of services
	sal_Int32 nServiceIndex = aTestInfo.getServiceIndex( aTestName );
	if( nServiceIndex == -1 )
	{
		sprintf( aMsg, "\nERROR: Service \"%s\" is not supported by the component!\n", aOServiceName.getStr() );
		errout( aMsg );
		return sal_False;
	}

	sprintf( aMsg, "> Testing service \"%s\"\n", aOServiceName.getStr() );
	out( aMsg );

	// Loop over implementations (usually only one per service)
	const Sequence<OUString>& rImplSeq = aTestInfo.aImplNamesForServices.getConstArray()[nServiceIndex];
	const OUString* pImpls = rImplSeq.getConstArray();
	sal_Int32 nImplCount = rImplSeq.getLength();
	for( sal_Int32 i = 0 ; i < nImplCount ; i++ )
	{
		OUString aImplName( pImpls[i] );
		if( aTestInfo.bMultiThread )
			doMultiThreadTestOnImpl( aTestInfo, aTestName, aImplName );
		else
			doTestOnImpl( aTestInfo, aTestName, aImplName );
	}
	return sal_True;
}



sal_Int32 runImpl( TestInfo& aTestInfo, Reference< XMultiServiceFactory > xSMgr ) throw (RuntimeException);

sal_Int32 runTest( int argc, char * argv[] )
	throw (RuntimeException)
{
	// Analyse command line
	TestInfo aTestInfo( argc, argv );
	Reference< XMultiServiceFactory > aSMgr;
	return runImpl( aTestInfo, aSMgr );
}

sal_Int32 runTest( const Sequence< OUString > & rArgs, Reference< XMultiServiceFactory > xSMgr )
	throw (RuntimeException)
{
	// Analyse command line
	TestInfo aTestInfo( rArgs );
	return runImpl( aTestInfo, xSMgr );
}

sal_Int32 runImpl( TestInfo& aTestInfo, Reference< XMultiServiceFactory > xSMgr )
	throw (RuntimeException)
{
	// Analyse test componenent using the new registry with only the test component registered in
	aTestInfo.analyseComponent();

	// No manager? Create own one assuming there is an applicat.rdb in our directory
	if( !xSMgr.is() )
	{
		// Create an empty write registry used later for createRegistryServiceFactory 
		Reference< XMultiServiceFactory > xSimpleMgr( createServiceFactory() );
		OUString aWriteRegFile( RTL_CONSTASCII_USTRINGPARAM("write.rdb") );
		Reference< XSimpleRegistry > xWriteReg = createEmptyRegistry( xSimpleMgr, aWriteRegFile );
		xWriteReg->close();

		// Create main service manager, use the new registry as write registry
		xSMgr = createRegistryServiceFactory( aWriteRegFile, OUString( RTL_CONSTASCII_USTRINGPARAM("applicat.rdb") ) );
		//Reference< XMultiServiceFactory > xSMgr( createRegistryServiceFactory( 
			//aWriteRegFile, OUString( RTL_CONSTASCII_USTRINGPARAM("applicat.rdb") ) ) );
	}

	// Register component to be tested
	Reference< XSimpleRegistry > xDummyReg;
	registerImplementation( xSMgr, xDummyReg, aTestInfo.aCmdComponentName, aTestInfo.eComponentType );

	
	// TODO: Only for test
	// register test components, should not be necessary later
	//OUString aTestComponent( RTL_CONSTASCII_USTRINGPARAM("test\\com\\sun\\star\\io\\teststm.dll") );
	//registerImplementation( xSMgr, xDummyReg, aTestComponent, CP_LIB );


	// Store service manager in test info
	aTestInfo.xSMgr = xSMgr;


	// Interface test?
	if( aTestInfo.bTestInterface )
	{
		doTest( aTestInfo, aTestInfo.aCmdTestInterfaceName );
	}
	// Service test?
	else
	{
		Sequence<OUString> aServicesToTest;

		// Create List of services to be tested
		if( aTestInfo.bTestService )
		{
			aServicesToTest.realloc( 1 );
			aServicesToTest.getArray()[0] = aTestInfo.aCmdTestServiceName;
		}
		else
		{
			aServicesToTest = aTestInfo.aSupportedServices;
		}

		// Performing tests
		const OUString* pNames = aServicesToTest.getConstArray();
		sal_Int32 nLen = aServicesToTest.getLength();

		out( "\n\n*** Testing services ***\n" );
		for( sal_Int32 i = 0 ; i < nLen ; i++ )
		{
			OUString aServiceName = pNames[i];

			doTest( aTestInfo, aServiceName );
		}
	}

	return 0;
}


void usingTestImpl()
{
	errout( "\nusing: test component [-r] [-m count] [-i name] [-s name] [-p path]\n" );
	errout( "\n" );
	errout( "Parameters:\n" );		
	errout( "component = component to be tested, e.g. a DLL or a class or jar file\n" );		
	//errout( "-r        = remote, starts the component in a different process\n" );		
	errout( "-m        = multi thread test, count = number of threads used \n" );		
	errout( "-i        = tests explicitely the interface given in name, if a test component\n" );
	errout( "            exists for it (the name has to be fully qualified: com.sun...)\n" );
	errout( "-s        = tests explicitely the service given in name, if a test component\n" );
	errout( "            exists for it (the name has to be fully qualified: com.sun...)\n" );
	errout( "-p        = defines a root path to the test components. It is used to search\n" );
	errout( "            the right test component by name, if it is not registered.\n" );
	errout( "            Example: If \"-p c:\\mytests\" is defined, the test components\n" );
	errout( "            will be searched in c:\\mytests\\com\\sun... If no path is\n" );
	errout( "            defined, the test components will be searched in .\\test\\com\\sun...\n" );
	errout( "\n" );
	errout( "With no -i or -s parameter all services supported by the component are tested.\n" );				
}	

static void errout( const char* pMsg )
{
	// Protect against other threads
	{
		MutexGuard guard( Mutex::getGlobalMutex() );
		if( b_stderr )
			fprintf( stderr, pMsg );
	}
}

static void out( const char* pMsg )
{
	// Protect against other threads
	{
		MutexGuard guard( Mutex::getGlobalMutex() );
		if( b_stdout )
			fprintf( stdout, pMsg );
	}
}


/*************************************************************************

      Source Code Control System - Update

      $Log: testimpl.cxx,v $
      Revision 1.1  2001/05/04 07:05:18  kr
      moved from grande to openoffice

      Revision 1.4  2001/03/16 16:44:08  jl
      using osl::Thread instead of vos::OThread

      Revision 1.3  2000/07/28 06:58:53  ab
      SAL_CALL added

      Revision 1.2  2000/07/23 14:51:18  pl
      unicode

      Revision 1.1  2000/05/31 09:05:33  ab
      implementation

      Revision 1.1.1.1  2000/04/18 11:54:39  ab
      Neu



*************************************************************************/
